!{\src2tex{textfont=tt}}
!!****f* ABINIT/radsintr
!! NAME
!!  radsintr
!!
!! FUNCTION
!! Computes the sine transformation of a radial function F(r) to V(q).
!! Computes integrals on generalized grid using corrected trapezoidal integration.
!! 
!! COPYRIGHT
!!  Copyright (C) 2010 ABINIT group (Camilo Espejo)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  mqgrid=number of grid points in q from 0 to qmax.
!!  qgrid(mqgrid)=q grid values (bohr**-1).
!!  radmesh <type(pawrad_type)>=data containing radial grid informations
!!  funr(radmesh%mesh_size)=F(r) on radial grid.
!!  
!!
!! OUTPUT
!!  funq(mqgrid)=\int_0^inf 4\pi\frac{\sin(2\pi r)}{2\pi r}r^2F(r)dr 
!!  yq1, yqn: d/dq (F(q)) at the ends of the interval.
!! SIDE EFFECTS
!!
!! NOTES
!!  This routine is a modified version of /src/65_psp/psp7lo.F90
!!
!! PARENTS
!!      m_xc_vdw,test_radsintr
!!
!! CHILDREN
!!      simp_gen
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine radsintr(funr,funq,mqgrid,qgrid,radmesh,yq1,yqn)  
                  
 use defs_basis
 use m_profiling
 use defs_datatypes
 use m_profiling
 use m_radmesh, only: ifromr, simp_gen

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'radsintr'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer , intent(in)  :: mqgrid
 real(dp), intent(out) :: yq1, yqn                       
 type(pawrad_type), intent(in) :: radmesh
!arrays
 real(dp), intent(in)  :: funr(radmesh%mesh_size),qgrid(mqgrid)
 real(dp), intent(out) :: funq(mqgrid)
!Local variables-------------------------------
!scalars
 integer :: iq,ir,irmax
 real(dp) :: arg,r0tor1,r1torm,rmtoin
 logical :: begin_r0
!arrays
 real(dp),allocatable :: ff(:),rzf(:)                                     
!character(len=500) :: message                   ! to be uncommented, if needed
 
! *************************************************************************
 
!DEBUG
!write (std_out,*) ' radsintr : enter'
!ENDDEBUG

!DEBUG                                           ! to be uncommented, if needed
!if(option/=1 .and. option/=2 )then
!write(message,'(a,a,a,a,a,a,i6)') ch10,&
!&  ' radsintr: BUG -',ch10,&
!&  '  The argument option should be 1 or 2,',ch10,&
!&  '  however, option=',option
!call wrtout(std_out,message,'COLL')
!call leave_new('COLL')
!endif
!if(sizein<1)then
!write(message,'(a,a,a,a,a,a,i6)') ch10,&
!&  ' radsintr: BUG -',ch10,&
!&  '  The argument sizein should be a positive number,',ch10,&
!&  '  however, sizein=',sizein
!call wrtout(std_out,message,'COLL')
!call leave_new('COLL')
!endif
!ENDDEBUG


!DEBUG
!write (std_out,*) ' radsintr : exit'
!stop
!ENDDEBUG

!************************************************************************

 irmax=ifromr(radmesh,min(20._dp,radmesh%rmax))

!Particular case of a null fuction to transform
 if (maxval(abs(funr(1:irmax)))<=1.e-20_dp) then
   funq=zero;yq1=zero;yqn=zero
   return
 end if

 ABI_ALLOCATE(ff,(radmesh%mesh_size))
 ABI_ALLOCATE(rzf,(radmesh%mesh_size))
 ff=zero
 rzf=zero
!Is mesh beginning with r=0 ?
 begin_r0=(radmesh%rad(1)<1.e-20_dp)

!Store r.F(r)
 do ir=1,irmax
   ff(ir)=radmesh%rad(ir)*funr(ir) !ff is part of the integrand   
 end do

!===========================================
!=== Compute v(q) for q=0 separately
!===========================================

!Integral from 0 to r1 (only if r1<>0)
 r0tor1=zero;if (.not.begin_r0) &
& r0tor1=(funr(1)*third)*radmesh%rad(1)**three

!Integral from r1 to rmax
 do ir=1,irmax
   if (abs(ff(ir))>1.e-20_dp) then
     rzf(ir)=radmesh%rad(ir)*ff(ir) !sin(2\pi*q*r)/(2*\pi*q*r)-->1 for q=0
   end if                          !so the integrand is 4*\pi*r^2*F(r)
 end do
 call simp_gen(r1torm,rzf,radmesh)

!Integral from rmax to infinity
!This part is neglected... might be improved.
 rmtoin=zero

!Sum of the three parts

 funq(1)=four_pi*(r0tor1+r1torm+rmtoin)

!===========================================
!=== Compute v(q) for other q''s
!===========================================

!Loop over q values
 do iq=2,mqgrid
   arg=two_pi*qgrid(iq)

!  Integral from 0 to r1 (only if r1<>0)
   r0tor1=zero;if (.not.begin_r0) &
&   r0tor1=(funr(1)/arg)*( sin(arg*radmesh%rad(1))/arg - radmesh%rad(1)*&
&   cos(arg*radmesh%rad(1)) )

!  Integral from r1 to rmax
   do ir=1,irmax
     if (abs(ff(ir))>1.e-20_dp) rzf(ir)=sin(arg*radmesh%rad(ir))*ff(ir)
   end do
   call simp_gen(r1torm,rzf,radmesh)

!  Integral from rmax to infinity
!  This part is neglected... might be improved.
   rmtoin=zero

!  Store F(q)
   funq(iq) = two/qgrid(iq)*(r0tor1+r1torm+rmtoin)
 end do


!===========================================
!=== Compute derivatives of F(q)
!=== at ends of interval
!===========================================

!yq(0)=zero
 yq1=zero

!yp(qmax)=$ 
 arg=two_pi*qgrid(mqgrid)

!Integral from 0 to r1 (only if r1<>0)
 r0tor1=zero;if (.not.begin_r0) &
& r0tor1=(funr(1)/arg)*( sin(arg*radmesh%rad(1))*(radmesh%rad(1)**2-&
& three/arg**2) + three*radmesh%rad(1)*cos(arg*radmesh%rad(1))/arg )   



!Integral from r1 to rmax
 do ir=1,irmax
   if (abs(ff(ir))>1.e-20_dp) rzf(ir)=( radmesh%rad(ir)*cos(arg*radmesh%rad(ir))-&
&   sin(arg*radmesh%rad(ir))/arg )*ff(ir)
 end do
 call simp_gen(r1torm,rzf,radmesh)

!Integral from rmax to infinity
!This part is neglected... might be improved.
 rmtoin=zero

!Sum of the three parts
 yqn=(four*pi/qgrid(mqgrid))*(r0tor1+r1torm+rmtoin)

 ABI_DEALLOCATE(ff)
 ABI_DEALLOCATE(rzf)

end subroutine radsintr
!!***
