!{\src2tex{textfont=tt}}
!!****m* ABINIT/m_fftw3
!! NAME
!! m_fftw3
!!
!! FUNCTION
!!  This module provides wrappers for the FFTW3 routines: in-place and out-of-place version.
!!
!! COPYRIGHT
!! Copyright (C) 2009-2013 ABINIT group (MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! NOTES
!!  1) MPI parallelism is in testing stage
!!  2) For better performance the FFT divisions should contain small factors  (/2, 3, 5, 7, 11/).
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

! It seems that MKL wrappers do not like the advanced interfaces for
! r2c and c2r transforms although they work fine if the true FFTW3 library is used.
#define DEV_RC_BUG
!#undef DEV_RC_BUG

#define FFTLIB "FFTW3"
#define FFT_PREF(name) CONCAT(fftw3_,name)
#define SPAWN_THREADS_HERE(ndat, nthreads) fftw3_spawn_threads_here(ndat, nthreads)

MODULE m_fftw3

 use defs_basis
 use defs_abitypes
 use m_profiling
 use m_errors
 use m_timer
 use m_xomp
 use m_blas
 use m_cgtools
 use m_cplxtools
#ifdef HAVE_FC_ISO_C_BINDING
 use iso_c_binding
#else
#error "Compiler does not provide the module iso_c_binding, cannot use m_fftw3.F90"
 !use m_iso_c_binding
#endif

 use m_numeric_tools,   only : imax_loc
 use m_fstrings,        only : strcat
 use m_fft_mesh,        only : zpad_t, zpad_init, zpad_free

 implicit none

#ifdef HAVE_FFT_FFTW3_MPI
 include 'fftw3-mpi.f03'
#endif
!This should be done but MKL fftw hasn't always this include file
! #ifdef HAVE_FFT_FFTW3
!  include 'fftw3.f03'
! #endif

 private

! Entry points for client code
 public :: fftw3_fourdp         ! 3D FFT of lengths nx, ny, nz. Mainly used for densities or potentials.
 public :: fftw3_fourwf         ! FFT transform of wavefunctions (high-level interface).
 public :: fftw3_fftrisc
 public :: fftw3_fftug          ! G-->R. 3D zero-padded FFT of lengths nx, ny, nz. Mainly used for wavefunctions
 public :: fftw3_fftur          ! R-->G, 3D zero-padded FFT of lengths nx, ny, nz. Mainly used for wavefunctions
 public :: fftw3_use_lib_threads 

! Low-level routines.
 public :: fftw3_cleanup        ! Reset FFTW to the pristine state it was in when you started your program, 
 public :: fftw3_init_threads   ! one-time initialization required to use FFTW3 threads.
 public :: fftw3_set_nthreads   ! Set the number of threads you want FFTW3 to use when HAVE_FFT_FFTW3_THREADS is defined.
 public :: fftw3_r2c_op         ! Real to complex transform (out-of-place version).
 public :: fftw3_c2r_op         ! Complex to real transform (out-of-place version).
 public :: fftw3_c2c_op         ! complex to complex transform (out-of-place version).
 public :: fftw3_c2c_ip         ! complex to complex transform (in-place version).
 public :: fftw3_many_dft_op    ! Driver routine for many out-of-place 3D complex-to-complex FFTs.
 public :: fftw3_many_dft_ip    ! Driver routine for many in-place 3D complex-to-complex FFTs.
 public :: fftw3_fftpad         ! Driver routines for zero-padded FFT of wavefunctions.
 public :: fftw3_fftpad_dp      ! Driver routines for zero-padded FFT of wavefunctions.
 public :: fftw3_fftug_dp       ! Driver routines for zero-padded FFT of wavefunctions.

 public :: fftwmpi_get_work_array
 public :: fftwmpi_free_work_array
 public :: fftwmpi_c2r_tr       ! Complex to real transform (transposed MPI version).
 public :: fftwmpi_r2c_tr       ! Real to complex transform (transposed MPI version).
 public :: fftwmpi_many_dft_ip  ! Driver routine for many in-place 3D complex-to-complex FFTs with MPI version.
 public :: fftwmpi_many_dft_tr  ! Driver routine for many transposed 3D complex-to-complex FFTs with MPI version.

#ifdef HAVE_FFT_FFTW3_MPI
! flags copied from fftw3.f
 integer,public,parameter :: ABI_FFTW_FORWARD=FFTW_FORWARD
 integer,public,parameter :: ABI_FFTW_BACKWARD=FFTW_BACKWARD
 integer,public,parameter :: ABI_FFTW_ESTIMATE=FFTW_ESTIMATE
 integer,public,parameter :: ABI_FFTW_MEASURE=FFTW_MEASURE
 ! end flags copied from fftw3.f
 integer,public,parameter :: ABI_FFTW_MPI_TRANSPOSED_IN=FFTW_MPI_TRANSPOSED_IN
 integer,public,parameter :: ABI_FFTW_MPI_TRANSPOSED_OUT=FFTW_MPI_TRANSPOSED_OUT
 ! end flags copies from fftw3-mpi.f03
#else
 integer,public,parameter :: ABI_FFTW_FORWARD=-1
 integer,public,parameter :: ABI_FFTW_BACKWARD=+1
 integer,public,parameter :: ABI_FFTW_ESTIMATE=64
 integer,public,parameter :: ABI_FFTW_MEASURE=0
! end flags copied from fftw3.f
 integer,public,parameter :: ABI_FFTW_MPI_TRANSPOSED_IN=536870912
 integer,public,parameter :: ABI_FFTW_MPI_TRANSPOSED_OUT=1073741824
! end flags copies from fftw3-mpi.f03
#endif

! ==========================================================================================
! ==== Variables introduced for the FFTW3 interface in abinit. Not belonging to fftw3.f ====
! ==========================================================================================

 integer,public,parameter :: NULL_PLAN = 0
 ! MKL wrappers might return NULL_PLAN if a particular FFTW3 feature is not available

 integer,public,parameter :: KIND_FFTW_PLAN = 8
 ! It should be at least integer*@SIZEOF_INT_P@
 ! MKL wrappers requires it to be integer*8, so do _not_ use C_INTPTR_T.

 integer,private,save :: THREADS_INITED = 0
 ! 1 if treads have been initialized. 0 otherwise.

 logical,private,save :: USE_LIB_THREADS = .FALSE.
!!***

!----------------------------------------------------------------------

!!****t* m_fftw3/fftw3_plan3_t
!! NAME
!! fftw3_plan3_t
!!
!! FUNCTION
!!  Structure storing the pointer to the FFTW plan as well as the options used to generate it.
!!
!! SOURCE

 type,private :: fftw3_plan3_t
   integer :: isign=0                           ! Sign of the exponential in the FFT
   integer :: ndat=-1                           ! Number of FFTs associated to the plan
   integer :: flags=-HUGE(0)                    ! FFTW3 flags used to construct the plan.
   integer(KIND_FFTW_PLAN) :: plan=NULL_PLAN    ! FFTW3 plan.
   integer :: nthreads=1                        ! The number of threads associated to the plan.
   integer :: idist=-1
   integer :: odist=-1
   integer :: istride=-1
   integer :: ostride=-1
   integer :: n(3)=-1                           ! The number of FFT divisions.
   integer :: inembed(3)=-1
   integer :: onembed(3)=-1
   !integer(C_INT) :: alignment(2)              ! The alignment of the arrays used to construct the plan.
 end type fftw3_plan3_t
!!***

 interface fftw3_fftrisc
   module procedure fftw3_fftrisc_sp
   module procedure fftw3_fftrisc_dp
 end interface fftw3_fftrisc

 interface fftw3_fftug
   module procedure fftw3_fftug_dp 
   module procedure fftw3_fftug_spc
   module procedure fftw3_fftug_dpc
 end interface fftw3_fftug

 interface fftw3_fftur
   module procedure fftw3_fftur_dp 
   module procedure fftw3_fftur_spc
   module procedure fftw3_fftur_dpc
 end interface fftw3_fftur

 interface fftw3_c2c_op     
   module procedure fftw3_c2c_op_spc   
   module procedure fftw3_c2c_op_dpc
 end interface fftw3_c2c_op      

 interface fftw3_c2c_ip      
   module procedure fftw3_c2c_ip_spc    
   module procedure fftw3_c2c_ip_dpc
 end interface fftw3_c2c_ip      

 interface fftw3_fftpad      
   module procedure fftw3_fftpad_dp     
   module procedure fftw3_fftpad_spc 
   module procedure fftw3_fftpad_dpc   
 end interface fftw3_fftpad      

#ifdef  HAVE_FFT_FFTW3
  ! Overloaded planner.
 interface fftw3_plan_many_dft
   module procedure dplan_many_dft
   module procedure cplan_many_dft
   module procedure zplan_many_dft
 end interface fftw3_plan_many_dft
                                   
 interface fftw3_execute_dft
   module procedure fftw3_execute_dft_dp
   module procedure fftw3_execute_dft_spc
   module procedure fftw3_execute_dft_dpc
 end interface fftw3_execute_dft

 interface fftw3_alloc_real
   !module procedure fftw3_alloc_real_sp
   module procedure fftw3_alloc_real_dp
 end interface fftw3_alloc_real

 interface fftw3_alloc_complex
   module procedure fftw3_alloc_complex_spc
   module procedure fftw3_alloc_complex_dpc
 end interface fftw3_alloc_complex

!!   FDahm :In case of ffw3-mpi flavor, one must include fftw3-mpi.F03 so 
!! the next few lines cause compiler errors because of name redefinition
!! In deed, i think this should be avoid and replace by 'include fftw3.f03'
!! MG: These bindings are needed when we use the FFTW3 wrappers provided by 
!!     the MKL library. I agree that we should use the include file fftw3.F03
!!     but this implies that
!!     1) we cannot use the procedures defined in this module to call the MKL wrappers
!!     2) we drop support for FFTW3 versions < 3.2 since these version do not provide 
!!        the F2003 interface
!!     I don't have any problem in dropping support for old versions of FFTW3
!!     We just have to handle the migration on the different slaves of the test farm
!!     builders with MKL should use m_dfti.F90
!!     builders with FFTW3 should provide a recent version of the library
!!   
#ifndef HAVE_FFT_FFTW3_MPI
 ! Fortran binding for fftw_malloc
 interface fftw_malloc
   type(C_PTR) function fftw_malloc(alloc_size) bind(C, name='fftw_malloc')
     import
     integer(C_SIZE_T), value :: alloc_size
   end function fftw_malloc
 end interface fftw_malloc
 ! Fortran binding for fftw_free
 interface fftw_free
    subroutine fftw_free(cptr) bind(C, name='fftw_free')
     import
     type(C_PTR), value :: cptr
   end subroutine fftw_free
end interface
#endif
#endif

CONTAINS  !===========================================================

!!****f* m_fftw3/fftw3_fourdp
!! NAME
!!  fftw3_fourdp
!!
!! FUNCTION
!! Driver routine for 3D FFT of lengths nx, ny, nz. Mainly used for densities or potentials.
!! FFT Transform is out-of-place
!!
!! INPUTS
!! cplex=1 if fofr is real, 2 if fofr is complex
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimension of the array.
!! ndat = Number of FFTS
!! isign= +1 : fofg(G) => fofr(R); 
!!        -1 : fofr(R) => fofg(G)
!! fofg(2,ldx*ldy*ldz*ndat)=The array to be transformed.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator. Defaults to FFTW_ESTIMATE.
!!
!! OUTPUT 
!! fofr(cplex,ldx*ldy*ldz*ndat)=The FFT of fofg
!!
!! PARENTS
!!      fourdp,m_fft
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fourdp(cplex,nx,ny,nz,ldx,ldy,ldz,ndat,isign,fofg,fofr,fftw_flags)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fourdp'
!End of the abilint section

 implicit none 

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: cplex,nx,ny,nz,ldx,ldy,ldz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
!arrays
 real(dp),intent(inout) :: fofg(2*ldx*ldy*ldz*ndat)
 real(dp),intent(inout) :: fofr(cplex*ldx*ldy*ldz*ndat)

!Local variables-------------------------------
!scalars
 integer :: my_flags

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 select case (cplex)
 case (2) ! Complex to Complex.

   select case (isign)
   case (ABI_FFTW_BACKWARD) ! +1
     call fftw3_many_dft_op(nx,ny,nz,ldx,ldy,ldz,ndat,isign,fofg,fofr,fftw_flags=my_flags)

   case (ABI_FFTW_FORWARD)  ! -1
     call fftw3_many_dft_op(nx,ny,nz,ldx,ldy,ldz,ndat,isign,fofr,fofg,fftw_flags=my_flags)

   case default
     MSG_BUG("Wrong isign")
   end select 

 case (1) ! Real case.

   select case (isign)
   case (ABI_FFTW_FORWARD) ! -1; R --> G 
     call fftw3_r2c_op(nx,ny,nz,ldx,ldy,ldz,ndat,fofr,fofg,fftw_flags=my_flags)

   case (ABI_FFTW_BACKWARD) ! +1; G --> R
     call fftw3_c2r_op(nx,ny,nz,ldx,ldy,ldz,ndat,fofg,fofr,fftw_flags=my_flags)

   case default
     MSG_BUG("Wrong isign")
   end select

 case default 
   MSG_BUG(" Wrong value for cplex")
 end select 

end subroutine fftw3_fourdp
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fourwf
!! NAME
!! fftw3_fourwf
!!
!! FUNCTION
!! Carry out composite Fourier transforms between real and reciprocal (G) space.
!! Wavefunctions, contained in a sphere in reciprocal space,
!! can be FFT to real space. They can also be FFT from real space
!! to a sphere. Also, the density maybe accumulated, and a local potential can be applied.
!!
!! The different options are :
!! - option=0 --> reciprocal to real space and output the result.
!! - option=1 --> reciprocal to real space and accumulate the density.
!! - option=2 --> reciprocal to real space, apply the local potential to the wavefunction
!!                in real space and produce the result in reciprocal space.
!! - option=3 --> real space to reciprocal space.
!!                NOTE that in this case, fftalg=1x1 MUST be used. This may be changed in the future.
!!
!! INPUTS
!! cplex= if 1 , denpot is real, if 2 , denpot is complex
!!    (cplex=2 only allowed for option=2, and istwf_k=1)
!!    not relevant if option=0 or option=3, so cplex=0 can be used to minimize memory
!! fofgin(2,npwin)=holds input wavefunction in G vector basis sphere.
!!                 (intent(in) but the routine sphere can modify it for another iflag)
!! gboundin(2*mgfft+8,2)=sphere boundary info for reciprocal to real space
!! gboundout(2*mgfft+8,2)=sphere boundary info for real to reciprocal space
!! istwf_k=option parameter that describes the storage of wfs
!! kg_kin(3,npwin)=reduced planewave coordinates, input
!! kg_kout(3,npwout)=reduced planewave coordinates, output
!! mgfft=maximum size of 1D FFTs
!! ndat=number of FFT to do in //
!! ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!! npwin=number of elements in fofgin array (for option 0, 1 and 2)
!! npwout=number of elements in fofgout array (for option 2 and 3)
!! ldx,ldy,ldz=ngfft(4),ngfft(5),ngfft(6), dimensions of fofr.
!! option= if 0: do direct FFT
!!         if 1: do direct FFT, then sum the density
!!         if 2: do direct FFT, multiply by the potential, then do reverse FFT
!!         if 3: do reverse FFT only
!! weight_r=weight to be used for the accumulation of the density in real space
!!         (needed only when option=1)
!!
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!! Input/Output
!! for option==0, fofgin(2,npwin*ndat)=holds input wavefunction in G sphere;
!!                fofr(2,ldx*ldy*ldz) contains the output Fourier Transform of fofgin;
!!                no use of denpot, fofgout and npwout.
!! for option==1, fofgin(2,npwin*ndat)=holds input wavefunction in G sphere;
!!                denpot(cplex*ldx,ldy,ldz) contains the input density at input,
!!                and the updated density at output (accumulated);
!!                no use of fofgout and npwout.
!! for option==2, fofgin(2,npwin*ndat)=holds input wavefunction in G sphere;
!!                denpot(cplex*ldx,ldy,ldz) contains the input local potential;
!!                fofgout(2,npwout*ndat) contains the output function;
!! for option==3, fofr(2,ldx*ldy*ldz*ndat) contains the input real space wavefunction;
!!                fofgout(2,npwout*ndat) contains its output Fourier transform;
!!                no use of fofgin and npwin.
!!
!! PARENTS
!!      fourwf
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fourwf(cplex,denpot,fofgin,fofgout,fofr,gboundin,gboundout,istwf_k,&
&  kg_kin,kg_kout,mgfft,ndat,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fourwf'
 use interfaces_52_fft_mpi_noabirule
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: cplex,istwf_k,ldx,ldy,ldz,ndat,npwin,npwout,option,mgfft
 real(dp),intent(in) :: weight_r
!arrays
 integer,intent(in) :: gboundin(2*mgfft+8,2),gboundout(2*mgfft+8,2)
 integer,intent(in) :: kg_kin(3,npwin),kg_kout(3,npwout),ngfft(18)
 real(dp),intent(inout) :: denpot(cplex*ldx,ldy,ldz),fofgin(2,npwin*ndat)
 real(dp),intent(inout) :: fofr(2,ldx*ldy*ldz*ndat)
 real(dp),intent(out) :: fofgout(2,npwout*ndat)

!Local variables-------------------------------
!scalars
 integer,parameter :: me_g0=1,ndat1=1
 integer :: nx,ny,nz,fftalg,fftalga,fftalgc,fftcache,dat,ptg,ptr,ptgin,ptgout,nthreads
 character(len=500) :: msg
 logical :: use_fftrisc
!arrays
 !real(dp),allocatable :: saveden(:,:,:)
#if 0
 logical :: use_fftbox
 integer,parameter :: shiftg(3)=(/0,0,0/)
 integer :: symm(3,3)
#endif

! *************************************************************************

 nx=ngfft(1); ny=ngfft(2); nz=ngfft(3)
 fftalg=ngfft(7); fftalga=fftalg/100; fftalgc=mod(fftalg,10)
 fftcache=ngfft(8)

 if (ALL(option /= (/0,1,2,3/))) then
   write(msg,'(a,i0,a)')' The option number',option,' is not allowed. Only option=0, 1, 2 or 3 are allowed presently.'
   MSG_ERROR(msg)
 end if

 if (option==1 .and. cplex/=1) then
   write(msg,'(a,i0)')' With the option number 1, cplex must be 1 but it is cplex=',cplex
   MSG_ERROR(msg)
 end if

 if (option==2 .and. (cplex/=1 .and. cplex/=2)) then
   write(msg,'(a,i0)')' With the option number 2, cplex must be 1 or 2, but it is cplex=',cplex
   MSG_ERROR(msg)
 end if

 use_fftrisc = (fftalgc==2)
 if (istwf_k==2.and.option==3) use_fftrisc = .FALSE.
 if (istwf_k>2.and.ANY(option==(/0,3/))) use_fftrisc = .FALSE.

 nthreads = xomp_get_num_threads(open_parallel=.TRUE.)

 if (use_fftrisc) then
   !call wrtout(std_out,ABI_FUNC//" calls fftw3_fftrisc","COLL")

   if (ndat==1) then
     call fftw3_fftrisc(cplex,denpot,fofgin,fofgout,fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&      mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)

   else
     ! All this boilerplate code is needed because the caller might pass zero-sized arrays 
     ! for the arguments that are not referenced and we don't want to have problems at run-time.
     ! Moreover option 1 requires a special treatment when threads are started at this level.

     SELECT CASE (option)
     CASE (0)
       !
       ! fofgin -> fofr, no use of denpot, fofgout and npwout.
       if (.not.fftw3_spawn_threads_here(ndat,nthreads)) then
         do dat=1,ndat
           ptg = 1 + (dat-1)*npwin
           ptr = 1 + (dat-1)*ldx*ldy*ldz
           call fftw3_fftrisc_dp(cplex,denpot,fofgin(1,ptg),fofgout,fofr(1,ptr),gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&            mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
         end do
       else
!$OMP PARALLEL DO PRIVATE(ptg,ptr)
         do dat=1,ndat
           ptg = 1 + (dat-1)*npwin
           ptr = 1 + (dat-1)*ldx*ldy*ldz
           call fftw3_fftrisc_dp(cplex,denpot,fofgin(1,ptg),fofgout,fofr(1,ptr),gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&            mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
         end do
       end if

     CASE (1)
       !fofgin -> local ur and accumulate density in denpot 
       ! TODO this is delicate part to do in parallel, as one should OMP reduce denpot.
       ! but this causes problems with the stack.

       do dat=1,ndat
         ptg = 1 + (dat-1)*npwin
         ptr = 1 + (dat-1)*ldx*ldy*ldz
         call fftw3_fftrisc_dp(cplex,denpot,fofgin(1,ptg),fofgout,fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&          mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
       end do

       ! This version seems not to be efficient
       !!!  !$OMP PARALLEL PRIVATE(ptg,ptr,saveden) 
       !!!         ABI_MALLOC(saveden, (ldx,ldy,ldz))
       !!!         saveden = zero
       !!!  !$OMP DO
       !!!         do dat=1,ndat
       !!!           ptg = 1 + (dat-1)*npwin
       !!!           ptr = 1 + (dat-1)*ldx*ldy*ldz
       !!!           call fftw3_fftrisc_dp(cplex,saveden,fofgin(1,ptg),fofgout,fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
       !!!  &          mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
       !!!         end do
       !!!  !$OMP END DO NOWAIT
       !!!  !$OMP CRITICAL (OMPC_addrho)
       !!!         denpot = denpot + saveden
       !!!  !$OMP END CRITICAL (OMPC_addrho)
       !!!         ABI_FREE(saveden)
       !!!  !$OMP END PARALLEL

     CASE (2)
       ! <G|vloc(r)|fofgin(r)> in fofgout 
       if (.not.fftw3_spawn_threads_here(ndat,nthreads)) then
         do dat=1,ndat
           ptgin  = 1 + (dat-1)*npwin
           ptgout = 1 + (dat-1)*npwout
           call fftw3_fftrisc_dp(cplex,denpot,fofgin(1,ptgin),fofgout(1,ptgout),fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&            mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
         end do
       else
!$OMP PARALLEL DO PRIVATE(ptgin,ptgout)
         do dat=1,ndat
           ptgin  = 1 + (dat-1)*npwin
           ptgout = 1 + (dat-1)*npwout
           call fftw3_fftrisc_dp(cplex,denpot,fofgin(1,ptgin),fofgout(1,ptgout),fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&            mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
         end do
       end if

     CASE (3)
       ! fofr -> fofgout
       if (.not.fftw3_spawn_threads_here(ndat,nthreads)) then
         do dat=1,ndat
           ptr    = 1 + (dat-1)*ldx*ldy*ldz
           ptgout = 1 + (dat-1)*npwout
           call fftw3_fftrisc_dp(cplex,denpot,fofgin,fofgout(1,ptgout),fofr(1,ptr),gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&            mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
         end do
       else
!$OMP PARALLEL DO PRIVATE(ptr,ptgout)
         do dat=1,ndat
           ptr    = 1 + (dat-1)*ldx*ldy*ldz
           ptgout = 1 + (dat-1)*npwout
           call fftw3_fftrisc_dp(cplex,denpot,fofgin,fofgout(1,ptgout),fofr(1,ptr),gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
&            mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight_r)
         end do
       end if

     CASE DEFAULT
       write(msg,'(a,i0,a)')'Option',option,' is not allowed. Only option=0, 1, 2 or 3 are allowed presently.'
       MSG_ERROR(msg)
     END SELECT

   end if

 else

#if 1
   SELECT CASE (option)
   CASE (0)
     !
     ! FFT u(g) --> u(r) 
     if (.not.fftw3_spawn_threads_here(ndat,nthreads)) then
       call fftw3_fftug_dp(fftalg,fftcache,npwin,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_kin,gboundin,fofgin,fofr)
     else
!$OMP PARALLEL DO PRIVATE(ptg, ptr)
       do dat=1,ndat
         ptg = 1 + (dat-1)*npwin
         ptr = 1 + (dat-1)*ldx*ldy*ldz
         call fftw3_fftug_dp(fftalg,fftcache,npwin,nx,ny,nz,ldx,ldy,ldz,ndat1,&
&          istwf_k,mgfft,kg_kin,gboundin,fofgin(1,ptg),fofr(1,ptr))
       end do
     end if

   CASE (1)
     ! TODO this is delicate part to do in parallel, as one should OMP reduce denpot.
     call fftw3_fftug_dp(fftalg,fftcache,npwin,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_kin,gboundin,fofgin,fofr)
     call cg_addtorho(nx,ny,nz,ldx,ldy,ldz,ndat,weight_r,fofr,denpot)

   CASE (2)

     if (.not.fftw3_spawn_threads_here(ndat,nthreads)) then
       call fftw3_fftug_dp(fftalg,fftcache,npwin,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_kin,gboundin,fofgin,fofr)
       call cg_vlocpsi(nx,ny,nz,ldx,ldy,ldz,ndat,cplex,denpot,fofr)

       !  The data for option==2 is now in fofr.
       call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,-1,gboundout)

       call cg_box2gsph(nx,ny,nz,ldx,ldy,ldz,ndat,npwout,kg_kout,fofr,fofgout)
     else

!$OMP PARALLEL DO PRIVATE(ptg, ptr)
       do dat=1,ndat
         ptg = 1 + (dat-1)*npwin
         ptr = 1 + (dat-1)*ldx*ldy*ldz
         call fftw3_fftug_dp(fftalg,fftcache,npwin,nx,ny,nz,ldx,ldy,ldz,ndat1,&
&          istwf_k,mgfft,kg_kin,gboundin,fofgin(1,ptg),fofr(1,ptr))

         call cg_vlocpsi(nx,ny,nz,ldx,ldy,ldz,ndat1,cplex,denpot,fofr(1,ptr))
                                                                                                                       
         !  The data for option==2 is now in fofr.
         call fftw3_fftpad_dp(fofr(1,ptr),nx,ny,nz,ldx,ldy,ldz,ndat1,mgfft,-1,gboundout)
                                                                                                                       
         ptg = 1 + (dat-1)*npwout
         call cg_box2gsph(nx,ny,nz,ldx,ldy,ldz,ndat1,npwout,kg_kout,fofr(1,ptr),fofgout(1,ptg))
       end do
     end if

   CASE (3) 
     !  The data for option==3 is already in fofr.
     if (.not.fftw3_spawn_threads_here(ndat,nthreads)) then
       call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,-1,gboundout)
       call cg_box2gsph(nx,ny,nz,ldx,ldy,ldz,ndat,npwout,kg_kout,fofr,fofgout)
     else
!$OMP PARALLEL DO PRIVATE(ptg, ptr)
       do dat=1,ndat
         ptg = 1 + (dat-1)*npwout
         ptr = 1 + (dat-1)*ldx*ldy*ldz
         call fftw3_fftpad_dp(fofr(1,ptr),nx,ny,nz,ldx,ldy,ldz,ndat1,mgfft,-1,gboundout)
         call cg_box2gsph(nx,ny,nz,ldx,ldy,ldz,ndat1,npwout,kg_kout,fofr(1,ptr),fofgout(1,ptg))
       end do
     end if

   CASE DEFAULT
     write(msg,'(a,i0,a)')'Option',option,' is not allowed. Only option=0, 1, 2 or 3 are allowed presently.'
     MSG_ERROR(msg)
   END SELECT


#else
   symm=0; symm(1,1)=1; symm(2,2)=1; symm(3,3)=1
   use_fftbox = .FALSE.
#ifdef HAVE_OPENMP
   use_fftbox = (ndat>1)
#endif
   !use_fftbox = .TRUE.

   SELECT CASE (option)
   CASE (0)
     !
     ! FFT u(g) --> u(r) 
     call sphere(fofgin,ndat,npwin,fofr,nx,ny,nz,ldx,ldy,ldz,kg_kin,istwf_k,1,me_g0,shiftg,symm,one)
                                                                                                     
     if (use_fftbox) then
       call fftw3_many_dft_ip(nx,ny,nz,ldx,ldy,ldz,ndat,ABI_FFTW_BACKWARD,fofr) 
     else
       call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,ABI_FFTW_BACKWARD,gboundin)
     end if

   CASE (1)
     ! TODO this is delicate part to do in parallel, as one should OMP reduce denpot.

     call sphere(fofgin,ndat,npwin,fofr,nx,ny,nz,ldx,ldy,ldz,kg_kin,istwf_k,1,me_g0,shiftg,symm,one)
                                                                                                     
     if (use_fftbox) then
       call fftw3_many_dft_ip(nx,ny,nz,ldx,ldy,ldz,ndat,ABI_FFTW_BACKWARD,fofr) 
     else
       call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,ABI_FFTW_BACKWARD,gboundin)
     end if

     call cg_addtorho(nx,ny,nz,ldx,ldy,ldz,ndat,weight_r,fofr,denpot)

   CASE (2)

     call sphere(fofgin,ndat,npwin,fofr,nx,ny,nz,ldx,ldy,ldz,kg_kin,istwf_k,1,me_g0,shiftg,symm,one)
                                                                                                     
     if (use_fftbox) then
       call fftw3_many_dft_ip(nx,ny,nz,ldx,ldy,ldz,ndat,ABI_FFTW_BACKWARD,fofr) 
     else
       call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,ABI_FFTW_BACKWARD,gboundin)
     end if

     call cg_vlocpsi(nx,ny,nz,ldx,ldy,ldz,ndat,cplex,denpot,fofr)
                                                                                             
     ! The data for option==2 is now in fofr.
     if (use_fftbox) then
       call fftw3_many_dft_ip(nx,ny,nz,ldx,ldy,ldz,ndat,ABI_FFTW_FORWARD,fofr) 
     else
       call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,ABI_FFTW_FORWARD,gboundout)
     end if
                                                                                             
     call cg_box2gsph(nx,ny,nz,ldx,ldy,ldz,ndat,npwout,kg_kout,fofr,fofgout)

   CASE (3)
     !  The data for option==3 is already in fofr.
     call fftw3_fftpad_dp(fofr,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,ABI_FFTW_FORWARD,gboundout)

     call cg_box2gsph(nx,ny,nz,ldx,ldy,ldz,ndat,npwout,kg_kout,fofr,fofgout)

   CASE DEFAULT
     write(msg,'(a,i0,a)')'Option',option,' is not allowed. Only option=0, 1, 2 or 3 are allowed presently.'
     MSG_ERROR(msg)
   END SELECT
#endif

 end if

end subroutine fftw3_fourwf
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftrisc_sp
!! NAME
!! fftw3_fftrisc_sp
!!
!! FUNCTION
!! Carry out Fourier transforms between real and reciprocal (G) space,
!! for wavefunctions, contained in a sphere in reciprocal space,
!! in both directions. Also accomplish some post-processing.
!!
!! NOTES
!! Specifically uses rather sophisticated algorithms, based on S Goedecker
!! routines, specialized for superscalar RISC architecture.
!! Zero padding : saves 7/12 execution time
!! Bi-dimensional data locality in most of the routine : cache reuse
!! For k-point (0 0 0) : takes advantage of symmetry of data.
!! Note however that no blocking is used, in both 1D z-transform
!! or subsequent 2D transform. This should be improved.
!!
!! INPUTS
!!  cplex= if 1 , denpot is real, if 2 , denpot is complex
!!     (cplex=2 only allowed for option=2 when istwf_k=1)
!!     one can also use cplex=0 if option=0 or option=3
!!  fofgin(2,npwin)=holds input wavefunction in G vector basis sphere.
!!  gboundin(2*mgfft+8,2)=sphere boundary info for reciprocal to real space
!!  gboundout(2*mgfft+8,2)=sphere boundary info for real to reciprocal space
!!  istwf_k=option parameter that describes the storage of wfs
!!  kg_kin(3,npwin)=reduced planewave coordinates, input
!!  kg_kout(3,npwout)=reduced planewave coordinates, output
!!  mgfft=maximum size of 1D FFTs
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  npwin=number of elements in fofgin array (for option 0, 1 and 2)
!!  npwout=number of elements in fofgout array (for option 2 and 3)
!!  ldx,ldy,ldz=ngfft(4),ngfft(5),ngfft(6), dimensions of fofr.
!!  option= if 0: do direct FFT
!!          if 1: do direct FFT, then sum the density
!!          if 2: do direct FFT, multiply by the potential, then do reverse FFT
!!          if 3: do reverse FFT only
!!  weight=weight to be used for the accumulation of the density in real space
!!          (needed only when option=1)
!!
!! OUTPUT
!!  (see side effects)
!!
!! OPTIONS
!!  The different options are:
!!  - reciprocal to real space and output the result (when option=0),
!!  - reciprocal to real space and accumulate the density (when option=1) or
!!  - reciprocal to real space, apply the local potential to the wavefunction
!!    in real space and produce the result in reciprocal space (when option=2)
!!  - real space to reciprocal space (when option=3).
!!  option=0 IS NOT ALLOWED when istwf_k>2
!!  option=3 IS NOT ALLOWED when istwf_k>=2
!!
!! SIDE EFFECTS
!!  for option==0, fofgin(2,npwin)=holds input wavefunction in G sphere;
!!                 fofr(2,ldx,ldy,ldz) contains the Fourier Transform of fofgin;
!!                 no use of denpot, fofgout and npwout.
!!  for option==1, fofgin(2,npwin)=holds input wavefunction in G sphere;
!!                 denpot(cplex*ldx,ldy,ldz) contains the input density at input,
!!                 and the updated density at output;
!!                 no use of fofgout and npwout.
!!  for option==2, fofgin(2,npwin)=holds input wavefunction in G sphere;
!!                 denpot(cplex*ldx,ldy,ldz) contains the input local potential;
!!                 fofgout(2,npwout) contains the output function;
!!  for option==3, fofr(2,ldx,ldy,ldz) contains the real space wavefunction;
!!                 fofgout(2,npwout) contains its Fourier transform;
!!                 no use of fofgin and npwin.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftrisc_sp(cplex,denpot,fofgin,fofgout,fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
& mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftrisc_sp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: cplex,istwf_k,mgfft,ldx,ldy,ldz,npwin,npwout,option
 real(dp),intent(in) :: weight
!arrays
 integer,intent(in) :: gboundin(2*mgfft+8,2),gboundout(2*mgfft+8,2)
 integer,intent(in) :: kg_kin(3,npwin),kg_kout(3,npwout),ngfft(18)
 real(sp),intent(in) :: fofgin(2,npwin)
 real(dp),intent(inout) :: denpot(cplex*ldx,ldy,ldz)
 real(sp),intent(inout) :: fofr(2,ldx*ldy*ldz)
 real(sp),intent(out) :: fofgout(2,npwout)

! *************************************************************************

#ifdef HAVE_FFT_FFTW3 

#undef  FFT_PRECISION 
#undef  MYKIND 
#undef  MYCZERO 
#undef  MYCMPLX
#undef  MYCONJG

#define FFT_PRECISION FFTW3_SINGLE
#define MYKIND SPC
#define MYCZERO (0._sp,0._sp)
#define MYCMPLX  CMPLX
#define MYCONJG  CONJG

#include "fftw3_fftrisc.finc"

#else 
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/cplex,gboundin(1,1),gboundout(1,1),istwf_k,kg_kin(1,1),kg_kout(1,1)/))
 ABI_UNUSED((/mgfft,ngfft(1),npwin,npwout,ldx,ldy,ldz,option/))
 ABI_UNUSED((/denpot(1,1,1),weight/))
 ABI_UNUSED((/fofgin(1,1),fofgout(1,1),fofr(1,1)/))
#endif

end subroutine fftw3_fftrisc_sp
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftrisc_dp
!! NAME
!! fftw3_fftrisc_dp
!!
!! FUNCTION
!! Carry out Fourier transforms between real and reciprocal (G) space,
!! for wavefunctions, contained in a sphere in reciprocal space,
!! in both directions. Also accomplish some post-processing.
!!
!! NOTES
!! Specifically uses rather sophisticated algorithms, based on S Goedecker
!! routines, specialized for superscalar RISC architecture.
!! Zero padding : saves 7/12 execution time
!! Bi-dimensional data locality in most of the routine : cache reuse
!! For k-point (0 0 0) : takes advantage of symmetry of data.
!! Note however that no blocking is used, in both 1D z-transform
!! or subsequent 2D transform. This should be improved.
!!
!! INPUTS
!!  cplex= if 1 , denpot is real, if 2 , denpot is complex
!!     (cplex=2 only allowed for option=2 when istwf_k=1)
!!     one can also use cplex=0 if option=0 or option=3
!!  fofgin(2,npwin)=holds input wavefunction in G vector basis sphere.
!!  gboundin(2*mgfft+8,2)=sphere boundary info for reciprocal to real space
!!  gboundout(2*mgfft+8,2)=sphere boundary info for real to reciprocal space
!!  istwf_k=option parameter that describes the storage of wfs
!!  kg_kin(3,npwin)=reduced planewave coordinates, input
!!  kg_kout(3,npwout)=reduced planewave coordinates, output
!!  mgfft=maximum size of 1D FFTs
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  npwin=number of elements in fofgin array (for option 0, 1 and 2)
!!  npwout=number of elements in fofgout array (for option 2 and 3)
!!  ldx,ldy,ldz=ngfft(4),ngfft(5),ngfft(6), dimensions of fofr.
!!  option= if 0: do direct FFT
!!          if 1: do direct FFT, then sum the density
!!          if 2: do direct FFT, multiply by the potential, then do reverse FFT
!!          if 3: do reverse FFT only
!!  weight=weight to be used for the accumulation of the density in real space
!!          (needed only when option=1)
!!
!! OUTPUT
!!  (see side effects)
!!
!! OPTIONS
!!  The different options are:
!!  - reciprocal to real space and output the result (when option=0),
!!  - reciprocal to real space and accumulate the density (when option=1) or
!!  - reciprocal to real space, apply the local potential to the wavefunction
!!    in real space and produce the result in reciprocal space (when option=2)
!!  - real space to reciprocal space (when option=3).
!!  option=0 IS NOT ALLOWED when istwf_k>2
!!  option=3 IS NOT ALLOWED when istwf_k>=2
!!
!! SIDE EFFECTS
!!  for option==0, fofgin(2,npwin)=holds input wavefunction in G sphere;
!!                 fofr(2,ldx,ldy,ldz) contains the Fourier Transform of fofgin;
!!                 no use of denpot, fofgout and npwout.
!!  for option==1, fofgin(2,npwin)=holds input wavefunction in G sphere;
!!                 denpot(cplex*ldx,ldy,ldz) contains the input density at input,
!!                 and the updated density at output;
!!                 no use of fofgout and npwout.
!!  for option==2, fofgin(2,npwin)=holds input wavefunction in G sphere;
!!                 denpot(cplex*ldx,ldy,ldz) contains the input local potential;
!!                 fofgout(2,npwout) contains the output function;
!!  for option==3, fofr(2,ldx,ldy,ldz) contains the real space wavefunction;
!!                 fofgout(2,npwout) contains its Fourier transform;
!!                 no use of fofgin and npwin.
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftrisc_dp(cplex,denpot,fofgin,fofgout,fofr,gboundin,gboundout,istwf_k,kg_kin,kg_kout,&
& mgfft,ngfft,npwin,npwout,ldx,ldy,ldz,option,weight)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftrisc_dp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: cplex,istwf_k,mgfft,ldx,ldy,ldz,npwin,npwout,option
 real(dp),intent(in) :: weight
!arrays
 integer,intent(in) :: gboundin(2*mgfft+8,2),gboundout(2*mgfft+8,2)
 integer,intent(in) :: kg_kin(3,npwin),kg_kout(3,npwout),ngfft(18)
 real(dp),intent(in) :: fofgin(2,npwin)
 real(dp),intent(inout) :: denpot(cplex*ldx,ldy,ldz),fofr(2,ldx*ldy*ldz)
 real(dp),intent(out) :: fofgout(2,npwout)

! *************************************************************************

#ifdef HAVE_FFT_FFTW3 

#undef  FFT_PRECISION 
#undef  MYKIND 
#undef  MYCZERO 
#undef  MYCMPLX
#undef  MYCONJG

#define FFT_PRECISION FFTW3_DOUBLE
#define MYKIND DPC
#define MYCZERO (0._dp,0._dp)
#define MYCMPLX  DCMPLX
#define MYCONJG  DCONJG

#include "fftw3_fftrisc.finc"

#else 
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/cplex,gboundin(1,1),gboundout(1,1),istwf_k,kg_kin(1,1),kg_kout(1,1)/))
 ABI_UNUSED((/mgfft,ngfft(1),npwin,npwout,ldx,ldy,ldz,option/))
 ABI_UNUSED((/denpot(1,1,1),fofgin(1,1),fofgout(1,1),fofr(1,1),weight/))
#endif

end subroutine fftw3_fftrisc_dp
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftug_dp
!! NAME
!! fftw3_fftug_dp
!!
!! FUNCTION
!! Compute ndat zero-padded FFTs from G to R space. 
!! Mainly used for the transform of wavefunctions.
!! TARGET: dp arrays with real and imaginary part
!!
!! INPUTS
!! fftalg=FFT algorith (see input variable)
!! fftcache=size of the cache (kB)
!! npw_k=number of plane waves for this k-point.
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimensions of the array.
!! ndat=Number of transforms
!! istwf_k=Option describing the storage of the wavefunction. 
!! mgfft=Max number of FFT divisions (used to dimension gbound)
!! kg_k(3,npw_k)=G-vectors in reduced coordinates
!! gbound(2*mgfft+8,2)=Table for padded-FFT. See sphereboundary.
!!  ug(npw_k*ndat)=wavefunctions in reciprocal space. 
!!
!! OUTPUT
!!  ur(ldx*ldy*ldz*ndat)=wavefunctions in real space. 
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftug_dp(fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k,gbound,ug,ur)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftug_dp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: fftalg,fftcache
 integer,intent(in) :: npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2),kg_k(3,npw_k)
 real(dp),target,intent(in) :: ug(2*npw_k*ndat)
 real(dp),target,intent(inout) :: ur(2*ldx*ldy*ldz*ndat)  

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dist=2
 real(dp) :: fofgout(2,0) 
 real(dp),ABI_CONTIGUOUS pointer :: real_ug(:,:),real_ur(:,:) 

! *************************************************************************

#undef TK_PREF
#define TK_PREF(name) CONCAT(cg_,name)

#include "fftug.finc"

#else
 ! Silence compiler warning
 MSG_ERROR("FFT_FFTW3 support not activated")
 ABI_UNUSED((/fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k(1,1),gbound(1,1)/))
 ABI_UNUSED((/ug(1),ur(1)/))
#endif

end subroutine fftw3_fftug_dp
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftug_spc
!! NAME
!! fftw3_fftug_spc
!!
!! FUNCTION
!! Compute ndat zero-padded FFTs from G-->R. 
!! Mainly used for the transform of wavefunctions.
!! TARGET: spc arrays
!!
!! INPUTS
!! fftalg=FFT algorith (see input variable)
!! fftcache=size of the cache (kB)
!! npw_k=number of plane waves for this k-point.
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimensions of the array.
!! ndat=Number of transforms
!! istwf_k=Option describing the storage of the wavefunction. 
!! mgfft=Max number of FFT divisions (used to dimension gbound)
!! kg_k(3,npw_k)=G-vectors in reduced coordinates
!! gbound(2*mgfft+8,2)=Table for padded-FFT. See sphereboundary.
!!  ug(npw_k*ndat)=wavefunctions in reciprocal space.
!!
!! OUTPUT
!!  ur(ldx*ldy*ldz*ndat)=wavefunctions in real space
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftug_spc(fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k,gbound,ug,ur)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftug_spc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: fftalg,fftcache
 integer,intent(in) :: npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2),kg_k(3,npw_k)
 complex(spc),target,intent(in) :: ug(npw_k*ndat)
 complex(spc),target,intent(inout) :: ur(ldx*ldy*ldz*ndat)  

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dist=1
!arrays
 real(sp) :: fofgout(2,0) 
 real(sp),ABI_CONTIGUOUS pointer :: real_ug(:,:),real_ur(:,:) 

! *************************************************************************

#undef TK_PREF
#define TK_PREF(name) CONCAT(cplx_,name)

#include "fftug.finc"

#else
 ! Silence compiler warning
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k(1,1),gbound(1,1)/))
 ABI_UNUSED((/ug(1),ur(1)/))
#endif

end subroutine fftw3_fftug_spc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftug_dpc
!! NAME
!! fftw3_fftug_dpc
!!
!! FUNCTION
!! Compute ndat zero-padded FFTs. 
!! Mainly used for the transform of wavefunctions.
!! TARGET: DPC arrays
!!
!! INPUTS
!! fftalg=FFT algorith (see input variable)
!! fftcache=size of the cache (kB)
!! npw_k=number of plane waves for this k-point.
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimensions of the array.
!! ndat=Number of transforms
!! istwf_k=Option describing the storage of the wavefunction. 
!! mgfft=Max number of FFT divisions (used to dimension gbound)
!! kg_k(3,npw_k)=G-vectors in reduced coordinates
!! gbound(2*mgfft+8,2)=Table for padded-FFT. See sphereboundary.
!!  ug(npw_k*ndat)=wavefunctions in reciprocal space
!!
!! OUTPUT
!!  ur(ldx*ldy*ldz*ndat)=wavefunctions in real space.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftug_dpc(fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k,gbound,ug,ur)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftug_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: fftalg,fftcache
 integer,intent(in) :: npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2),kg_k(3,npw_k)
 complex(dpc),target,intent(in) :: ug(npw_k*ndat)
 complex(dpc),target,intent(inout) :: ur(ldx*ldy*ldz*ndat)  

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dist=1
!arrays
 real(dp) :: fofgout(2,0) 
 real(dp),ABI_CONTIGUOUS pointer :: real_ug(:,:),real_ur(:,:) 

! *************************************************************************

#undef TK_PREF
#define TK_PREF(name) CONCAT(cplx_,name)

#include "fftug.finc"

#else
 ! Silence compiler warning
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k(1,1),gbound(1,1)/))
 ABI_UNUSED((/ug(1),ur(1)/))
#endif

end subroutine fftw3_fftug_dpc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftur_dp
!! NAME
!! fftw3_fftur_dp
!!
!! FUNCTION
!! Compute ndat zero-padded FFTs from R- to G-space . 
!! Mainly used for the transform of wavefunctions.
!! TARGET: dp arrays
!!
!! INPUTS
!! fftalg=FFT algorith (see input variable)
!! fftcache=size of the cache (kB)
!! npw_k=number of plane waves for this k-point.
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimensions of the array.
!! ndat=Number of transforms
!! istwf_k=Option describing the storage of the wavefunction. 
!! mgfft=Max number of FFT divisions (used to dimension gbound)
!! kg_k(3,npw_k)=G-vectors in reduced coordinates
!! gbound(2*mgfft+8,2)=Table for padded-FFT. See sphereboundary.
!!
!! SIDE EFFECT
!! ur(ldx*ldy*ldz*ndat)= In input: wavefunctions in real space. 
!!                       Destroyed in output. Do not use ur anymore!
!! OUTPUT
!! ug(npw_k*ndat)=wavefunctions in reciprocal space. 
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftur_dp(fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k,gbound,ur,ug)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftur_dp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: fftalg,fftcache
 integer,intent(in) :: npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2),kg_k(3,npw_k)
 real(dp),target,intent(inout) :: ur(2*ldx*ldy*ldz*ndat)  
 real(dp),target,intent(out) :: ug(2*npw_k*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dist=2
!arrays
 real(dp) :: dum_ugin(2,0)
 real(dp),ABI_CONTIGUOUS pointer :: real_ug(:,:),real_ur(:,:) 

! *************************************************************************

#undef TK_PREF
#define TK_PREF(name) CONCAT(cg_,name)

#include "fftur.finc"

#else
 ! Silence compiler warning
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/fftalg,fftcache/))
 ABI_UNUSED((/npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k(1,1),gbound(1,1)/))
 ABI_UNUSED((/ug(1),ur(1)/))
#endif

end subroutine fftw3_fftur_dp
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftur_spc
!! NAME
!! fftw3_fftur_spc
!!
!! FUNCTION
!! Compute ndat zero-padded FFTs from R- to G-space . 
!! Mainly used for the transform of wavefunctions.
!! TARGET: spc arrays
!!
!! INPUTS
!! fftalg=FFT algorith (see input variable)
!! fftcache=size of the cache (kB)
!! npw_k=number of plane waves for this k-point.
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimensions of the array.
!! ndat=Number of transforms
!! istwf_k=Option describing the storage of the wavefunction. 
!! mgfft=Max number of FFT divisions (used to dimension gbound)
!! kg_k(3,npw_k)=G-vectors in reduced coordinates
!! gbound(2*mgfft+8,2)=Table for padded-FFT. See sphereboundary.
!!
!! SIDE EFFECT
!! ur(ldx*ldy*ldz*ndat)= In input: wavefunctions in real space. 
!!                       Destroyed in output. Do not use ur anymore!
!! OUTPUT
!! ug(npw_k*ndat)=wavefunctions in reciprocal space. 
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftur_spc(fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k,gbound,ur,ug)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftur_spc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: fftalg,fftcache
 integer,intent(in) :: npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2),kg_k(3,npw_k)
 complex(spc),target,intent(inout) :: ur(ldx*ldy*ldz*ndat)  
 complex(spc),target,intent(out) :: ug(npw_k*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dist=1
!arrays
 real(sp) :: dum_ugin(2,0)
 real(sp),ABI_CONTIGUOUS pointer :: real_ug(:,:),real_ur(:,:) 

! *************************************************************************

#undef TK_PREF
#define TK_PREF(name) CONCAT(cplx_,name)

#include "fftur.finc"

#else
 ! Silence compiler warning
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/fftalg,fftcache/))
 ABI_UNUSED((/npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k(1,1),gbound(1,1)/))
 ABI_UNUSED((/ug(1),ur(1)/))
#endif

end subroutine fftw3_fftur_spc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftur_dpc
!! NAME
!! fftw3_fftur_dpc
!!
!! FUNCTION
!! Compute ndat zero-padded FFTs from R ro G.
!! Mainly used for the transform of wavefunctions.
!! TARGET: DPC arrays
!!
!! INPUTS
!! fftalg=FFT algorith (see input variable)
!! fftcache=size of the cache (kB)
!! npw_k=number of plane waves for this k-point.
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Leading dimensions of the array.
!! ndat=Number of transforms
!! istwf_k=Option describing the storage of the wavefunction. 
!! mgfft=Max number of FFT divisions (used to dimension gbound)
!! kg_k(3,npw_k)=G-vectors in reduced coordinates
!! gbound(2*mgfft+8,2)=Table for padded-FFT. See sphereboundary.
!!
!! SIDE EFFECT
!! ur(ldx*ldy*ldz*ndat)= In input: wavefunctions in real space. 
!!                       Destroyed in output. Do not use ur anymore!
!! OUTPUT
!! ug(npw_k*ndat)=wavefunctions in reciprocal space
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftur_dpc(fftalg,fftcache,npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k,gbound,ur,ug)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftur_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: fftalg,fftcache
 integer,intent(in) :: npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2),kg_k(3,npw_k)
 complex(dpc),target,intent(inout) :: ur(ldx*ldy*ldz*ndat)  
 complex(dpc),target,intent(out) :: ug(npw_k*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dist=1
!arrays
 real(dp) :: dum_ugin(2,0)
 real(dp),ABI_CONTIGUOUS pointer :: real_ug(:,:),real_ur(:,:) 

! *************************************************************************

#undef TK_PREF
#define TK_PREF(name) CONCAT(cplx_,name)

#include "fftur.finc"

#else
 ! Silence compiler warning
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/fftalg,fftcache/))
 ABI_UNUSED((/npw_k,nx,ny,nz,ldx,ldy,ldz,ndat,istwf_k,mgfft,kg_k(1,1),gbound(1,1)/))
 ABI_UNUSED((/ug(1),ur(1)/))
#endif

end subroutine fftw3_fftur_dpc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_c2c_ip_spc
!! NAME
!!  fftw3_c2c_ip_spc
!!
!! FUNCTION
!! Driver routine for in-place 3D complex-complex FFT.
!! TARGET: Simple precision complex arrays.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimensions of the array.
!! ndat=Number of FFTs to be done.
!! isign= +1 : ff(G) => ff(R); -1 : ff(R) => ff(G)
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator. 
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! SIDE EFFECTS
!!  ff(ldx*ldy*ldz*ndat)=
!!    In input: the complex array to be transformed.
!!    In output: the Fourier transformed in the space specified by isign.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_c2c_ip_spc(nx,ny,nz,ldx,ldy,ldz,ndat,isign,ff,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_c2c_ip_spc'
!End of the abilint section

 implicit none 

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
!arrays
 complex(spc),intent(inout) :: ff(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: my_flags,dist,ii,stride
 integer(KIND_FFTW_PLAN) :: my_plan 
!arrays
 integer :: embed(rank),n(rank)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags=fftw_flags

 stride = 1
 dist   = ldx*ldy*ldz
 embed  = (/ldx,ldy,ldz/)
 n      = (/nx ,ny ,nz /) 

 my_plan = fftw3_plan_many_dft(rank, n, ndat, ff, embed, stride, dist, ff, embed, stride, dist, isign, my_flags, nt_all)

 ! Now perform the 3D FFT via FFTW.
 call sfftw_execute_dft(my_plan, ff, ff)

 call fftw3_destroy_plan(my_plan)

 if (isign==ABI_FFTW_FORWARD) then ! -1, FFTW returns not normalized FTs
   call xscal(ldx*ldy*ldz*ndat, REAL(one/(nx*ny*nz),KIND=sp), ff, 1) 
 end if

#else 
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,isign/))
 ABI_UNUSED(ff)
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftw3_c2c_ip_spc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftpad_spc
!! NAME
!!  fftw3_fftpad_spc
!!
!! FUNCTION
!!  This routine transforms wavefunctions using 3D zero-padded FFTs with FFTW3. 
!!  The 3D ffts are computed only on lines and planes which have non zero elements. 
!!  These lines and planes are defined by the two vectors do_fft_x(ldy*nz) and do_fft_y(nz) 
!!  FFT transform is in-place. Target: complex arrays.
!!  
!! INPUTS
!!   nx,ny,nz=Logical dimensions of the FFT mesh.
!!   ldx,ldy,ldz=Physical dimension of the f array (to avoid cache conflicts).
!!   ndat=Number of FFT transforms.
!!   mgfft=MAX(nx,ny,nz), only used to dimension gbound.
!!   isign=The sign of the transform.
!!   gbound(2*mgfft+8,2)= The boundaries of the basis sphere of G vectors at a given k-point.
!!     See sphereboundary for more info.
!!
!! SIDE EFFECTS
!!  ff(ldx*ldy*ldz*ndat)=
!!    input: The array with the data to be transformed.
!!    output: The results of the FFT.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftpad_spc(ff,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign,gbound)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftpad_spc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2)
 complex(spc),intent(inout) :: ff(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
 integer,parameter :: dst=1
 real(sp) :: fact

! *************************************************************************

#include "fftw3_fftpad.finc"

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign/))
 ABI_UNUSED(gbound(1,1))
 ABI_UNUSED(ff(1))
#endif

end subroutine fftw3_fftpad_spc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_c2c_ip_dpc
!! NAME
!!  fftw3_c2c_ip_dpc
!!
!! FUNCTION
!! Driver routine for in-place 3D complex-complex FFT.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimensions of the array.
!! ndat=Number of FFTs to be done.
!! isign= +1 : ff(G) => ff(R); -1 : ff(R) => ff(G)
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! SIDE EFFECTS
!!  ff(ldx*ldy*ldz*ndat)=
!!    In input: the complex array to be transformed.
!!    In output: the Fourier transformed in the space specified by isign.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_c2c_ip_dpc(nx,ny,nz,ldx,ldy,ldz,ndat,isign,ff,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_c2c_ip_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
!arrays
 complex(dpc),intent(inout) :: ff(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: my_flags,dist,ii,stride
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: embed(rank),n(rank)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags=fftw_flags

 stride = 1
 dist   = ldx*ldy*ldz
 embed  = (/ldx,ldy,ldz/)
 n      = (/nx ,ny ,nz /)

 my_plan = fftw3_plan_many_dft(rank, n, ndat, ff, embed, stride, dist, ff, embed, stride, dist, isign, my_flags, nt_all)

 ! Now perform the 3D FFT via FFTW.
 call dfftw_execute_dft(my_plan, ff, ff)

 call fftw3_destroy_plan(my_plan)

 if (isign==ABI_FFTW_FORWARD) then ! -1, FFTW returns not normalized FTs
  call ZDSCAL(ldx*ldy*ldz*ndat, one/(nx*ny*nz), ff, 1)
 end if

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,isign/))
 ABI_UNUSED(ff)
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftw3_c2c_ip_dpc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_c2c_op_spc
!! NAME
!!  fftw3_c2c_op_spc
!!
!! FUNCTION
!! Driver routine for out-of-place 3D complex-complex FFT of lengths nx, ny, nz.
!! TARGET: single precision complex arrays
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimensions of the array.
!! ndat=Number of FFTs to be done.
!! isign= +1 : ff(G) => gg(R); -1 : ff(R) => gg(G)
!! ff(ldx*ldy*ldz*ndat)=The array to be transformed.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator. 
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT 
!! gg(ldx*ldy*ldz*ndat)=The FFT of ff.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_c2c_op_spc(nx,ny,nz,ldx,ldy,ldz,ndat,isign,ff,gg,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_c2c_op_spc'
!End of the abilint section

 implicit none 

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,isign,ndat
 integer,optional,intent(in) :: fftw_flags
!arrays
 complex(spc),intent(in) :: ff(ldx*ldy*ldz*ndat)
 complex(spc),intent(out) :: gg(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3 
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: my_flags,dist,ii,stride
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: embed(rank),n(rank)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 stride = 1
 dist   = ldx*ldy*ldz
 embed  = (/ldx,ldy,ldz/) 
 n      = (/nx ,ny ,nz/) 

 my_plan = fftw3_plan_many_dft(rank, n, ndat, ff, embed, stride, dist, gg, embed, stride, dist, isign, my_flags, nt_all)

 ! Now perform the 3D FFT via FFTW.
 call sfftw_execute_dft(my_plan, ff, gg)

 call fftw3_destroy_plan(my_plan)

 if (isign==ABI_FFTW_FORWARD) then ! -1, FFTW returns not normalized FTs
   call xscal(ldx*ldy*ldz*ndat, REAL(one/(nx*ny*nz), KIND=sp), gg, 1)  
 end if

#else 
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,isign/))
 ABI_UNUSED(ff)
 ABI_UNUSED(gg)
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftw3_c2c_op_spc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_c2c_op_dpc
!! NAME
!!  fftw3_c2c_op_dpc
!!
!! FUNCTION
!! Driver routine for out-of-place 3D complex-complex FFT of lengths nx, ny, nz.
!! TARGET: single precision complex arrays
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimensions of the array.
!! ndat=Number of FFTs to be done.
!! isign= +1 : ff(G) => gg(R); -1 : ff(R) => gg(G)
!! ff(ldx*ldy*ldz*ndat)=The array to be transformed.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! gg(ldx*ldy*ldz*ndat)=The FFT of ff.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_c2c_op_dpc(nx,ny,nz,ldx,ldy,ldz,ndat,isign,ff,gg,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_c2c_op_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,isign,ndat
 integer,optional,intent(in) :: fftw_flags
!arrays
 complex(dpc),intent(in) :: ff(ldx*ldy*ldz*ndat)
 complex(dpc),intent(out) :: gg(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: my_flags,dist,ii,stride
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: embed(rank),n(rank)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 stride = 1
 dist   = ldx*ldy*ldz
 embed  = (/ldx,ldy,ldz/)
 n      = (/nx ,ny ,nz/)

 my_plan = fftw3_plan_many_dft(rank, n, ndat, ff, embed, stride, dist, gg, embed, stride, dist, isign, my_flags, nt_all)

 ! Now perform the 3D FFT via FFTW.
 call dfftw_execute_dft(my_plan, ff, gg)

 call fftw3_destroy_plan(my_plan)

 if (isign==ABI_FFTW_FORWARD) then ! -1, FFTW returns not normalized FTs
   call xscal(ldx*ldy*ldz*ndat, one/(nx*ny*nz), gg, 1)  
 end if

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,isign/))
 ABI_UNUSED(ff)
 ABI_UNUSED(gg)
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftw3_c2c_op_dpc
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_r2c_op
!! NAME
!!  fftw3_r2c_op
!!
!! FUNCTION
!! Driver routine for out-of-place 3D real-to-complex FFT of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimensions of the f array (to avoid cache conflicts).
!! ff(ldx*ldy*ldz*ndat)=The real array to be transformed.
!! ndat=Number of FFTs to be done.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! gg(2,nx*ny*nz*ndat)=The forward FFT of ff.
!!
!! NOTES
!!  FIXME For the time-being. No augmentation of the mesh to reduce memory conflicts, as MKL crashes
!!  if the advanced interface is used.
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_r2c_op(nx,ny,nz,ldx,ldy,ldz,ndat,ff,gg,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_r2c_op'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat
 integer,optional,intent(in) :: fftw_flags
!arrays
 real(dp),intent(in) :: ff(ldx*ldy*ldz*ndat)
 real(dp),intent(out) :: gg(2,ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: nhp,my_flags,idist,odist,padx,i1,i2,i3,igp,igf,imgf,ii,stride
 integer :: i1inv,i2inv,i3inv,idat,padatf
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: inembed(rank),onembed(rank),n(rank)
 integer,allocatable :: i1inver(:),i2inver(:),i3inver(:)
 real(dp),allocatable :: gg_hp(:,:),gg_test(:,:)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 idist = ldx*ldy*ldz
 nhp = (nx/2+1)*ny*nz
 odist = nhp

 stride = 1
 n      = (/nx,ny,nz/)
 inembed= (/ldx,ldy,ldz/)
 onembed= (/(nx/2+1),ny,nz/)

 ABI_MALLOC(gg_hp,(2,nhp*ndat))

#ifdef DEV_RC_BUG
 if (ndat/=1) MSG_ERROR("ndat/=1 + MKL not coded")

 if (ANY( (/nx,ny,nz/) /= (/ldx,ldy,ldz/) )) then
   MSG_ERROR("Augmentation not supported")
 end if

 call dfftw_plan_dft_r2c_3d(my_plan, nx, ny, nz, ff, gg_hp, my_flags)
 if (my_plan==NULL_PLAN) then
   MSG_ERROR("dfftw_plan_dft_r2c_3d returned NULL_PLAN")
 end if

 !fftw_plan fftw_plan_many_dft_r2c(int rank, const int *n, int howmany,
 !  double *in, const int *inembed, int istride, int idist,
 !  fftw_complex *out, const int *onembed, int ostride, int odist, unsigned flags);
#else
 my_plan = dplan_many_dft_r2c(rank, n, ndat, ff, inembed, stride, idist, gg_hp, onembed, stride, odist, my_flags, nt_all)
#endif

 ! Now perform the 3D FFT via FFTW. r2c are always ABI_FFTW_FORWARD
 call dfftw_execute_dft_r2c(my_plan, ff, gg_hp)

 call fftw3_destroy_plan(my_plan)

 call ZDSCAL(nhp*ndat, one/(nx*ny*nz), gg_hp, 1)  ! FFTW returns not normalized FTs
 ! Reconstruct full FFT: Hermitian redundancy: out[i] is the conjugate of out[n-i]
 padx = (nx/2+1)

 ABI_MALLOC(i1inver,(padx))
 ABI_MALLOC(i2inver,(ny))
 ABI_MALLOC(i3inver,(nz))

 i1inver(1)=1
 do i1=2,padx
   i1inver(i1)=nx+2-i1
 end do

 i2inver(1)=1
 do i2=2,ny
   i2inver(i2)=ny+2-i2
 end do

 i3inver(1)=1
 do i3=2,nz
   i3inver(i3)=nz+2-i3
 end do

 igp=0
 do idat=1,ndat
   padatf = (idat-1)*ldx*ldy*ldz
   do i3=1,nz
     i3inv = i3inver(i3)
     do i2=1,ny
       i2inv = i2inver(i2)
       do i1=1,padx
         igp = igp+1
         igf = i1 + (i3-1)*ldx*ldy + (i2-1)*ldx + padatf
         gg(:,igf) =  gg_hp(:,igp)
         i1inv = i1inver(i1)
         if (i1inv/=i1) then
           imgf = i1inv + (i3inv-1)*ldx*ldy + (i2inv-1)*ldx + padatf
           gg(1,imgf) =  gg_hp(1,igp)
           gg(2,imgf) = -gg_hp(2,igp)
         end if
       end do
     end do
   end do
 end do

 ABI_FREE(i1inver)
 ABI_FREE(i2inver)
 ABI_FREE(i3inver)

 ABI_FREE(gg_hp)

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz/))
 ABI_UNUSED(ff)
 ABI_UNUSED(gg(1,1))
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftw3_r2c_op
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_c2r_op
!! NAME
!!  fftw3_c2r_op
!!
!! FUNCTION
!! Driver routine for out-of-place 3D complex-to-real FFT of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Physical dimension of the f array (to avoid cache conflicts).
!! ndat=Number of FFTs to be done.
!! ff(2*ldx*ldy*ldz*ndat)=The complex array to be transformed.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! gg(ldx*ldy*ldz*ndat)=The backwards real FFT of ff.
!!
!! NOTES
!!  FIXME For the time-being. No augmentation of the mesh to reduce memory conflicts, as MKL crashes
!!  if the advanced interface is used.
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_c2r_op(nx,ny,nz,ldx,ldy,ldz,ndat,ff,gg,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_c2r_op'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat
 integer,optional,intent(in) :: fftw_flags
!arrays
 real(dp),intent(in) :: ff(2,ldx*ldy*ldz*ndat)
 real(dp),intent(out) :: gg(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: nhp,my_flags,padx,i2,i3,igp,igf,idat,padatf,padatp,ii,idist,odist,stride
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: inembed(rank),onembed(rank),n(rank)
 real(dp),allocatable :: ff_hp(:,:)

! *************************************************************************

#ifdef DEV_RC_BUG
 if (ANY( (/nx,ny,nz/) /= (/ldx,ldy,ldz/) )) then
   MSG_ERROR("Augmentation not supported")
 end if
#endif

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 stride  = 1
 nhp     = (nx/2+1)*ny*nz
 idist   = nhp
 odist   = ldx*ldy*ldz
 n       = (/nx,ny,nz/)
 inembed = (/(nx/2+1),ny,nz/)
 onembed = (/ldx,ldy,ldz/)

 ! Fill the Hermitian part: Hermitian redundancy: out[i] is the conjugate of out[n-i]
 ABI_MALLOC(ff_hp,(2,nhp*ndat))

 padx = (nx/2+1)
 do idat=1,ndat
   padatf=(idat-1)*ldx*ldy*ldz
   padatp=(idat-1)*padx*ny*nz
!$OMP PARALLEL DO PRIVATE(igf,igp)
   do i3=1,nz
     do i2=1,ny
       igf = (i3-1)*ldx*ldy + (i2-1)*ldx   + padatf
       igp = (i3-1)*padx*ny + (i2-1)*padx  + padatp
       ff_hp(:,igp+1:igp+padx) = ff(:,igf+1:igf+padx)
     end do
   end do
 end do

 ! NOTE: The c2r transform destroys its input array even for out-of-place transforms.
#ifdef DEV_RC_BUG
 if (ndat/=1) MSG_ERROR("ndat/=1 + MKL not coded")
 call dfftw_plan_dft_c2r_3d(my_plan, nx, ny, nz, ff_hp, gg, my_flags)
 if (my_plan==NULL_PLAN) then
   MSG_ERROR("dfftw_plan_dft_c2r_3d returned NULL_PLAN")
 end if
#else
 my_plan = dplan_many_dft_c2r(rank, n, ndat, ff_hp, inembed, stride, idist, gg, onembed, stride, odist, my_flags, nt_all)
#endif

 ! Now perform the 3D FFT via FFTW. c2r are always ABI_FFTW_BACKWARD
 call dfftw_execute_dft_c2r(my_plan, ff_hp, gg)

 call fftw3_destroy_plan(my_plan)

 ABI_FREE(ff_hp)

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz/))
 ABI_UNUSED(ff(1,1))
 ABI_UNUSED(gg(1))
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftw3_c2r_op
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_many_dft_op
!! NAME
!!  fftw3_many_dft_op
!!
!! FUNCTION
!! Driver routine for many out-of-place 3D complex-to-complex FFTs of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimension of the fin and fout arrays (to avoid cache conflicts).
!! ndat=Number of FFTs to be done.
!! fin(2*ldx*ldy*ldz*ndat)=The complex array to be transformed.
!! isign=sign of Fourier transform exponent: current convention uses
!!   +1 for transforming from G to r,
!!   -1 for transforming from r to G.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! fout(2,ldx*ldy*ldz*ndat)=The Fourier transform of fin.
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_many_dft_op(nx,ny,nz,ldx,ldy,ldz,ndat,isign,fin,fout,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_many_dft_op'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
!arrays
 real(dp),intent(in) :: fin(2*ldx*ldy*ldz*ndat)
 real(dp),intent(out) :: fout(2*ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: my_flags,dist,ii,stride
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: embed(rank),n(rank)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 stride = 1
 dist   = ldx*ldy*ldz
 embed  = (/ldx,ldy,ldz/)
 n      = (/nx ,ny ,nz /)

 my_plan = fftw3_plan_many_dft(rank, n, ndat, fin, embed, stride, dist, fout, embed, stride, dist, isign, my_flags, nt_all)

 ! Now perform the 3D FFT via FFTW.
 call dfftw_execute_dft(my_plan, fin, fout)

 call fftw3_destroy_plan(my_plan)

 if (isign==ABI_FFTW_FORWARD) then ! -1, FFTW returns not normalized FTs
  call ZDSCAL(ldx*ldy*ldz*ndat, one/(nx*ny*nz), fout, 1) 
  !call cg_zscal(ldx*ldy*ldz*ndat, (/one/(nx*ny*nz), zero/), fout) 
 end if

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,ndat,isign/))
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
 ABI_UNUSED(fin(1))
 ABI_UNUSED(fout(1))
#endif

end subroutine fftw3_many_dft_op
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_many_dft_ip
!! NAME
!!  fftw3_many_dft_ip
!!
!! FUNCTION
!! Driver routine for many in-place 3D complex-to-complex FFTs of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimension of the finout array (to avoid cache conflicts).
!! ndat=Number of FFTs to be done.
!! isign=sign of Fourier transform exponent: current convention uses
!!   +1 for transforming from G to r,
!!   -1 for transforming from r to G.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! finout(2,ldx*ldy*ldz*ndat)=
!!   In input: The complex array to be transformed.
!!   In output: The FFT results.
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_many_dft_ip(nx,ny,nz,ldx,ldy,ldz,ndat,isign,finout,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_many_dft_ip'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
!arrays
 real(dp),intent(inout) :: finout(2*ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3,nt_all=-1
 integer :: my_flags,dist,ii,stride
 integer(KIND_FFTW_PLAN) :: my_plan
!arrays
 integer :: embed(rank),n(rank)

! *************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 stride = 1
 dist   = ldx*ldy*ldz
 embed  = (/ldx,ldy,ldz/)
 n      = (/nx ,ny ,nz /)

 my_plan = fftw3_plan_many_dft(rank, n, ndat, finout, embed, stride, dist, finout,embed, stride, dist, isign, my_flags, nt_all)

 ! Now perform the 3D FFT via FFTW.
 call dfftw_execute_dft(my_plan, finout, finout)

 call fftw3_destroy_plan(my_plan)

 if (isign==ABI_FFTW_FORWARD) then ! -1, FFTW returns not normalized FTs
  call ZDSCAL(ldx*ldy*ldz*ndat, one/(nx*ny*nz), finout, 1) 
  !call cg_zscal(ldx*ldy*ldz*ndat, (/one/(nx*ny*nz),zero/), finout) 
 end if

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,ndat,isign/))
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
 ABI_UNUSED(finout(1))
#endif

end subroutine fftw3_many_dft_ip
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_cleanup
!! NAME
!!  fftw3_cleanup
!!
!! FUNCTION
!!  Reset FFTW to the pristine state it was in when you started your program,
!!  All existing plans become undefined.
!!
!! NOTES
!!  FFTW planner saves some other persistent data, such as the accumulated wisdom and a list of
!!  algorithms available in the current configuration. If you want to deallocate all of that and reset
!!  FFTW to the pristine state it was in when you started your program, you can call fftw3_cleanup();
!!  After calling fftw3_cleanup, all existing plans become undefined, and you should not attempt to
!!  execute them nor to destroy them. You can however create and execute/destroy new plans, in which case
!!  FFTW starts accumulating wisdom information again.
!!  fftw3_cleanup does not deallocate your plans, however. To prevent memory leaks, you must still call
!!  fftw_destroy_plan before executing fftw3_cleanup
!!
!! PARENTS
!!      driver
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_cleanup()


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_cleanup'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_MPI
 call fftw_mpi_cleanup()
#endif
#ifdef HAVE_FFT_FFTW3_THREADS
 if (THREADS_INITED==1) then
   call dfftw_cleanup_threads()
   THREADS_INITED = 0
 end if
#elif defined HAVE_FFT_FFTW3
 call dfftw_cleanup()
#else
 MSG_ERROR("FFTW3 support not activated")
#endif

end subroutine fftw3_cleanup
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_destroy_plan
!! NAME
!!  fftw3_destroy_plan
!!
!! FUNCTION
!!  Release the memory allocate for the plan.
!!
!! INPUTS
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_destroy_plan(plan)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_destroy_plan'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer(KIND_FFTW_PLAN),intent(in) :: plan

! *************************************************************************

#ifdef HAVE_FFT_FFTW3
!$OMP CRITICAL (OMPC_fftw3_destroy_plan)
  call dfftw_destroy_plan(plan)
!$OMP END CRITICAL (OMPC_fftw3_destroy_plan)

#else
 if (.FALSE.) write(std_out,*)plan
#endif

end subroutine fftw3_destroy_plan
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_init_threads
!! NAME
!!  fftw3_init_threads
!!
!! FUNCTION
!!  This function performs the one-time initialization required to use FFTW3 threads.
!!  It does nothing if HAVE_FFT_FFTW3_THREADS is not defined.
!!
!! INPUTS
!!
!! SIDE EFFECTS
!!  The one-time initialization required to use FFTW3 threads is performed when the routine
!!  is called for the first time.
!!
!! PARENTS
!!      driver,fftprof
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_init_threads()


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_init_threads'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars

!Local variables ------------------------------
!scalars
 integer :: iret

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_THREADS
 if (THREADS_INITED==0) then
   call dfftw_init_threads(iret)
   !
   if (iret==0) then
     MSG_WARNING(" dfftw_init_threads returned 0; threaded FFTW3 is not being used!")
   else
     THREADS_INITED=1
   end if
   !
 end if

#ifndef HAVE_OPENMP
  MSG_WARNING("Using FFTW3 with threads but HAVE_OPENMP is not defined!")
#endif
#endif

#ifdef HAVE_FFT_FFTW3_MPI
  call fftw_mpi_init
  !write(msg,'(a)') "- Calling  fftw_mpi_init()"
  !call wrtout(std_out,msg,"COLL")
  call fftw3_set_nthreads
#endif
  
 RETURN
 ABI_UNUSED(iret)

end subroutine fftw3_init_threads
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_set_nthreads
!! NAME
!!  fftw3_set_nthreads
!!
!! FUNCTION
!!  This function sets the number of threads you want FFTW3 to use (or actually, the maximum number).
!!  It also performs any one-time initialization required to use FFTW3 threads.
!!  All plans subsequently created with any planner routine will use nthreads threads.
!!  If you pass an nthreads argument of 1 (the default), threads are disabled for subsequent plans.
!!  It does nothing if HAVE_FFT_FFTW3_THREADS is not defined.
!!
!! INPUTS
!!  [nthreads]=The number of threads you want FFTW3 to use.  Default xomp_get_max_threads()
!!
!! PARENTS
!!      m_fft_prof,m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_set_nthreads(nthreads)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_set_nthreads'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,optional,intent(in) :: nthreads

!Local variables ------------------------------
!scalars
#ifdef HAVE_FFT_FFTW3_THREADS
 integer :: istat,nt
 !character(len=500) :: msg,str_nthreads
#endif

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_THREADS
 if (THREADS_INITED==0) then
   MSG_WARNING("Threads are not initialized")
 end if

 if (PRESENT(nthreads)) then
   if (nthreads<=0) then
     nt = xomp_get_max_threads()
   else
     nt = nthreads
   end if
 else
   nt = xomp_get_max_threads()
 end if

 call dfftw_plan_with_nthreads(nt)

#ifndef HAVE_OPENMP
  MSG_WARNING("Using FFTW3 with threads but HAVE_OPENMP is not defined!")
#endif

#else
 if (PRESENT(nthreads)) then
   ABI_UNUSED(nthreads)
 end if
#endif


end subroutine fftw3_set_nthreads
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftpad_dp
!! NAME
!!  fftw3_fftpad_dp
!!
!! FUNCTION
!!  This routine transforms wavefunctions using 3D zero-padded FFTs with FFTW3.
!!  The 3D ffts are computed only on lines and planes which have non zero elements.
!!  These lines and planes are defined by the two vectors do_fft_x(ldy*nz) and do_fft_y(nz)
!!  FFT transform is in-place.
!!
!! INPUTS
!!   nx,ny,nz=Logical dimensions of the FFT mesh.
!!   ldx,ldy,ldz=Physical dimension of the f array (to avoid cache conflicts).
!!   ndat=Number of FFT transforms.
!!   mgfft=MAX(nx,ny,nz), only used to dimension gbound
!!   isign=The sign of the transform.
!!   gbound(2*mgfft+8,2)= The boundaries of the basis sphere of G vectors at a given k-point.
!!     See sphereboundary for more info.
!!
!! SIDE EFFECTS
!!   ff(2*ldx*ldy*ldz*ndat)=
!!     input: The array with the data to be transformed.
!!     output: The results of the FFT.
!!
!! PARENTS
!!      m_fftw3
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftpad_dp(ff,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign,gbound)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftpad_dp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2)
 real(dp),intent(inout) :: ff(2*ldx*ldy*ldz*ndat)

!Local variables-------------------------------
!scalars
#ifdef HAVE_FFT_FFTW3
 integer,parameter :: dst=2
 real(dp) :: fact

! *************************************************************************

#include "fftw3_fftpad.finc"

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,mgfft,isign/))
 ABI_UNUSED(gbound(1,1))
 ABI_UNUSED(ff(1))
#endif

end subroutine fftw3_fftpad_dp
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_fftpad_dpc
!! NAME
!!  fftw3_fftpad_dpc
!!
!! FUNCTION
!!  This routine transforms wavefunctions using 3D zero-padded FFTs with FFTW3.
!!  The 3D ffts are computed only on lines and planes which have non zero elements.
!!  These lines and planes are defined by the two vectors do_fft_x(ldy*nz) and do_fft_y(nz)
!!  FFT transform is in-place. Target: complex arrays.
!!
!! INPUTS
!!   nx,ny,nz=Logical dimensions of the FFT mesh.
!!   ldx,ldy,ldz=Physical dimension of the f array (to avoid cache conflicts).
!!   ndat=Number of FFT transforms.
!!   mgfft=MAX(nx,ny,nz), only used to dimension gbound.
!!   isign=The sign of the transform.
!!   gbound(2*mgfft+8,2)= The boundaries of the basis sphere of G vectors at a given k-point.
!!     See sphereboundary for more info.
!!
!! SIDE EFFECTS
!!  ff(ldx*ldy*ldz*ndat)=
!!    input: The array with the data to be transformed.
!!    output: The results of the FFT.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_fftpad_dpc(ff,nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign,gbound)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_fftpad_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign
!arrays
 integer,intent(in) :: gbound(2*mgfft+8,2)
 complex(dpc),intent(inout) :: ff(ldx*ldy*ldz*ndat)

#ifdef HAVE_FFT_FFTW3
!Local variables-------------------------------
!scalars
 integer,parameter :: dst=1
 real(dp) :: fact

! *************************************************************************

#include "fftw3_fftpad.finc"

#else
 MSG_ERROR("FFTW3 support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,ndat,mgfft,isign/))
 ABI_UNUSED(gbound(1,1))
 ABI_UNUSED(ff(1))
#endif

end subroutine fftw3_fftpad_dpc
!!***

#ifdef HAVE_FFT_FFTW3

!----------------------------------------------------------------------

!!****f* m_fftw3/dplan_many_dft
!! NAME
!!
!! FUNCTION
!!
!! INPUTS
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function dplan_many_dft(rank,n,howmany,fin,inembed,istride,idist,fout,onembed,ostride,odist,sign,flags,nthreads) result(plan)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'dplan_many_dft'
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: rank,howmany,istride,ostride, sign,flags,idist,odist,nthreads
 integer,intent(in) :: n(rank),inembed(rank),onembed(rank)
 integer(KIND_FFTW_PLAN) :: plan
!arrays
 real(dp) :: fin(*),fout(*)

!Local variables-------------------------------
 character(len=500) :: msg,frmt

! *************************************************************************
 
!$OMP CRITICAL (OMPC_dfftw_plan_many_dft)
 call fftw3_set_nthreads(nthreads)

 call dfftw_plan_many_dft(plan, rank, n, howmany, &
&  fin, inembed, istride, idist, fout, onembed, ostride, odist, sign, flags)
!$OMP END CRITICAL (OMPC_dfftw_plan_many_dft)

 if (plan==NULL_PLAN) then
   call wrtout(std_out,"dfftw_plan_many_dft returned NULL_PLAN!","COLL")
   write(frmt,*)"(a,",rank,"(1x,i0),3(a,i0),a,2(a,",rank,"(1x,i0),2(a,i0),a))"
   write(msg,frmt)&
&    " n= ",n," howmany= ",howmany," sign= ",sign," flags= ",flags,ch10,&
&    " inembed= ",inembed," istride= ",istride," idist=",idist,ch10,    &
&    " onembed= ",onembed," ostride= ",ostride," odist=",idist,ch10
   call wrtout(std_out,msg,"COLL")
   MSG_ERROR("Check FFTW library and/or abinit code")
 end if

end function dplan_many_dft
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/cplan_many_dft
!! NAME
!!
!! FUNCTION
!!  
!! INPUTS
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
!! FIXME  technically it should be intent(inout) since FFTW3 can destroy the input for particular flags.

function cplan_many_dft(rank,n,howmany,fin,inembed,istride,idist,fout,onembed,ostride,odist,sign,flags,nthreads) result(plan)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'cplan_many_dft'
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: rank,howmany,istride,ostride, sign,flags,idist,odist,nthreads
 integer,intent(in) :: n(rank),inembed(rank),onembed(rank)
 integer(KIND_FFTW_PLAN) :: plan
!arrays
 complex(spc) :: fin(*),fout(*) 

!Local variables-------------------------------
 character(len=500) :: msg,frmt

! *************************************************************************
 
!$OMP CRITICAL (OMPC_cplan_many_dft)
 call fftw3_set_nthreads(nthreads)

 call sfftw_plan_many_dft(plan, rank, n, howmany, &
&  fin, inembed, istride, idist, fout, onembed, ostride, odist, sign, flags)
!$OMP END CRITICAL (OMPC_cplan_many_dft)

 if (plan==NULL_PLAN) then ! handle the error
   call wrtout(std_out,"sfftw_plan_many_dft returned NULL_PLAN (complex version)","COLL")
   write(frmt,*)"(a,",rank,"(1x,i0),3(a,i0),a,2(a,",rank,"(1x,i0),2(a,i0),a))"
   write(msg,frmt)&
&    " n = ",n," howmany = ",howmany," sign = ",sign," flags = ",flags,ch10,&
&    " inembed = ",inembed," istride = ",istride," idist =",idist,ch10,     &
&    " onembed = ",onembed," ostride = ",ostride," odist =",idist,ch10
   call wrtout(std_out,msg,"COLL")
   MSG_ERROR("Check FFTW library and/or abinit code")
 end if

end function cplan_many_dft
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/zplan_many_dft
!! NAME
!!
!! FUNCTION
!!
!! INPUTS
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
!! FIXME  technically it should be intent(inout) since FFTW3 can destroy the input for particular flags.

function zplan_many_dft(rank,n,howmany,fin,inembed,istride,idist,fout,onembed,ostride,odist,sign,flags,nthreads) result(plan)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'zplan_many_dft'
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: rank,howmany,istride,ostride, sign,flags,idist,odist,nthreads
 integer,intent(in) :: n(rank),inembed(rank),onembed(rank)
 integer(KIND_FFTW_PLAN) :: plan
!arrays
 complex(dpc) :: fin(*),fout(*)

!Local variables-------------------------------
 character(len=500) :: msg,frmt

! *************************************************************************
 
!$OMP CRITICAL (OMPC_zplan_many_dft)
 call fftw3_set_nthreads(nthreads)

 call dfftw_plan_many_dft(plan, rank, n, howmany, &
&  fin, inembed, istride, idist, fout, onembed, ostride, odist, sign, flags)
!$OMP END CRITICAL (OMPC_zplan_many_dft)

 if (plan==NULL_PLAN) then ! handle the error
   call wrtout(std_out,"dfftw_plan_many_dft returned NULL_PLAN (complex version)","COLL")
   write(frmt,*)"(a,",rank,"(1x,i0),3(a,i0),a,2(a,",rank,"(1x,i0),2(a,i0),a))"
   write(msg,frmt)&
&    " n = ",n," howmany = ",howmany," sign = ",sign," flags = ",flags,ch10,&
&    " inembed = ",inembed," istride = ",istride," idist =",idist,ch10,     &
&    " onembed = ",onembed," ostride = ",ostride," odist =",idist,ch10
   call wrtout(std_out,msg,"COLL")
   MSG_ERROR("Check FFTW library and/or abinit code")
 end if

end function zplan_many_dft
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/dplan_many_dft_r2c
!! NAME
!!
!! FUNCTION
!!
!! INPUTS
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
!! FIXME  technically it should be intent(inout) since FFTW3 can destroy the input
!! for particular flags.

function dplan_many_dft_r2c(rank,n,howmany,fin,inembed,istride,idist,fout,onembed,ostride,odist,flags,nthreads) result(plan)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'dplan_many_dft_r2c'
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: rank,howmany,istride,ostride,flags,idist,odist,nthreads
 integer,intent(in) :: n(rank),inembed(rank),onembed(rank)
 integer(KIND_FFTW_PLAN) :: plan
!arrays
 real(dp) :: fin(*),fout(*)

!Local variables-------------------------------
 character(len=500) :: msg,frmt

! *************************************************************************

!$OMP CRITICAL (OMPC_dplan_many_dft_r2c)
 call fftw3_set_nthreads(nthreads)

 call dfftw_plan_many_dft_r2c(plan, rank, n, howmany, &
&  fin, inembed, istride, idist, fout, onembed, ostride, odist, flags)
!$OMP END CRITICAL (OMPC_dplan_many_dft_r2c)

 if (plan==NULL_PLAN) then ! handle the error.
   call wrtout(std_out,"dfftw_plan_many_dft_r2c returned NULL_PLAN","COLL")
   write(frmt,*)"(a,",rank,"(1x,i0),2(a,i0),a,2(a,",rank,"(1x,i0),2(a,i0),a))"
   write(msg,frmt)&
&    " n = ",n," howmany = ",howmany," flags = ",flags,ch10,&
&    " inembed = ",inembed," istride = ",istride," idist = ",idist,ch10,&
&    " onembed = ",onembed," ostride = ",ostride," odist = ",idist,ch10
   call wrtout(std_out,msg,"COLL")
   MSG_ERROR("Check FFTW library and/or abinit code")
 end if

end function dplan_many_dft_r2c
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/dplan_many_dft_c2r
!! NAME
!!
!! FUNCTION
!!
!! INPUTS
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function dplan_many_dft_c2r(rank,n,howmany,fin,inembed,istride,idist,fout,onembed,ostride,odist,flags, nthreads) result(plan)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'dplan_many_dft_c2r'
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: rank,howmany,istride,ostride,flags,idist,odist,nthreads
 integer,intent(in) :: n(rank),inembed(rank),onembed(rank)
 integer(KIND_FFTW_PLAN) :: plan
!arrays
 real(dp) :: fin(*),fout(*)

!Local variables-------------------------------
 character(len=500) :: msg,frmt

! *************************************************************************

!$OMP CRITICAL (OMPC_dplan_many_dft_c2r)
 call fftw3_set_nthreads(nthreads)

 call dfftw_plan_many_dft_c2r(plan, rank, n, howmany, &
&  fin, inembed, istride, idist, fout, onembed, ostride, odist, flags)
!$OMP END CRITICAL (OMPC_dplan_many_dft_c2r)

 if (plan==NULL_PLAN) then ! handle the error.
   call wrtout(std_out,"dfftw_plan_many_dft_c2r returned NULL_PLAN","COLL")
   write(frmt,*)"(a,",rank,"(1x,i0),2(a,i0),a,2(a,",rank,"(1x,i0),2(a,i0),a))"
   write(msg,frmt)&
&    " n = ",n," howmany = ",howmany," flags = ",flags,ch10,&
&    " inembed = ",inembed," istride = ",istride," idist = ",idist,ch10,&
&    " onembed = ",onembed," ostride = ",ostride," odist = ",idist,ch10
   call wrtout(std_out,msg,"COLL")
   MSG_ERROR("Check FFTW library and/or abinit code")
 end if

end function dplan_many_dft_c2r
!!***

#endif

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_execute_dft_dp
!! NAME
!! fftw3_execute_dft_dp
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! NOTES
!!  This interface is used to perform complex to complex FFT with real arrays 
!!  containing the real and imaginary part. I have to admit that this interface
!!  is a bit ambiguous since FFTW3 provides routines for real-to-real transforms.
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

#ifdef HAVE_FFT_FFTW3

subroutine fftw3_execute_dft_dp(plan, in, out)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_execute_dft_dp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer(KIND_FFTW_PLAN),intent(in) :: plan 
 real(C_DOUBLE),intent(inout) :: in(*)
 real(C_DOUBLE),intent(out) :: out(*)
!arrays

! *************************************************************************

 call dfftw_execute_dft(plan, in, out)

end subroutine fftw3_execute_dft_dp
!!***

#endif

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_execute_dft_spc
!! NAME
!! fftw3_execute_dft_spc
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

#ifdef HAVE_FFT_FFTW3

subroutine fftw3_execute_dft_spc(plan, in, out)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_execute_dft_spc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer(KIND_FFTW_PLAN),intent(in) :: plan 
 complex(C_FLOAT_COMPLEX),intent(inout) :: in(*)
 complex(C_FLOAT_COMPLEX),intent(out) :: out(*)
!arrays

! *************************************************************************

 call sfftw_execute_dft(plan, in, out)

end subroutine fftw3_execute_dft_spc
!!***

#endif


!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_execute_dft_dpc
!! NAME
!! fftw3_execute_dft_dpc
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

#ifdef HAVE_FFT_FFTW3

subroutine fftw3_execute_dft_dpc(plan, in, out)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_execute_dft_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer(KIND_FFTW_PLAN),intent(in) :: plan 
 complex(C_DOUBLE_COMPLEX),intent(inout) :: in(*)
 complex(C_DOUBLE_COMPLEX),intent(out) :: out(*)
!arrays

! *************************************************************************

 call dfftw_execute_dft(plan, in, out)

end subroutine fftw3_execute_dft_dpc
!!***

#endif


!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_alloc_real_dp
!! NAME
!! fftw3_alloc_real_dp
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

#ifdef HAVE_FFT_FFTW3

subroutine fftw3_alloc_real_dp(size,cptr,fptr)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_alloc_real_dp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: size
 real(dp),ABI_CONTIGUOUS pointer :: fptr(:)
 type(C_PTR),intent(out) :: cptr
!arrays

! *************************************************************************

 cptr = fftw_malloc( INT(size*C_DOUBLE, KIND=C_SIZE_T))
 if (.not. C_ASSOCIATED(cptr)) then
   MSG_ERROR("fftw_malloc returned NULL!")
 end if

 call c_f_pointer(cptr, fptr, [size])

end subroutine fftw3_alloc_real_dp
!!***

#endif

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_alloc_complex_spc
!! NAME
!! fftw3_alloc_complex_spc
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

#ifdef HAVE_FFT_FFTW3

subroutine fftw3_alloc_complex_spc(size,cptr,fptr)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_alloc_complex_spc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: size
 complex(spc),ABI_CONTIGUOUS pointer :: fptr(:)
 type(C_PTR),intent(out) :: cptr
!arrays

! *************************************************************************

 cptr = fftw_malloc( INT(2*size*C_FLOAT, KIND=C_SIZE_T))
 if (.not. C_ASSOCIATED(cptr)) then
   MSG_ERROR("fftw_malloc returned NULL!")
 end if

 call c_f_pointer(cptr, fptr, [size])

end subroutine fftw3_alloc_complex_spc
!!***

#endif 

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_alloc_complex_dpc
!! NAME
!! fftw3_alloc_complex_dpc
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

#ifdef HAVE_FFT_FFTW3

subroutine fftw3_alloc_complex_dpc(size,cptr,fptr)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_alloc_complex_dpc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: size
 complex(dpc),ABI_CONTIGUOUS pointer :: fptr(:)
 type(C_PTR),intent(out) :: cptr
!arrays

! *************************************************************************

 cptr = fftw_malloc( INT(2*size*C_DOUBLE, KIND=C_SIZE_T))
 if (.not. C_ASSOCIATED(cptr)) then
   MSG_ERROR("fftw_malloc returned NULL!")
 end if

 call c_f_pointer(cptr, fptr, [size])

end subroutine fftw3_alloc_complex_dpc
!!***

#endif 

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_spawn_threads_here
!! NAME
!! fftw3_spawn_threads_here
!!
!! FUNCTION
!!  Helper function that returns true if FFT calls should be OMP
!!  parallelized in the client code.
!!
!! INPUTS
!!  ndat=Number of FFT transforms to do
!!  nthreads = Number of threads available
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function fftw3_spawn_threads_here(ndat,nthreads) result(ans)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_spawn_threads_here'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ndat,nthreads
 logical :: ans
!arrays

! *************************************************************************

 ans = .FALSE.
#ifdef HAVE_OPENMP
 ans = (nthreads > 1 .and. MOD(ndat,nthreads) == 0 .and. .not. USE_LIB_THREADS)
#else
 ABI_UNUSED((/ndat,nthreads/))
#endif

end function fftw3_spawn_threads_here
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftw3_use_lib_threads
!! NAME
!! fftw3_use_lib_threads
!!
!! FUNCTION
!!
!! INPUTS
!!
!! PARENTS
!!      m_fft
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftw3_use_lib_threads(logvar)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftw3_use_lib_threads'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 logical,intent(in) :: logvar
!arrays

! *************************************************************************
 
 USE_LIB_THREADS = logvar

end subroutine fftw3_use_lib_threads
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftwmpi_get_work_array
!! NAME
!!  fftwmpi_get_work_array
!!
!! FUNCTION
!! Driver routine for allocate fftw work arrray for 3D complex-to-complex FFTs of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ndat=Number of FFTs to be done.
!!
!! OUTPUT
!! cdata_f,cdata_r: C pointers to use for fourier andreal data
!! n0,n0_tr : local size on the shared dimension (nz or ny if transposed mode is used)
!! offset,offset_tr : offset per process in continuous tabx
!!
!! PARENTS
!!      fftwmpi_fourwf
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE
subroutine fftwmpi_get_work_array(cdata_f,cdata_r,rank,nx,ny,nz,ndat,mpi_enreg,n0,offset,n0_tr,offset_tr)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_get_work_array'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ndat,rank
 type(MPI_type),intent(in) :: mpi_enreg
 integer(C_INTPTR_T), intent(out) :: n0, offset, n0_tr, offset_tr
 type(C_PTR), intent(out) :: cdata_f,cdata_r
!arrays

!Local variables-------------------------------
#ifdef HAVE_FFT_FFTW3_MPI
!scalars
!arrays
 integer(C_INTPTR_T)  :: alloc_local,fft_sizes(4)
#endif

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_MPI
 fft_sizes(1)=nz
 fft_sizes(2)=ny
 fft_sizes(3)=nx
 fft_sizes(4)=ndat

 alloc_local = fftw_mpi_local_size_many_transposed(rank,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, mpi_enreg%comm_fft, &
      n0,offset, &
      n0_tr,offset_tr)

 cdata_f = fftw_alloc_complex(alloc_local)
 cdata_r = fftw_alloc_complex(alloc_local)
#else
  MSG_ERROR("FFTW3_MPI support not activated")
  ABI_UNUSED((/nx,ny,nz,ndat,rank/))
  ABI_UNUSED(mpi_enreg%nproc_fft)
  cdata_f = C_NULL_PTR
  cdata_r = C_NULL_PTR
  n0        = 0
  offset    = 0
  n0_tr     = 0
  offset_tr = 0
#endif

end subroutine fftwmpi_get_work_array
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftwmpi_free_work_array
!! NAME
!!  fftwmpi_free_work_array
!!
!! FUNCTION
!! Driver routine for freeing fftw work arrray
!!
!! INPUTS
!!
!! OUTPUT
!! cdata_f,cdata_r: C pointers to free for fourier andreal data
!!
!! PARENTS
!!      fftwmpi_fourwf
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE
subroutine fftwmpi_free_work_array(cdata_f,cdata_r)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_free_work_array'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(C_PTR), intent(inout) :: cdata_f,cdata_r
!arrays

!Local variables-------------------------------
!scalars
!arrays

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_MPI
 call fftw_free(cdata_r)
 call fftw_free(cdata_f)
#else
 MSG_ERROR("FFTW3_MPI support not activated")
 if(.false.) then
   cdata_r = C_NULL_PTR
   cdata_f = C_NULL_PTR
 end if
#endif

end subroutine fftwmpi_free_work_array
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftwmpi_many_dft_ip
!! NAME
!!  fftwmpi_many_dft_ip
!!
!! FUNCTION
!! Driver routine for many out-of-place 3D complex-to-complex FFTs of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimension of the fin and fout arrays (to avoid cache conflicts).
!! ndat=Number of FFTs to be done.
!! fin(2*ldx*ldy*ldz*ndat)=The complex array to be transformed.
!! isign=sign of Fourier transform exponent: current convention uses
!!   +1 for transforming from G to r,
!!   -1 for transforming from r to G.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! fout(2,ldx*ldy*ldz*ndat)=The Fourier transform of fin.
!!
!! PARENTS
!!      fftwmpi_fourdp,fftwmpi_fourwf
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftwmpi_many_dft_ip(mpi_enreg,nx,ny,nz,ldx,ldy,ldz,ndat,isign,fin,fout,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_many_dft_ip'
!End of the abilint section

  implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 real(dp),intent(in) :: fin(2,ldx,ldy,ldz*ndat)
 real(dp),intent(out) :: fout(2,ldx,ldy,ldz*ndat)

#ifdef HAVE_FFT_FFTW3_MPI
!Local variables-------------------------------
!scalars
 integer,parameter :: rank=3
 integer :: my_flags
 real(dp):: facteur_fft
 !arrays
 
 !FFTWMPI stuff
 type(C_PTR) :: plan, cdata
 complex(C_DOUBLE_COMPLEX), pointer :: data(:,:,:)
 integer(C_INTPTR_T) :: i, j, k, alloc_local, local_n0, local_n0_offset,fft_sizes(4)

!*************************************************************************

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags
 ! get local data size and allocate (note dimension reversal)

 fft_sizes(1)=nz
 fft_sizes(2)=ny
 fft_sizes(3)=nx
 fft_sizes(4)=ndat

 alloc_local = fftw_mpi_local_size_many(3,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK, mpi_enreg%comm_fft, &
      local_n0,local_n0_offset)

 cdata = fftw_alloc_complex(alloc_local)
 call c_f_pointer(cdata, data, [fft_sizes(3),fft_sizes(2), local_n0])

 plan =  fftw_mpi_plan_many_dft(3,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, &
      data,data,mpi_enreg%comm_fft,isign,my_flags)

 do k=1, local_n0*ndat
    do j=1, ny
       do i=1, nx
          data(i,j,k) = CMPLX( fin(1,i,j,k),fin(2,i,j,k),C_DOUBLE_COMPLEX)
       end do
    end do
 end do

 ! compute transform (as many times as desired)
 call fftw_mpi_execute_dft(plan, data, data)


 if(isign==ABI_FFTW_FORWARD) then
    facteur_fft = 1.d0 / (1.d0*nx*ny*nz)
 else
    facteur_fft = 1.d0
 endif
 do k=1, local_n0*ndat
    do j=1, ny
       do i=1, nx
          fout(1,i,j,k) =  real(data(i,j,k)) * facteur_fft
          fout(2,i,j,k) = aimag(data(i,j,k)) * facteur_fft
       end do
    end do
 end do

 call fftw_destroy_plan(plan)
 call fftw_free(cdata)

#else
 MSG_ERROR("FFTW3_MPI support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz,ndat,isign/))
 ABI_UNUSED(mpi_enreg%nproc_fft)
 if (PRESENT(fftw_flags)) then
    ABI_UNUSED(fftw_flags)
 end if
 ABI_UNUSED(fin(1,1,1,1))
 ABI_UNUSED(fout(1,1,1,1))
#endif

end subroutine fftwmpi_many_dft_ip
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftwmpi_many_dft_tr
!! NAME
!!  fftwmpi_many_dft_tr
!!
!! FUNCTION
!! Driver routine for many out-of-place 3D complex-to-complex FFTs of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimension of the fin and fout arrays (to avoid cache conflicts).
!! ndat=Number of FFTs to be done.
!! fin(2*ldx*ldy*ldz*ndat)=The complex array to be transformed.
!! isign=sign of Fourier transform exponent: current convention uses
!!   +1 for transforming from G to r,
!!   -1 for transforming from r to G.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! fout(2,ldx*ldy*ldz*ndat)=The Fourier transform of fin.
!!
!! PARENTS
!!      fftwmpi_fourwf
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftwmpi_many_dft_tr(mpi_enreg,nx,ny,nz,ndat,isign,fin,fout,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_many_dft_tr'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ndat,isign
 integer,optional,intent(in) :: fftw_flags
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 complex(C_DOUBLE_COMPLEX),pointer,  intent(inout)  :: fin(:,:,:)
 complex(C_DOUBLE_COMPLEX),pointer,  intent(out) :: fout(:,:,:)

!Local variables-------------------------------
#ifdef HAVE_FFT_FFTW3_MPI
!scalars
 integer :: my_flags
 !FFTWMPI stuff
 type(C_PTR) :: plan
 integer(C_INTPTR_T) :: fft_sizes(4)
#endif

!Uncomment to instrument for scalasca
!#ifdef EPIK
!#include "epik_user.inc"
! EPIK_USER_REG(r1,"fftw_mpi_plan")
! EPIK_USER_REG(r2,"fftw_mpi_exec")
! EPIK_USER_REG(r3,"fftw_destoy")
!#endif

!*************************************************************************

#ifdef HAVE_FFT_FFTW3_MPI
 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags
 my_flags = ior(my_flags,FFTW_DESTROY_INPUT)

 fft_sizes(1)=nz
 fft_sizes(2)=ny
 fft_sizes(3)=nx
 fft_sizes(4)=ndat

!#ifdef EPIK
! EPIK_USER_START(r1)
!#endif
 plan = fftw_mpi_plan_many_dft(3,fft_sizes(1:3),fft_sizes(4), &
        FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, &
        fin,fout,mpi_enreg%comm_fft,isign,my_flags)
!#ifdef EPIK
! EPIK_USER_END(r1)
! EPIK_USER_START(r2)
!#endif

!Compute transform (as many times as desired)
 call fftw_mpi_execute_dft(plan, fin, fout)
!#ifdef EPIK
! EPIK_USER_END(r2)
! EPIK_USER_START(r3)
!#endif
 call fftw_destroy_plan(plan)
!#ifdef EPIK
! EPIK_USER_END(r3)
!#endif

#else
 MSG_ERROR("FFTW3_MPI support not activated")
 ABI_UNUSED((/nx,ny,nz,ndat,isign/))
 ABI_UNUSED(mpi_enreg%nproc_fft)
 if (PRESENT(fftw_flags)) then
    ABI_UNUSED(fftw_flags)
 end if
 ABI_UNUSED(fin(1,1,1))
 ABI_UNUSED(fout(1,1,1))
#endif

end subroutine fftwmpi_many_dft_tr
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftwmpi_c2r_tr
!! NAME
!!  fftwmpi_c2r_tr
!!
!! FUNCTION
!! Driver routine for transposed out-of-place 3D complex-to-real FFT of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of point along the three directions.
!! ldx,ldy,ldz=Physical dimension of the f array (to avoid cache conflicts).
!! ndat=Number of FFTs to be done.
!! ff(2,nx*ny*nz*ndat)=The complex array to be transformed.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! gg(2,nx*ny*nz*ndat)=The backwards real FFT of ff.
!!
!! NOTES
!! LOCAL DATA IN FOURIER SPACE : TRANSPOSED ORDER
!! real space     --> dim = [  nx  | ny | nz/np_fft]
!! fourier  space --> dim = [ nx/2 | nz | ny/np_ff ]
!!
!! PARENTS
!!      fftwmpi_fourdp
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftwmpi_c2r_tr(mpi_enreg,nx,ny,nz,ldx,ldy,ldz,ndat,ff,gg,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_c2r_tr'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat
 integer,optional,intent(in) :: fftw_flags
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 real(dp),intent(in) :: ff(2,nx*ny*nz*ndat)
 real(dp),intent(out) :: gg(nx*ny*nz*ndat)

!Local variables-------------------------------
#ifdef HAVE_FFT_FFTW3_MPI
!scalars
 integer,parameter :: rank=3
 integer :: np_fft

!FFTWMPI stuff
 type(C_PTR) :: plan_bw, cdata_cplx,cdata_real
 complex(C_DOUBLE_COMPLEX), pointer :: data_cplx(:,:,:)
 real(C_DOUBLE), pointer :: data_real(:,:,:)
 integer(C_INTPTR_T) :: i, j, k, alloc_local,fft_sizes(4),demi_nx
 integer(C_INTPTR_T) :: local_n0, local_n0_offset, local_n0_tr, local_n0_tr_offset
#endif

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_MPI
 np_fft = mpi_enreg%nproc_fft

 demi_nx = nx/2 + 1
 fft_sizes(1)=nz
 fft_sizes(2)=ny
 fft_sizes(3)=demi_nx
 fft_sizes(4)=ndat

 if(ndat/=1) then
    MSG_ERROR("fftwmpi_c2r_tr: ndat > 1 not implemented yet")
 end if

 alloc_local = fftw_mpi_local_size_many_transposed(rank,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, mpi_enreg%comm_fft, &
      local_n0,local_n0_offset, &
      local_n0_tr,local_n0_tr_offset)

 cdata_cplx = fftw_alloc_complex(alloc_local)
 cdata_real = fftw_alloc_real(alloc_local*2)

 !dimensions are  (x/2,z,y) in Fourrier's Space
 call c_f_pointer(cdata_cplx, data_cplx, [demi_nx  ,fft_sizes(1),local_n0_tr])
 !dimensions in real space : (nx,ny,nz/nproc)
 call c_f_pointer(cdata_real, data_real, [2*demi_nx,fft_sizes(2),local_n0])


 fft_sizes(3)=nx
 plan_bw =  fftw_mpi_plan_many_dft_c2r(rank,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, &
      data_cplx, data_real , &
      mpi_enreg%comm_fft,ior(ABI_FFTW_ESTIMATE,ABI_FFTW_MPI_TRANSPOSED_IN))

 do k=1, nz*ndat
    do j=1, ny / np_fft
       do i=1, demi_nx
          !data_cplx(i,k,j) = ff(:,i,j,k)
          data_cplx(i,k,j) = CMPLX( ff(1, i+ ldx*((j-1) + (ldy/np_fft)*(k-1)) ) , &
               ff(2, i+ldx*((j-1) + (ldy/np_fft)*(k-1)) ) , C_DOUBLE_COMPLEX)
       end do
    end do
 end do

 ! compute transform (as many times as desired)
 call fftw_mpi_execute_dft_c2r(plan_bw, data_cplx, data_real)

 do k=1, local_n0*ndat
    do j=1, ny
       do i=1, nx
          gg(i+nx*((j-1) + ny*(k-1))) = data_real(i,j,k)
       end do
    end do
 end do

 call fftw_destroy_plan(plan_bw)
 call fftw_free(cdata_cplx)
 call fftw_free(cdata_real)

#else
 MSG_ERROR("FFTW3_MPI support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz/))
 ABI_UNUSED(ff(1,1))
 ABI_UNUSED(gg(1))
 ABI_UNUSED(mpi_enreg%nproc_fft)
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftwmpi_c2r_tr
!!***

!----------------------------------------------------------------------

!!****f* m_fftw3/fftwmpi_r2c_tr
!! NAME
!!  fftwmpi_r2c_tr
!!
!! FUNCTION
!! Driver routine for out-of-place 3D real-to-complex FFT of lengths nx, ny, nz.
!!
!! INPUTS
!! nx,ny,nz=Number of points along the three directions.
!! ldx,ldy,ldz=Physical dimensions of the f array (to avoid cache conflicts).
!! ff(nx*ny*nz*ndat)=The real array to be transformed.
!! ndat=Number of FFTs to be done.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator.
!!   Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT
!! gg(2,nx*ny*nz*ndat)=The forward FFT of ff.
!!
!! NOTES
!! LOCAL DATA FOR FOURIER TRANSFORMS : TRANSPOSED ORDER AND DISTRIBUTED
!! real space     --> dim = [  nx  | ny | nz/np_fft ]
!! fourier  space --> dim = [  nx | nz | ny/np_fft ]
!! we can't take in account the symetric of the real case because after
!! fft have been computed, the symetric data needed are dispatched over
!! other process in parallel
!!
!! PARENTS
!!      fftwmpi_fourdp
!!
!! CHILDREN
!!      c_f_pointer,fftw_destroy_plan,fftw_free,fftw_mpi_execute_dft
!!
!! SOURCE

subroutine fftwmpi_r2c_tr(mpi_enreg,nx,ny,nz,ldx,ldy,ldz,ndat,ff,gg,fftw_flags)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_r2c_tr'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx,ny,nz,ldx,ldy,ldz,ndat
 integer,optional,intent(in) :: fftw_flags
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 real(dp),intent(in) :: ff(nx*ny*nz*ndat)
 real(dp),intent(out) :: gg(2,nx*ny*nz*ndat)

!Local variables-------------------------------
#ifdef HAVE_FFT_FFTW3_MPI
 !scalars
 integer,parameter :: rank=3
 integer :: my_flags,i1,i2,i3,igf
 integer :: idat,padatf,np_fft
 real(dp):: facteur_fft

 !FFTWMPI stuff
 type(C_PTR) :: plan_fw, cdata_cplx,cdata_real
 complex(C_DOUBLE_COMPLEX), pointer :: data_cplx(:,:,:),data_real(:,:,:)
 integer(C_INTPTR_T) :: i, j, k, alloc_local,fft_sizes(4)
 integer(C_INTPTR_T) :: local_n0, local_n0_offset, local_n0_tr, local_n0_tr_offset
#endif

! *************************************************************************

#ifdef HAVE_FFT_FFTW3_MPI
 np_fft = mpi_enreg%nproc_fft

 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 fft_sizes(1)=nz
 fft_sizes(2)=ny
 fft_sizes(3)=nx
 fft_sizes(4)=ndat

 !Get parallel sizes
 alloc_local = fftw_mpi_local_size_many_transposed(rank,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, mpi_enreg%comm_fft, &
      local_n0,local_n0_offset, &
      local_n0_tr,local_n0_tr_offset)


 !Allocate data and reference it
 cdata_cplx = fftw_alloc_complex(alloc_local)
 cdata_real = fftw_alloc_complex(alloc_local)

 !! local data in fourier  space --> dim = [ nx | nz | ny/np_fft ]
 call c_f_pointer(cdata_cplx, data_cplx, [fft_sizes(3),fft_sizes(1),local_n0_tr])
 !! local data in real space     --> dim = [  nx  | ny | nz/np_fft ]
 call c_f_pointer(cdata_real, data_real, [fft_sizes(3),fft_sizes(2),local_n0])

 !Create Plan C2C (nx,ny,nz)
 plan_fw =  fftw_mpi_plan_many_dft(rank,fft_sizes(1:3),fft_sizes(4), &
      FFTW_MPI_DEFAULT_BLOCK,FFTW_MPI_DEFAULT_BLOCK, &
      data_real, data_cplx , &
      mpi_enreg%comm_fft,ABI_FFTW_FORWARD,ior(ABI_FFTW_ESTIMATE,ABI_FFTW_MPI_TRANSPOSED_OUT))

 ! Copy input data in correct format
 do k=1,local_n0*ndat
    do j=1, ny
       do i=1, nx
          !data_real(i,j,k) = ff(i,j,k)
          data_real(i,j,k) = CMPLX( ff(i+ldx*((j-1) + ldy*(k-1))),0.d0,C_DOUBLE_COMPLEX)
       end do
    end do
 end do

 !Compute transform
 call fftw_mpi_execute_dft(plan_fw, data_real, data_cplx)

 facteur_fft = 1.d0 / (1.d0*nx*ny*nz)
 do idat=1,ndat
    padatf=(idat-1)*ldx*ldy*ldz
    do i3=1,nz
       do i2=1,ny / np_fft ! equilvalent a local_n0_tr
          do i1=1,nx
             igf = i1 + nx*( (i2-1) + (i3-1)*ny/np_fft  ) + padatf
             gg(1,igf) = real(data_cplx(i1,i3,i2)) * facteur_fft
             gg(2,igf) =aimag(data_cplx(i1,i3,i2)) * facteur_fft
          end do
       end do
    end do
 end do

 call fftw_destroy_plan(plan_fw)
 call fftw_free(cdata_cplx)
 call fftw_free(cdata_real)

#else
 MSG_ERROR("FFTW3_MPI support not activated")
 ABI_UNUSED((/nx,ny,nz,ldx,ldy,ldz/))
 ABI_UNUSED(ff)
 ABI_UNUSED(gg(1,1))
 ABI_UNUSED(mpi_enreg%nproc_fft)
 if (PRESENT(fftw_flags)) then
   ABI_UNUSED(fftw_flags)
 end if
#endif

end subroutine fftwmpi_r2c_tr
!!***

END MODULE m_fftw3
!!***
