!{\src2tex{textfont=tt}}
!!****f* ABINIT/fit_gen
!! NAME
!!  fit_gen
!!
!! FUNCTION
!!  Fits a given input function f(r) to a sum of gaussians
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  accuracy= Wanted accuracy. 
!!  maxterm= maximum number of terms allowed.
!!    The searching algorithm gradually increases the number of terms up to maxterm.
!!  mparam= maximum number of parameters
!!         if(option==1)mparam=maxterm*4
!!         if(option==2)mparam=maxterm*6
!!         if(option==3)mparam=maxterm*2
!!         if(option==4)mparam=maxterm*4
!!  nr= number of real space points
!!  pawrad= pawrad type
!!  option=1 fit to a1 cos(a2 x^2)+ a3 sin( a4 x^2)
!!         2 fit to a1 exp(-a2 x^2)*(a3 cos (a4 x^2) + a5 sin (a6 x^2) )
!!         3 fit to a1 cos (k x^2) + a2 sin (k x^2)
!!         4 fit to a1 exp(-a2 x^2)* (a3 cos(k x^2)+ a4 sin (k x^2))
!!  outfile= filename to write out fitted functions.
!!  rpaw=paw radius
!!  y(nr)= function to fit
!! 
!!
!! OUTPUT
!! nparam_out= number of parameters found.
!! param_out(nparam_out)= parameters (coefficients and factors of complex gaussians).
!! 
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"
 subroutine fit_gen(accuracy,maxterm,mparam,nparam_out,nr,&
&param_out,pawrad,option,outfile,rpaw,y)
    
 use defs_basis
 use defs_datatypes
 use m_profiling

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fit_gen'
 use interfaces_14_hidewrite
 use interfaces_28_numeric_noabirule
 use interfaces_64_atompaw, except_this_one => fit_gen
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in)::maxterm,mparam,nr
 integer,intent(in)::option 
 real(dp),intent(in)::accuracy,rpaw
 real(dp),intent(inout)::y(nr)
 character(80),intent(in)::outfile
 type(pawrad_type),intent(in) :: pawrad
 !
 integer,intent(out)::nparam_out
 real(dp),intent(out)::param_out(mparam)!------------------------------------------------------------------

!Local variables-------------------------------
 integer :: ii
 integer:: lim1,lim2,maxiter
 integer :: minterm,mterm,nparam,nskip,nterm,ntrial
 integer :: startexp,step,verbosity
 integer,allocatable::iord(:)
 logical :: first,found
 logical,allocatable::skip(:)
 real(dp):: chisq,error,minsep
 !arrays
 integer::constrains(mparam)
 real(dp)::limit(mparam),param_tmp(mparam),weight(mparam)
 real(dp),allocatable::error_trial(:),sep(:),y_out(:)
 character(len=500) :: message                   ! to be uncommented, if needed
 
! *************************************************************************
 
!DEBUG
!write (std_out,*) ' fit_gen : enter'
!ENDDEBUG


!arguments
!---- Local variables
!------------------------------------------------------------------

!
!initialize variables
 step=3
 startexp=2
 maxiter=50
 if(accuracy < 1e-7) maxiter=150
 if(accuracy < 1e-8) maxiter=200
 if(accuracy<=1e-9) then
   write(message,'(3a)')"WARNING: the required accuracy is too low",ch10,&
&   "The algorithm may not converge"
   call wrtout(std_out,message,'COLL')
 end if
!
!optimized variables:
!
 if(option==4) then
   if(accuracy < 1e-4) startexp=10 !it might not find it with less than this
   if(accuracy < 1e-7) startexp=20 
   if(accuracy < 1e-8) startexp=40 
 end if
!
!number of trials, depending of the option chosen
 ntrial=6
 if(option==3) ntrial=1
 if(option==4) ntrial=1
!
!Allocate objects
!
 ABI_ALLOCATE(sep,(ntrial))
 ABI_ALLOCATE(error_trial,(ntrial))
 ABI_ALLOCATE(iord,(ntrial))
 ABI_ALLOCATE(skip,(ntrial))
 ABI_ALLOCATE(y_out,(nr))
!
!fill with zeros
!
 nparam_out=0
 y_out=0.d0
!
!Find different trial values for "sep"
 do ii=1,ntrial 
   sep(ii)=1.0d0+real(ii-1,dp)*0.1d0
 end do
!
!debug
!do ii=1,nr
!write(500,*)pawrad%rad(ii),y(ii)
!end do
!
!Start a minimization, changing the
!initial guess several times.
!only when ntrial>1
!
 verbosity=0 !print the minimum at the terminal
 first=.true.
 found=.false.
 error=999999999
 error_trial=0.d0 
 skip=.false.
 nskip=0
 message='   sep    nterm    chisq       best'
 call wrtout(std_out,message,'COLL')
 do mterm=startexp,maxterm,step
   lim1=mterm
   lim2=mterm+step-1
   if(lim2>maxterm)lim2=maxterm 
!  
   if(found)cycle   !if already found, then
!  just cycle
!  Find the worst trial guess
   if(mterm>2 .and. ntrial-nskip >3) then
!    order by error
     iord(:)=(/(ii, ii=1,ntrial)/)
     call sort_dp(ntrial,error_trial,iord(:),tol12)
!    
!    reverse
     iord = iord(ubound(iord,1):lbound(iord,1):-1)
!    
!    message='sep   Sum(chisq)'
!    call wrtout(std_out,message,'COLL')
!    do ii=ntrial,1,-1
!    if(skip(iord(ii)) .eqv. .false.) write(std_out,*)sep(iord(ii)),error_trial(ii)
!    end do
     if(nskip==0 ) then !first_time, eliminate 3 trials
       nskip=nskip+3
     else  !eliminate 1 by one
       nskip=nskip+1
     end if
     skip(iord(1:nskip))=.true.
!    write(std_out,fmt='(a)',advance='no')'Skipped: '
!    do ii=1,ntrial
!    if(skip(ii) .eqv. .true.)  &
!    &write(std_out,fmt='(f6.4,x)', advance='no')sep(ii)
!    end do
!    write(std_out,*)
     error_trial=0.d0 
!    error_trial(iord(1:nskip))="Infinity" !only for ifort
     error_trial(iord(1:nskip))=5000
   end if
!  
   if(mterm>=10 .and. ntrial>=5) then
     skip(:)=.true.
     do ii=1,5
       sep(ii)=1.0d0+real(ii,dp)*0.05d0
       skip(ii)=.false.
     end do
   end if
!  
   do ii=1,ntrial
     if(skip(ii)) cycle
     nterm_cycle: do nterm=lim1,lim2
!      
       if(option==1) nparam=nterm*4
       if(option==2) nparam=nterm*6
       if(option==3) nparam=nterm*2
       if(option==4) nparam=nterm*4
!      
!      set initial guess 
       if(option==1) then
         call setparam3(nterm,nparam,param_tmp(1:nparam),sep(ii))
       elseif(option==2) then
         call setparam1(nterm,nparam,nr,&
&         param_tmp(1:nparam),sep(ii),pawrad%rad(1:nr),y)
       elseif(option==3) then
         call setparam4(nparam,param_tmp(1:nparam))
       elseif(option==4) then
         call setparam5(nterm,nparam,nr,&
&         param_tmp(1:nparam),rpaw,y)
       end if
!      
!      
       call constrains_ini(accuracy,weight(1:nparam),constrains(1:nparam),&
&       limit(1:nparam),nparam,nterm,nr,option,rpaw,y)
!      
       call fit(accuracy,chisq,constrains(1:nparam),&
&       limit(1:nparam),maxiter,nparam,nterm,nr,option,outfile,param_tmp,&
&       verbosity,weight(1:nparam),pawrad%rad(1:nr),y,y_out)


!      
!      if there was an error, cycle
       if(abs(chisq+1.d0)<tol8) cycle
       if(abs(chisq)==chisq*chisq)cycle
       if(chisq .ne. chisq) cycle !if it is a NaN
!      
       error_trial(ii)=error_trial(ii)+chisq
!      
       write(message,'(f6.3,x,i4,2x,2(e13.6,x))')sep(ii),nterm,chisq,error
       call wrtout(std_out,message,'COLL')
       if(chisq<error) then
         error=chisq
         minsep=sep(ii)
         minterm=nterm
       end if
       if(chisq>accuracy)cycle
       if(first) then !first time
         minsep=sep(ii)
         minterm=nterm
         lim2=minterm
         error=chisq
         first=.false.
         found=.true.
         exit nterm_cycle
       elseif(nterm<minterm) then
         minsep=sep(ii)
         minterm=nterm
         lim2=minterm
         error=chisq
         exit nterm_cycle
       elseif(nterm==minterm) then
         if(chisq< error) then
           minsep=sep(ii)
           error=chisq
           exit nterm_cycle
         end if 
       end if
     end do nterm_cycle !nterm
   end do !sep
 end do !maxterm
 if(found .eqv. .false.) then
   write(message,'(2a)')"WARNING: the required accuracy was not reached",ch10
   call wrtout(std_out,message,'COLL')
 end if
 write(message,'(a)')"Best parameters:"
 call wrtout(std_out,message,'COLL')
 write(message,'("sep= ",f4.2,x,&
& "nterm= ",i5,x,"chisq= ",e13.6)')minsep,minterm,error
 call wrtout(std_out,message,'COLL')
!
!run again with the best parameters
 nterm=minterm
 if(option==1)nparam=4*nterm
 if(option==2)nparam=6*nterm
 if(option==3)nparam=2*nterm
 if(option==4)nparam=4*nterm
 
!set initial guess 
 if(option==1) then 
   call setparam3(nterm,nparam,param_tmp(1:nparam),minsep)
 elseif (option==2) then
   call setparam1(nterm,nparam,nr,&
&   param_tmp(1:nparam),minsep,pawrad%rad(1:nr),y)
 elseif (option==3) then
   call setparam4(nparam,param_tmp(1:nparam))
 elseif (option==4) then
   call setparam5(nterm,nparam,nr,&
&   param_tmp(1:nparam),rpaw,y)
 end if
!
 call constrains_ini(accuracy,weight(1:nparam),constrains(1:nparam),&
& limit(1:nparam),nparam,nterm,nr,option,rpaw,y)
!
 verbosity=1
 call fit(accuracy,chisq,constrains(1:nparam),&
& limit(1:nparam),maxiter,nparam,nterm,nr,option,outfile,param_tmp,&
& verbosity,weight(1:nparam),pawrad%rad(1:nr),y,y_out)
 
!Fill out the output variables:
 if(option==1) then
   nparam_out=minterm*4
 elseif (option==2) then
   nparam_out=minterm*6
 elseif (option==3) then
   nparam_out=minterm*2
 elseif (option==4) then
   nparam_out=minterm*4
 end if
 param_out(:)=param_tmp
 
 y=y_out !at output modify y for the fitted y
 
 ABI_DEALLOCATE(y_out)
 ABI_DEALLOCATE(error_trial)
 ABI_DEALLOCATE(iord)
 ABI_DEALLOCATE(skip)


!DEBUG
!write (std_out,*) ' fit_gen : exit'
!stop
!ENDDEBUG

end subroutine fit_gen
!!***

!!****f* ABINIT/fit
!! NAME
!!  fit
!!
!! FUNCTION
!!  Fits a given input function f(r) to a sum of gaussians
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  accuracy= Wanted accuracy. 
!!  chisq= It measures how good is the fitting.
!!   It is defined here as sum_i^N_i f(x_i)-y(x_i)/N_i
!!  constrains= constraints for Gaussians
!!  limit(nparam)= it limits the widths of Gaussians
!!  maxiter=maximum number of iterations
!!  nparam= number of parameters (a constant times nterm)
!!         if(option==1)nparam=nterm*4
!!         if(option==2)nparam=nterm*6
!!         if(option==3)nparam=nterm*2
!!         if(option==4)nparam=nterm*4
!!  nterm= number of Gaussians 
!!  nx=number of points along the x axis
!!  option=1 fit to a1 cos(a2 x^2)+ a3 sin( a4 x^2)
!!         2 fit to a1 exp(-a2 x^2)*(a3 cos (a4 x^2) + a5 sin (a6 x^2) )
!!         3 fit to a1 cos (k x^2) + a2 sin (k x^2)
!!         4 fit to a1 exp(-a2 x^2)* (a3 cos(k x^2)+ a4 sin (k x^2))
!!  outfile= filename for output (only written if verbosity>1)
!!  verbosity= controls output volume
!!  weight(nparam)= weights for the fitting procedure
!!  x(nx)= points along the x axis
!!  y(nx)= function to be fitted
!!  rpaw ,optional= paw radius
!!
!! OUTPUT
!! y_out(nx)= fitted function
!!
!! SIDE EFFECTS
!! if(verbosity>1) output files are written with y(x) and y_out(x)
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine fit(accuracy,chisq,constrains,&
& limit,maxiter,nparam,nterm,nx,option,outfile,param,&
& verbosity,weight,x,y,y_out)

 use defs_basis
 use m_profiling

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fit'
 use interfaces_64_atompaw, except_this_one => fit
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer, intent(in) :: maxiter,nparam
 integer,intent(in)  :: nterm,nx,option,verbosity
 real(dp),intent(in) :: accuracy
 !real(dp),optional,intent(in)::rpaw
 !arrays
 integer,intent(in)::constrains(nparam)
 real(dp),intent(in)::limit(nparam),weight(nparam)
 real(dp),intent(in)::x(nx),y(nx)
 real(dp),intent(inout)::param(nparam)
 real(dp),intent(out)::chisq,y_out(nx)
 character(80),intent(in)::outfile
!----------------------------------------
!local variables
!----------------------------------------
 integer::ix
 real(dp)::rerror
 !
 real(dp),allocatable::sy(:)
 integer, parameter :: wfn_unit=1007
!
! *************************************************************************

 ABI_ALLOCATE(sy,(nx))
!
 sy(:)=1.0d0
!
 call RestreinedLeastSquaresFit(&
& accuracy,chisq,constrains,limit,maxiter,&
& nterm,nparam,nx,option,param(1:nparam),&
& verbosity,weight,x,y)
!
 if(verbosity>=1) then
   if(option==1) then
     call calcgaussc2(nparam,nterm,nx,1,param,x,y_out)
   elseif(option==2) then
     call calcgaussc(nparam,nterm,nx,1,param,x,y_out)
   elseif(option==3) then
     call calcgaussc3(nparam,nterm,nx,1,param,x,y_out)
   elseif(option==4) then
     call calcgaussc4(nparam,nterm,nx,1,param,x,y_out)
   end if
!  
!  
   open(wfn_unit,file=outfile,form='formatted',status='unknown')
!  per_error=0.d0
   do ix=1, nx
     rerror=abs(y(ix)-y_out(ix))
     write(wfn_unit,'(6(e17.12,x))')x(ix),y(ix),y_out(ix),rerror
   end do
   close(wfn_unit)
!  
 end if
!
 ABI_DEALLOCATE(sy)
end subroutine fit
!!***

!!****f* ABINIT/calcgaussr
!! NAME
!!  calcgaussr
!!
!! FUNCTION
!!  Calculate derivatives for Gaussians
!!   Only relevant for fitting Gaussians algorithm.
!!   The Gaussians expressions are defined in the comments of "fit_gen"
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

subroutine calcgaussr(nterm,nparam,nx,opt,&
&param,x,y_out,deriv)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calcgaussr'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nx     !number of point in the x grid
 integer,intent(in)::nparam !number of parameters
 integer,intent(in)::nterm !number of gaussian expressions
 integer,intent(in)::opt    !option:
                            !1) calculate only f(x)
                            !2) calculate f(x) and its derivatives
 real(dp),intent(in)::param(nparam) !parameters
 real(dp),intent(in)::x(nx) !xgrid
 real(dp),intent(out)::y_out(nx) !f(x)
 real(dp),optional,intent(out)::deriv(nx,nparam) !derivatives

!Local variables-------------------------------
 integer::iexp,ii
 real(dp)::alpha1(nterm),alpha2(nterm),alpha3(nterm)
 real(dp)::term1(nx,nterm)
 real(dp)::aux1(nx)
 !real(dp)::step
! *********************************************************************

!
!Initialize
!
 y_out(:)=0.d0
!
!Get parameters from parameters array:
!
 alpha1(:)=param(1:nterm)
 alpha2(:)=param(nterm+1:2*nterm)
 alpha3(:)=param(2*nterm+1:3*nterm)
!
!alpha3
!set to constant values of x
!step=rpaw/real(nterm,dp)
!do ii=1,nterm
!raux=step*real(ii,dp)
!alpha3(ii)=raux
!end do
!
!
!
!calculate useful quantities
!
 do iexp=1,nterm
   aux1(:)=-alpha2(iexp)*(x(:)-alpha3(iexp))**2
   term1(:,iexp)=alpha1(iexp)*exp(aux1(:))
 end do
!
 do iexp=1,nterm
   y_out(:)=y_out(:)+term1(:,iexp)
 end do
!
!Calculate derivatives:
!
 if(opt==2) then
!  
!  alpha1
!  
   do iexp=1,nterm
     aux1(:)=term1(:,iexp)/alpha1(iexp)
     deriv(:,iexp)=aux1(:)
   end do
!  
!  alpha2
!  
   do iexp=1,nterm
     ii=nterm+iexp
     aux1(:)=-term1(:,iexp)*(x(:)-alpha3(iexp))
     deriv(:,ii)=aux1(:)
     deriv(:,ii)=0.1d0
   end do
!  
!  alpha3
!  
   do iexp=1,nterm
     ii=2*nterm+iexp
     aux1(:)=term1(:,iexp)*2.d0*alpha2(iexp)
     aux1(:)=aux1(:)*(x(:)-alpha3(iexp))
     deriv(:,ii)=aux1(:)
   end do
 end if
end subroutine calcgaussr
!!***

!!****f* ABINIT/calcgaussc3
!! NAME
!!  calcgaussc3
!!
!! FUNCTION
!!  Calculate expressions and derivatives for Gaussians fitting.
!!   The Gaussians expressions are defined in the comments of "fit_gen"
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

subroutine calcgaussc3(nparam,nterm,nx,opt,&
&param,x,y_out,deriv)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calcgaussc3'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nparam !number of parameters
 integer,intent(in)::nterm !number of gaussian expressions
 integer,intent(in)::nx     !number of point in the x grid
 integer,intent(in)::opt    !option:
                            !1) calculate only f(x)
                            !2) calculate f(x) and its derivatives
 real(dp),intent(in)::param(nparam) !parameters
 real(dp),intent(in)::x(nx) !xgrid
 real(dp),intent(out)::y_out(nx) !f(x)
 real(dp),optional,intent(out)::deriv(nx,nparam) !derivatives

!Local variables-------------------------------
 integer::iexp,ii
 real(dp)::sep
 real(dp)::alpha1(nterm),alpha2(nterm),alpha3(nterm)
 real(dp)::term1(nx,nterm)
 real(dp)::sin1(nx,nterm),cos1(nx,nterm)
 real(dp)::aux1(nx),aux2(nx)
! *********************************************************************

!
!Initialize
!
 y_out(:)=0.d0
!
 sep=1.2d0
!
!Get param from param array:
!
 alpha1(:)=param(1:nterm)
 alpha2(:)=param(nterm+1:2*nterm)
!
 do ii=1,nterm
   alpha3(ii)=sep**(ii)
 end do
!
!calculate useful quantities
!
 do iexp=1,nterm
   aux1(:)=alpha3(iexp)*x(:)**2
!  
   sin1(:,iexp)=sin(aux1(:))
   cos1(:,iexp)=cos(aux1(:))
 end do
!
 do iexp=1,nterm
   aux1(:)=alpha1(iexp)*sin1(:,iexp)
   aux2(:)=alpha2(iexp)*cos1(:,iexp)
   term1(:,iexp)=aux1(:)+aux2(:)
   y_out(:)=y_out(:)+term1(:,iexp)
 end do
!
!Calculate derivatives:
!
 if(opt==2) then
!  
!  alpha1
!  
   do iexp=1,nterm
     deriv(:,iexp)=sin1(:,iexp)
   end do
!  
!  alpha2
!  
   do iexp=1,nterm
     ii=nterm+iexp
     deriv(:,ii)=cos1(:,iexp)
   end do
 end if
end subroutine calcgaussc3
!!***

!!****f* ABINIT/calcgaussc2
!! NAME
!!  calcgaussc2
!!
!! FUNCTION
!!  Calculate expressions and derivatives for Gaussians fitting.
!!   The Gaussians expressions are defined in the comments of "fit_gen"
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE
subroutine calcgaussc2(nparam,nterm,nx,opt,&
&param,x,y_out,deriv)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calcgaussc2'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nparam !number of param
 integer,intent(in)::nterm  !number of gaussian expressions
 integer,intent(in)::nx     !number of point in the x grid
 integer,intent(in)::opt    !option:
                            !1) calculate only f(x)
                            !2) calculate f(x) and its derivatives
 real(dp),intent(in)::param(nparam) !parameters
 real(dp),intent(in)::x(nx) !xgrid
 real(dp),intent(out)::y_out(nx) !f(x)
 real(dp),optional,intent(out)::deriv(nx,nparam) !derivatives

!Local variables-------------------------------
 integer::iexp,ii
 real(dp)::alpha1(nterm),alpha2(nterm),alpha3(nterm)
 real(dp)::alpha4(nterm)
 real(dp)::term1(nx,nterm)
 real(dp)::sin1(nx,nterm),sin2(nx,nterm),cos1(nx,nterm),cos2(nx,nterm)
 real(dp)::aux1(nx),aux2(nx)
! *********************************************************************

!
!Initialize
!
 y_out(:)=0.d0
!
!Get param from param array:
!
 alpha1(:)=param(1:nterm)
 alpha2(:)=param(nterm+1:2*nterm)
 alpha3(:)=param(2*nterm+1:3*nterm)
 alpha4(:)=param(3*nterm+1:4*nterm)
!
!calculate useful quantities
!
!
 do iexp=1,nterm
   aux1(:)=alpha2(iexp)*x(:)**2
   sin1(:,iexp)=sin(aux1(:))
!  
   aux1(:)=alpha4(iexp)*x(:)**2
   sin2(:,iexp)=sin(aux1(:))
!  
   aux1(:)=alpha2(iexp)*x(:)**2
   cos1(:,iexp)=cos(aux1(:))
!  
   aux1(:)=alpha4(iexp)*x(:)**2
   cos2(:,iexp)=cos(aux1(:))
 end do
!
 do iexp=1,nterm
   aux1(:)=alpha1(iexp)*sin1(:,iexp)
   aux2(:)=alpha3(iexp)*cos2(:,iexp)
   term1(:,iexp)=aux1(:)+aux2(:)
   y_out(:)=y_out(:)+term1(:,iexp)
 end do
!
!Calculate derivatives:
!
 if(opt==2) then
!  
!  alpha1
!  
   do iexp=1,nterm
     deriv(:,iexp)=sin1(:,iexp)
   end do
!  
!  alpha2
!  
   do iexp=1,nterm
     ii=nterm+iexp
     aux1(:)=alpha1(iexp)*cos1(:,iexp)*x(:)**2
     deriv(:,ii)=aux1(:)
   end do
!  
!  alpha3
!  
   do iexp=1,nterm
     ii=2*nterm+iexp
     deriv(:,ii)=cos2(:,iexp)
   end do
!  
!  alpha4
!  
   do iexp=1,nterm
     ii=3*nterm+iexp
     aux1(:)=-alpha3(iexp)*sin2(:,iexp)*x(:)**2
     deriv(:,ii)=aux1(:)
   end do
 end if
end subroutine calcgaussc2
!!***

!!****f* ABINIT/calcgaussc
!! NAME
!!  calcgaussc
!!
!! FUNCTION
!!  Calculate expressions and derivatives for Gaussians fitting.
!!   The Gaussians expressions are defined in the comments of "fit_gen"
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE
!
subroutine calcgaussc(nparam,nterm,nx,opt,&
&param,x,y_out,deriv)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calcgaussc'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nparam !number of parameters
 integer,intent(in)::nterm !number of gaussian expressions
 integer,intent(in)::nx     !number of point in the x grid
 integer,intent(in)::opt    !option:
                            !1) calculate only f(x)
                            !2) calculate f(x) and its derivatives
 real(dp),intent(in)::param(nparam) !parameters
 real(dp),intent(in)::x(nx) !xgrid
 real(dp),intent(out)::y_out(nx) !f(x)
 real(dp),optional,intent(out)::deriv(nx,nparam) !derivatives

!Local variables-------------------------------
 integer::iexp,ii
 real(dp)::alpha1(nterm),alpha2(nterm),alpha3(nterm)
 real(dp)::alpha4(nterm),alpha5(nterm),alpha6(nterm)
 real(dp)::aux1(nx),aux2(nx)
 real(dp)::cos1(nx,nterm),cos2(nx,nterm),sin1(nx,nterm),sin2(nx,nterm)
 real(dp)::term1(nx,nterm),term2(nx,nterm)
! *********************************************************************

!
!Initialize
!
 y_out(:)=0.d0
!
!Get parameters from param array:
!
 alpha1(:)=param(1:nterm)
 alpha2(:)=param(nterm+1:2*nterm)
 alpha3(:)=param(2*nterm+1:3*nterm)
 alpha4(:)=param(3*nterm+1:4*nterm)
 alpha5(:)=param(4*nterm+1:5*nterm)
 alpha6(:)=param(5*nterm+1:6*nterm)
!
!calculate useful quantities
!
 do iexp=1,nterm
   aux1(:)=-alpha2(iexp)*x(:)**2
   term1(:,iexp)=alpha1(iexp)*exp(aux1(:))
 end do
!
 do iexp=1,nterm
   aux1(:)=alpha4(iexp)*x(:)**2
   sin1(:,iexp)=sin(aux1(:))
!  
   aux1(:)=alpha6(iexp)*x(:)**2
   sin2(:,iexp)=sin(aux1(:))
!  
   aux1(:)=alpha4(iexp)*x(:)**2
   cos1(:,iexp)=cos(aux1(:))
!  
   aux1(:)=alpha6(iexp)*x(:)**2
   cos2(:,iexp)=cos(aux1(:))
 end do
!
 do iexp=1,nterm
   aux1(:)=alpha3(iexp)*sin1(:,iexp)
   aux2(:)=alpha5(iexp)*cos2(:,iexp)
   term2(:,iexp)=aux1(:)+aux2(:)
   y_out(:)=y_out(:)+term1(:,iexp)*term2(:,iexp)
 end do
!
!Calculate derivatives:
!
 if(opt==2) then
!  
!  alpha1
!  
   do iexp=1,nterm
     aux1(:)=term1(:,iexp)/alpha1(iexp)
     aux2(:)=aux1(:)*term2(:,iexp)
     deriv(:,iexp)=aux2(:)
   end do
!  
!  alpha2
!  
   do iexp=1,nterm
     ii=nterm+iexp
     aux1(:)=-term1(:,iexp)*term2(:,iexp)
     aux2(:)=aux1(:)*x(:)**2
     deriv(:,ii)=aux2(:)
   end do
!  
!  alpha3
!  
   do iexp=1,nterm
     ii=2*nterm+iexp
     aux1(:)=term1(:,iexp)*sin1(:,iexp)
     deriv(:,ii)=aux1(:)
   end do
!  
!  alpha4
!  
   do iexp=1,nterm
     ii=3*nterm+iexp
     aux1(:)=term1(:,iexp)*alpha3(iexp)
     aux2(:)=cos1(:,iexp)*x(:)**2
     deriv(:,ii)=aux2(:)*aux1(:)
   end do
!  
!  alpha5
!  
   do iexp=1,nterm
     ii=4*nterm+iexp
     aux1(:)=term1(:,iexp)*cos2(:,iexp)
     deriv(:,ii)=aux1(:)
   end do
!  
!  alpha6
!  
   do iexp=1,nterm
     ii=5*nterm+iexp
     aux1(:)=-term1(:,iexp)*alpha5(iexp)
     aux2(:)=sin2(:,iexp)*x(:)**2
     deriv(:,ii)=aux1(:)*aux2(:)
   end do
 end if
end subroutine calcgaussc
!!***

!!****f* ABINIT/calcgaussc4
!! NAME
!!  calcgaussc4
!!
!! FUNCTION
!!  Calculate expressions and derivatives for Gaussians fitting.
!!   The Gaussians expressions are defined in the comments of "fit_gen"
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE
subroutine calcgaussc4(nparam,nterm,nx,opt,&
&param,x,y_out,deriv)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calcgaussc4'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nparam !number of parameters
 integer,intent(in)::nterm !number of gaussian expressions
 integer,intent(in)::nx     !number of point in the x grid
 integer,intent(in)::opt    !option:
                            !1) calculate only f(x)
                            !2) calculate f(x) and its derivatives
 real(dp),intent(in)::param(nparam) !parameters
 real(dp),intent(in)::x(nx) !xgrid
 real(dp),intent(out)::y_out(nx) !f(x)
 real(dp),optional,intent(out)::deriv(nx,nparam) !derivatives

!Local variables-------------------------------
 integer::iexp,ii
 real(dp)::raux,sep
 real(dp)::alpha1(nterm),alpha2(nterm),alpha3(nterm)
 real(dp)::alpha4(nterm),alpha5(nterm)
 real(dp)::aux1(nx),aux2(nx)
 real(dp)::cos1(nx,nterm),sin1(nx,nterm)
 real(dp)::term1(nx,nterm),term2(nx,nterm)
! *********************************************************************

!
!Initialize
!
 sep=1.1d0
 y_out(:)=0.d0
 
!Get parameters from param array:
!
 alpha1(:)=param(1:nterm)
 alpha2(:)=param(nterm+1:2*nterm)
 alpha3(:)=param(2*nterm+1:3*nterm)
 alpha4(:)=param(3*nterm+1:4*nterm)
!
!
 raux=(2.d0*pi)/real(nterm,dp)
 do ii=1,nterm
   alpha5(ii)=sep**(ii)
!  alpha5(ii)=raux*real(ii-1,dp)
 end do
!
!calculate useful quantities
!
 do iexp=1,nterm
   aux1(:)=-alpha2(iexp)*x(:)**2
   term1(:,iexp)=alpha1(iexp)*exp(aux1(:))
 end do
!
 do iexp=1,nterm
   aux1(:)=alpha5(iexp)*x(:)**2
!  
   sin1(:,iexp)=sin(aux1(:))
   cos1(:,iexp)=cos(aux1(:))
 end do
!
 do iexp=1,nterm
   aux1(:)=alpha3(iexp)*sin1(:,iexp)
   aux2(:)=alpha4(iexp)*cos1(:,iexp)
   term2(:,iexp)=aux1(:)+aux2(:)
   y_out(:)=y_out(:)+term1(:,iexp)*term2(:,iexp)
 end do
!
!Calculate derivatives:
!
 if(opt==2) then
!  
!  alpha1
!  
   do iexp=1,nterm
     aux1(:)=term1(:,iexp)/alpha1(iexp)
     aux2(:)=aux1(:)*term2(:,iexp)
     deriv(:,iexp)=aux2(:)
   end do
!  
!  alpha2
!  
   do iexp=1,nterm
     ii=nterm+iexp
     aux1(:)=-term1(:,iexp)*term2(:,iexp)
     aux2(:)=aux1(:)*x(:)**2
     deriv(:,ii)=aux2(:)
   end do
!  
!  alpha3
!  
   do iexp=1,nterm
     ii=2*nterm+iexp
     aux1(:)=term1(:,iexp)*sin1(:,iexp)
     deriv(:,ii)=aux1(:)
   end do
!  
!  alpha4
!  
   do iexp=1,nterm
     ii=3*nterm+iexp
     aux1(:)=term1(:,iexp)*cos1(:,iexp)
     deriv(:,ii)=aux1(:)
   end do
 end if
end subroutine calcgaussc4
!!***

!!****f* ABINIT/RestreinedLeastSquaresFit
!! NAME
!!  RestreinedLeastSquaresFit
!!
!! FUNCTION
!!  Fits a given function to a sum of Gaussians.
!!  Uses the Levenberg-Marquardt algorithm.
!!
!!  
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  The original Levemberg Marquardt routines were written by Armando Sole
!!  These were modified for the ARPUS spectra in the BigDFT code by A. Mirone.
!!  These were re-writen in Fortran and further modified in ABINIT for our particular needs.
!!
!! INPUTS
!!  option=1 fit to a1 cos(a2 x^2)+ a3 sin( a4 x^2)
!!         2 fit to a1 exp(-a2 x^2)*(a3 cos (a4 x^2) + a5 sin (a6 x^2) )
!!         3 fit to a1 cos (k x^2) + a2 sin (k x^2)
!!         4 fit to a1 exp(-a2 x^2)* (a3 cos(k x^2)+ a4 sin (k x^2))
!!  if(option==1)mparam=maxterm*4
!!  if(option==2)mparam=maxterm*6
!!  if(option==3)mparam=maxterm*2
!!  if(option==4)mparam=maxterm*4
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine RestreinedLeastSquaresFit(&
&accuracy,chisq,constrains,limit,maxiter,&
&nterm,nparam,nx,option,parameters,&
&verbosity,weight,x,y)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'RestreinedLeastSquaresFit'
 use interfaces_14_hidewrite
 use interfaces_64_atompaw, except_this_one => RestreinedLeastSquaresFit
!End of the abilint section

 implicit none

!Arguments -------------------------------
 real(dp),parameter::deltachi=tol10
 !
 integer, intent(in) ::maxiter,nparam,nterm,nx
 integer, intent(in) ::option ,verbosity
 integer, intent(in) ::constrains(nparam)
 real(dp),intent(in) ::accuracy
 real(dp),intent(out)::chisq
 !arrays
 real(dp),intent(in)::limit(nparam),weight(nparam)
 real(dp),intent(inout)::parameters(nparam)
 real(dp),intent(in)::x(nx),y(nx)

!Local variables-------------------------------
 integer::flag,ii,info,iter,jj,niter
 real(dp)::chisq0,flambda,eta,lastdeltachi
 !
 integer::ipvt(nparam)
 real(dp)::alpha(nparam,nparam)
 real(dp)::alpha0(nparam,nparam),beta(nparam)
 real(dp)::deltapar(nparam)
 real(dp)::tmp1(nparam,nparam)
 real(dp)::work(nparam)
 real(dp)::workpar(nparam)
 real(dp)::yfit(nx)
 character(len=500) :: message
! *********************************************************************

!
 flambda=1e-6
 if(accuracy<1e-5) flambda=1e-7
!iter=maxiter !later it is changed
 niter=0
!
 iter_loop: do iter=1,maxiter
!  
   call ChisqAlphaBeta(alpha0,beta,chisq0,&
&   nparam,nterm,nx,option,parameters,x,y)
!  
!  debug
!  write(*, '("B ",f20.7)')chisq0
!  call exit(1)
   flag=0
   lastdeltachi=chisq0
!  
!  
   while_flag: do
     if(flag .ne. 0) exit while_flag
!    
     tmp1=0.d0
     do ii=1,nparam  
       tmp1(ii,ii)=1.d0*flambda  !identity matrix * flambda
     end do
     alpha=alpha0+tmp1*alpha0
!    Invert alpha matrix
     tmp1=alpha
     call dgetrf(nparam,nparam,tmp1,nparam,ipvt,info)
     if (.not.info==0) then
       if(verbosity>1) then
         write(message,'(a)')'Matrix is singular'
         call wrtout(std_out,message,'COLL')
       end if
       chisq=-1.d0
       exit iter_loop
     end if
     call dgetri(nparam,tmp1,nparam,ipvt,work,nparam,info)
     deltapar=0.d0
     if (.not.info==0) then
       if(verbosity>2) then
         write(message,'(a)')'Matrix is singular'
         call wrtout(std_out,message,'COLL')
       end if
       chisq=-1.d0
       exit iter_loop
     end if
!    
     if(tmp1(1,1) .ne. tmp1(1,1)) then !If is NaN
       chisq=-1.d0
       exit iter_loop
     end if
     if(abs(tmp1(1,1)) == tmp1(1,1)*tmp1(1,1)) then !If is infinity
       chisq=-1.d0
       exit iter_loop
     end if
!    
     do ii=1,nparam
       do jj=1,nparam
         deltapar(ii)=deltapar(ii)+beta(jj)*tmp1(jj,ii)
       end do
     end do 
!    
     workpar(1:nparam)=parameters(1:nparam)+deltapar(1:nparam)*weight(1:nparam)
!    apply constrains
     call get_parameters(constrains,limit,nparam,workpar,workpar)
!    
     if(option==1) then
       call calcgaussc2(nparam,nterm,nx,1,workpar,x,yfit)
     elseif(option==2) then
       call calcgaussc(nparam,nterm,nx,1,workpar,x,yfit)
     elseif(option==3) then
       call calcgaussc3(nparam,nterm,nx,1,workpar,x,yfit)
     elseif(option==4) then
       call calcgaussc4(nparam,nterm,nx,1,workpar,x,yfit)
     end if
     chisq=0.d0
     do ii=1,nx
       chisq=chisq + ((y(ii)-yfit(ii)))**2
     end do
     chisq=chisq/real(nx,dp)
!    
!    write(*,'("chisq ",f12.5,"  chisq0 ",f12.5)')chisq,chisq0
!    
     if(chisq > chisq0) then
       flambda=flambda*2.0d0
       if( flambda > 1000.d0) then
         flag=1
!        iter=0
         if(verbosity>2) then
           write(message,'(a)')'flambda > 1000.d0'
           call wrtout(std_out,message,'COLL')
         end if
         exit iter_loop
       end if
     else
       flag=1
       parameters=workpar
       eta=0.d0
!      if (abs(chisq0)<accuracy) eta=1.d0
       lastdeltachi=(chisq0-chisq) !/(chisq0+eta)
       if(lastdeltachi<deltachi) cycle
       if(abs(chisq)<=accuracy) exit iter_loop
       chisq0=chisq
       flambda=flambda/2.d0
       if(verbosity>2) then
         write(message,'("iter = ",i4," chisq = ",e15.6)')iter,chisq
         call wrtout(std_out,message,'COLL')
       end if
     end if
   end do while_flag
 end do iter_loop
!
end subroutine RestreinedLeastSquaresFit
!!***

!!****f* ABINIT/ChisqAlphaBeta
!! NAME
!!  ChisqAlphaBeta
!!
!! FUNCTION
!!  Finds chisq, alpha and beta parameters for LSF using the Levenberg-Marquardt algorithm.
!!
!!  
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  The original Levemberg Marquardt routines were written by Armando Sole
!!  These were modified for the ARPUS spectra in the BigDFT code by A. Mirone.
!!  These were re-writen in Fortran and further modified in ABINIT for our particular needs.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE
subroutine ChisqAlphaBeta(alpha,beta,chisq,&
& nparam,nterm,nx,option,parameters,x,y)

 use defs_basis
 !

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'ChisqAlphaBeta'
 use interfaces_64_atompaw, except_this_one => ChisqAlphaBeta
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nparam,nterm,nx
 integer,intent(in)::option 
 real(dp),intent(out)::chisq
 real(dp),intent(in)::parameters(nparam),x(nx),y(nx)
 real(dp),intent(out)::alpha(nparam,nparam),beta(nparam)

!Local variables-------------------------------
 integer::ii,jj,kk
 real(dp)::help1
 !arrays
 real(dp)::deltay(nx),deriv(nx,nparam),derivi(nx)
 real(dp)::yfit(nx)
 real(dp)::help0(nx),help2(nx),help3(nparam)
! *********************************************************************

!
 if(option==1) then
   call calcgaussc2(nparam,nterm,nx,2,parameters,x,yfit,deriv)
 elseif(option==2) then
   call calcgaussc(nparam,nterm,nx,2,parameters,x,yfit,deriv)
 elseif(option==3) then
   call calcgaussc3(nparam,nterm,nx,2,parameters,x,yfit,deriv)
 elseif(option==4) then
   call calcgaussc4(nparam,nterm,nx,2,parameters,x,yfit,deriv)
 end if
 deltay=y-yfit
 help0=deltay
!
 do ii=1,nparam
   derivi(:)=deriv(:,ii)
   help1=0.d0
   do jj=1,nx
     help1=help1+help0(jj)*derivi(jj)
   end do
   beta(ii)=help1
!  help1 = innerproduct(deriv,weight*derivi)
!  below I use help3 instead for the array dimenstions
   help3=0.d0
   do kk=1,nparam
     do jj=1,nx
       help3(kk)=help3(kk)+deriv(jj,kk)*derivi(jj)
     end do
   end do
!  !
   alpha(:,ii)=help3(:)
 end do
!
 help2(:)=help0(:)*deltay(:)
 chisq=sum(help2)
 chisq=chisq/real(nx,dp)
end subroutine ChisqAlphaBeta
!!***

!!****f* ABINIT/setparam1
!! NAME
!!  setparam1
!!
!! FUNCTION
!!  Sets parameters for LSF
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

subroutine setparam1(nterm,nparam,nx,param,sep,x,y)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'setparam1'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nterm,nparam,nx
 real(dp),intent(in)::sep
 real(dp),intent(in)::x(nx),y(nx)
 real(dp),intent(out)::param(nparam)

!Local variables-------------------------------
 integer::ii,jj
 real(dp)::raux
! *********************************************************************

!
 param(:)=1.0d0
!exps=1.0/(x(nx)**2)
!
!alpha1
 
!raux=maxval( y(:),nx )
!maxval gives problems in some architectures:
 raux=-9999999
 do ii=1,nx
   if(raux<y(ii)) raux=y(ii)
 end do
 param(1:nterm)=raux
!
!alpha2
!
!y(r_c)=e^{-\alpha1 r_c}
 raux=-log(abs(y(nx))+tol10)/(x(nx)**2)
 param(nterm+1:nterm+2)=raux
!
!raux=0.5d0*pi/real(nterm,dp)
 do jj=1,nterm
   ii=jj+3*nterm
   param(ii)=sep**(jj)
!  param(ii)=raux*real(jj,dp)
 end do
!
 do jj=1,nterm
   ii=jj+5*nterm
   param(ii)=sep**(jj)
!  param(ii)=raux*real(jj,dp)
 end do
end subroutine setparam1
!!***

!!****f* ABINIT/setparam2
!! NAME
!!  setparam2
!!
!! FUNCTION
!!  Sets parameters for LSF
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
subroutine setparam2(nterm,nparam,nx,param,rpaw,x,y)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'setparam2'
 use interfaces_64_atompaw, except_this_one => setparam2
!End of the abilint section

 implicit none

 integer,intent(in)::nterm,nparam,nx
 real(dp),intent(in)::rpaw
 real(dp),intent(in)::x(nx),y(nx)
 real(dp),intent(out)::param(nparam)
 !
 integer::ii,jj
 real(dp)::exps,raux,sig,step
! *************************************************************************

!
 step=rpaw/real(nterm-1,dp)
!exps=1.0/(rpaw/(real(nterm,dp)/2.d0))**2
 exps=1.0/(step*1.0d0)**2
!alpha2 (width of gaussians)
!Set to exps*real(nterm,dp)**2, for a good guess
 param(nterm+1:2*nterm)=exps !*real(nterm,dp)**2
!
 do jj=1,nterm
!  alpha3
!  set to constant values of x
   ii=jj+2*nterm
   raux=step*real(jj-1,dp)
   param(ii)=raux
!  alpha1
!  set to the value of y at that point
   call findsign()
   param(jj)=sig
 end do
!
 contains
!!***

!!****f* setparam2/findsign
!! NAME
!!  findsign
!!
!! FUNCTION
!!  Finds the value of y at a given point
!!  This was taken out of setparam2 to make the code more
!!  readable.
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE
 subroutine findsign()

!Arguments -------------------------------

!Local variables-------------------------------

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'findsign'
!End of the abilint section

  integer::ix,minx
  real(dp)::dist,mindist,xx,yy
! *********************************************************************

   mindist=rpaw
   do ix=1,nx
     xx=x(ix)
     dist=abs(raux-xx)
     if(dist<mindist) then
       mindist=dist
       minx=ix
     end if
   end do
   yy=y(minx)
   sig=yy
 end subroutine findsign

end subroutine setparam2
!!***

!!****f* ABINIT/setparam3
!! NAME
!!  setparam3
!!
!! FUNCTION
!!  Sets parameters for LSF
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

subroutine setparam3(nterm,nparam,param,sep)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'setparam3'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nterm,nparam
 real(dp),intent(in)::sep
 !real(dp),intent(in)::x(nx),y(nx)
 real(dp),intent(out)::param(nparam)

!Local variables-------------------------------
 integer::ii,jj
! *********************************************************************

 param(:)=1.0d0
!
 do jj=1,nterm
   ii=jj+nterm
   param(ii)=sep**(jj)
!  param(ii)=raux*real(i,dp)
 end do
!
 do jj=1,nterm
   ii=jj+3*nterm
   param(ii)=sep**(jj)
!  param(ii)=raux*real(i,dp)
 end do
end subroutine setparam3
!!***

!!****f* ABINIT/setparam4
!! NAME
!!  setparam4
!!
!! FUNCTION
!!  Sets parameters for LSF
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE
 
subroutine setparam4(nparam,param)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'setparam4'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nparam
 real(dp),intent(out)::param(nparam)
!Local variables-------------------------------

! *********************************************************************
!
 param(:)=1.0d0
!
end subroutine setparam4
!!***

!!****f* ABINIT/setparam5
!! NAME
!!  setparam5
!!
!! FUNCTION
!!  Sets parameters for LSF
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

subroutine setparam5(nterm,nparam,nx,param,rpaw,y)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'setparam5'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nterm,nparam,nx
 real(dp),intent(in)::rpaw
 real(dp),intent(in)::y(nx)
 real(dp),intent(out)::param(nparam)
 !
!Local variables-------------------------------
 integer::ix
 real(dp)::raux,a1,r_c,m
! *********************************************************************

!
 param(:)=1.0d0
!
!alpha1
!a1=maxval( y(:),nx )
!maxval gives problems in some architectures:
 a1=-9999999
 do ix=1,nx
   if(a1<y(ix)) a1=y(ix)
 end do
 param(1:nterm)=a1
!
!alpha2
!
 r_c=rpaw+0.5d0 !paw sphere + a bit more.
!this is not arbitrary since it is an initial guess
 m=0.01d0
 raux=log(a1/m)/r_c**2
 param(nterm+1:nterm*2)=raux
!
end subroutine setparam5
!!***

!!****f* ABINIT/constrains_ini
!! NAME
!!  constrains_ini
!!
!! FUNCTION
!!  Initialise constrains for LSF.
!!  It will constrain the Gaussians width
!!  It will also constraint the Delta use in the LSF algorithm, to jump slowly at each step.
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

subroutine constrains_ini(accuracy,cons1,cons2,limit,nparam,nterm,nx,option,rpaw,y)
 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'constrains_ini'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,intent(in)::nterm,option,nparam,nx
 integer,intent(out)::cons2(nparam)
 real(dp),intent(in)::accuracy,rpaw
 real(dp),intent(in)::y(nx)
 real(dp),intent(out)::cons1(nparam),limit(nparam)

!Local variables-------------------------------
 integer,parameter::restricted=3
 integer :: ix
 real(dp)::rc,a1,mm,raux
! *********************************************************************

!
!DEFAULT: no weight
 cons1(:)=1.d0
 limit(:)=0.d0 
 cons2=1 
!
 if(option==4) then
   cons1(1:nterm)=0.5d0
   cons1(nterm+1:nterm*2)=0.5d0
   if(accuracy<=1e-8) then
     cons1(1:nterm)=0.2d0
     cons1(nterm+1:nterm*2)=0.3d0
   end if
 end if
!
 if(option==4) then
!  parameters

!  a1=maxval( y(:),nx)/real(nterm,dp)
!  maxval gives problems in some architectures:
   a1=-9999999
   do ix=1,nx
     if(a1<y(ix)) a1=y(ix)
   end do
   a1=a1/real(nterm,dp)

   mm=0.01
!  
   rc=1.7d0*rpaw
   raux=log(a1/mm)/rc**2  
!  
   cons2(nterm+1:nterm*2)=restricted
   limit(nterm+1:nterm*2)=raux
 end if
end subroutine constrains_ini
!!***

!!****f* ABINIT/get_parameters
!! NAME
!!  get_parameters
!!
!! FUNCTION
!!  Obtains parameters for LSF
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      fit_gen
!!
!! CHILDREN
!!
!! SOURCE

subroutine get_parameters(const,limit,nparam,param_in,param_out)
 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'get_parameters'
!End of the abilint section

 implicit none

!Arguments -------------------------------
 integer,parameter::positive=2
 integer,parameter::restricted=3
 integer,intent(in)::nparam
 integer,intent(in)::const(nparam)
 real(dp),intent(in)::limit(nparam),param_in(nparam)
 real(dp),intent(out)::param_out(nparam)
 !
!Local variables-------------------------------
 integer::ii
! *********************************************************************

!
 param_out=param_in
 do ii=1,nparam
   if(const(ii)==positive) param_out(ii)=abs(param_in(ii))
!  
   if(const(ii)==restricted) then
     if(param_in(ii)<limit(ii)) then
       param_out(ii)=limit(ii)
     end if
   end if
 end do
 
end subroutine get_parameters
!!***
