!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp7cc
!! NAME
!! psp7cc
!!
!! FUNCTION
!! Compute the pseudo core charge density, for use in the XC core correction,
!! following the function definition valid for the format 7 of psps (PAW).
!!
!! COPYRIGHT
!! Copyright (C) 1998-2013 ABINIT group (FJ,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!!  core_mesh <type(pawrad_type)>=paw radial mesh and related data for core density
!!  n1xccc=dimension of xccc1d ; 0 if no XC core correction is used
!!  rchrg=cut-off radius for the core density
!!  tncore(:)=pseudo core density
!!
!! OUTPUT
!!  xccc1d(n1xccc,6)=1D core charge function and its five first derivatives
!!
!! PARENTS
!!
!! CHILDREN
!!      nderiv_gen,smooth,spline,splint
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"


subroutine psp7cc(core_mesh,n1xccc,rchrg,tncore,xccc1d)

 use m_profiling

 use defs_basis
 use defs_datatypes
 use m_splines

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'psp7cc'
 use interfaces_28_numeric_noabirule
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: n1xccc
 real(dp),intent(in) :: rchrg
 type(pawrad_type),intent(in) :: core_mesh
!arrays
 real(dp),intent(in) :: tncore(core_mesh%mesh_size)
 real(dp),intent(out) :: xccc1d(n1xccc,6)

!Local variables-------------------------------
!scalars
 integer :: i1xccc
 real(dp) :: step1ccc,yp1,ypn
!arrays
 real(dp),allocatable :: der(:,:),der3(:),der4(:),gg1(:),gg2(:),gg3(:),gg4(:)
 real(dp),allocatable :: rad1cc(:),work1(:),work2(:)

!**********************************************************************

!Return if no core correction
 if (n1xccc==0) return

!Generate uniform grid xx1ccc in the box cut by rchrg:
 ABI_ALLOCATE(rad1cc,(n1xccc))
 step1ccc=rchrg/dble(n1xccc-1)
 do i1xccc=1,n1xccc
   rad1cc(i1xccc)=dble(i1xccc-1)*step1ccc
 end do

!Compute first and second derivative of tncore on input grid
 ABI_ALLOCATE(der,(core_mesh%mesh_size,2))
 call nderiv_gen(der,tncore,2,core_mesh)

!Compute third derivative of tncore on input grid
 ABI_ALLOCATE(der3,(core_mesh%mesh_size))
 if (core_mesh%radfact(1)>zero) then
   yp1=1.d0/12.d0/core_mesh%stepint/core_mesh%radfact(1) &
&   *(-25.d0*der(1,1)+48.d0*der(2,1)-36.d0*der(3,1)+16.d0*der(4,1)-3.d0*der(5,1))
 else
   yp1=(der(2,1)-der(1,1))/(core_mesh%rad(2)-core_mesh%rad(1))
 end if
 ypn=1.d0/12.d0/core_mesh%stepint &
& *(  3.d0*der(core_mesh%mesh_size-4,1)-16.d0*der(core_mesh%mesh_size-3,1) &
& +36.d0*der(core_mesh%mesh_size-2,1)-48.d0*der(core_mesh%mesh_size-1,1) &
& +25.d0*der(core_mesh%mesh_size  ,1)) / core_mesh%radfact(core_mesh%mesh_size)
 ABI_ALLOCATE(work1,(core_mesh%mesh_size))
 call spline(core_mesh%rad,der(:,1),core_mesh%mesh_size,yp1,ypn,der3)
 ABI_DEALLOCATE(work1)

!Compute fourth derivative of tncore on input grid
 ABI_ALLOCATE(der4,(core_mesh%mesh_size))
 if (core_mesh%radfact(1)>zero) then
   yp1=1.d0/12.d0/core_mesh%stepint/core_mesh%radfact(1) &
&   *(-25.d0*der(1,2)+48.d0*der(2,2)-36.d0*der(3,2)+16.d0*der(4,2)-3.d0*der(5,2))
 else
   yp1=(der(2,2)-der(1,2))/(core_mesh%rad(2)-core_mesh%rad(1))
 end if
 ypn=1.d0/12.d0/core_mesh%stepint &
& *(  3.d0*der(core_mesh%mesh_size-4,2)-16.d0*der(core_mesh%mesh_size-3,2) &
& +36.d0*der(core_mesh%mesh_size-2,2)-48.d0*der(core_mesh%mesh_size-1,2) &
& +25.d0*der(core_mesh%mesh_size  ,2)) / core_mesh%radfact(core_mesh%mesh_size)
 ABI_ALLOCATE(work1,(core_mesh%mesh_size))
 call spline(core_mesh%rad,der(:,2),core_mesh%mesh_size,yp1,ypn,der4)
 ABI_DEALLOCATE(work1)

!Interpolate tncore on the uniform grid
 if (core_mesh%radfact(1)>zero) then
   yp1=1.d0/12.d0/core_mesh%stepint/core_mesh%radfact(1) &
&   *(-25.d0*tncore(1)+48.d0*tncore(2)-36.d0*tncore(3)+16.d0*tncore(4)-3.d0*tncore(5))
 else
   yp1=(tncore(2)-tncore(1))/(core_mesh%rad(2)-core_mesh%rad(1))
 end if
 ypn=1.d0/12.d0/core_mesh%stepint &
& *(  3.d0*tncore(core_mesh%mesh_size-4)-16.d0*tncore(core_mesh%mesh_size-3) &
& +36.d0*tncore(core_mesh%mesh_size-2)-48.d0*tncore(core_mesh%mesh_size-1) &
& +25.d0*tncore(core_mesh%mesh_size)) / core_mesh%radfact(core_mesh%mesh_size)
 ABI_ALLOCATE(work1,(core_mesh%mesh_size))
 ABI_ALLOCATE(work2,(core_mesh%mesh_size))
 call spline(core_mesh%rad,tncore,core_mesh%mesh_size,yp1,ypn,work1)
 call splint(core_mesh%mesh_size,core_mesh%rad,tncore,work1,n1xccc,rad1cc,xccc1d(:,1))
 ABI_DEALLOCATE(work1)
 ABI_DEALLOCATE(work2)

!Interpolate first and second derivatives of tncore on the uniform grid
 ABI_ALLOCATE(gg1,(n1xccc))
 ABI_ALLOCATE(gg2,(n1xccc))
 call splint(core_mesh%mesh_size,core_mesh%rad,der(:,1),der3,n1xccc,rad1cc,gg1)
 call splint(core_mesh%mesh_size,core_mesh%rad,der(:,2),der4,n1xccc,rad1cc,gg2)
 gg2(1)=gg2(2)+(gg2(2)-gg2(3))
 call smooth(gg2,n1xccc,10) !Smooth to avoid numerical noise

!Compute third derivative of tncore on the uniform grid
 ABI_ALLOCATE(gg3,(n1xccc))
 gg3(n1xccc)=zero
 do i1xccc=1,n1xccc-1
   gg3(i1xccc)=(gg2(i1xccc+1)-gg2(i1xccc))*dble(n1xccc-1)
 end do
 do i1xccc=1,10
   gg3(i1xccc)=gg3(10)/rad1cc(10)*rad1cc(i1xccc)
   gg3(n1xccc-i1xccc)=gg3(n1xccc-10)/(rad1cc(n1xccc-10)-rad1cc(n1xccc))*(rad1cc(n1xccc-i1xccc)-rad1cc(n1xccc))
 end do
 call smooth(gg3,n1xccc,10) !Smooth to avoid numerical noise

!Compute fourth derivative of tncore on the uniform grid
 ABI_ALLOCATE(gg4,(n1xccc))
 gg4(n1xccc)=zero
 do i1xccc=1,n1xccc-1
   gg4(i1xccc)=(gg3(i1xccc+1)-gg3(i1xccc))*dble(n1xccc-1)
 end do
 call smooth(gg4,n1xccc,10) !Smooth to avoid numerical noise

!Write on xcc1d
 xccc1d(:,2)=gg1(:)
 xccc1d(:,3)=gg2(:)
 xccc1d(:,4)=gg3(:)
 xccc1d(:,5)=gg4(:)

!WARNING : fifth derivative not yet computed
 xccc1d(:,6)=zero

 ABI_DEALLOCATE(der)
 ABI_DEALLOCATE(der3)
 ABI_DEALLOCATE(der4)
 ABI_DEALLOCATE(gg1)
 ABI_DEALLOCATE(gg2)
 ABI_DEALLOCATE(gg3)
 ABI_DEALLOCATE(gg4)
 ABI_DEALLOCATE(rad1cc)

end subroutine psp7cc


!!***
