!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp7in
!! NAME
!! psp7in
!!
!! FUNCTION
!! Initialize pspcod=7 ("PAW pseudopotentials"):
!! continue to read the corresponding file and compute the form factors
!!
!! COPYRIGHT
!! Copyright (C) 1998-2013 ABINIT group (GJ, FJ, MT, FB)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  accuracy_p=accuracy wanted for Gaussians fit, only used for WVL+PAW case
!!  icoulomb==0 : usual reciprocal space computation
!!           =1 : free boundary conditions are used
!!  ipsp=id in the array of the currently read pseudo.
!!  ixc=exchange-correlation choice from main routine data file
!!  lloc=angular momentum choice of local pseudopotential
!!  lmax=value of lmax mentioned at the second line of the psp file
!!  psps <type(pseudopotential_type)>=at output, values depending on the read
!!                                    pseudo are set.
!!   | dimekb=dimension of ekb (see module defs_psp.f)
!!   | filpsp=name of formatted external file containing atomic psp data.
!!   | lmnmax=if useylm=1, max number of (l,m,n) comp. over all type of psps
!!   |           =if useylm=0, max number of (l,n)   comp. over all type of psps
!!   | lnmax=max. number of (l,n) components over all type of psps
!!   |           angular momentum of nonlocal pseudopotential
!!   | mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!   | mpssoang= 1+maximum (spin*angular momentum) for nonlocal pseudopotentials
!!   | mqgrid_ff=dimension of q (or G) grid for nl form factors (array ffspl)
!!   | mqgrid_vl=dimension of q (or G) grid for Vloc (array vlspl)
!!   | n1xccc=dimension of xccc1d ; 0 if no XC core correction is used
!!   | optnlxccc=option for nl XC core correction
!!   | positron=0 if electron GS calculation
!!   |              1 if positron GS calculation
!!   |              2 if electron GS calculation in presence of the positron
!!   | pspso=spin-orbit characteristics, govern the content of ffspl and ekb
!!   |          if =0 : this input requires NO spin-orbit characteristics of the psp
!!   |          if =2 : this input requires HGH characteristics of the psp
!!   |          if =3 : this input requires HFN characteristics of the psp
!!   |          if =1 : this input will be changed at output to 1, 2, 3, according
!!   |                  to the intrinsic characteristics of the psp file
!!   | qgrid_ff(mqgrid_ff)=values of q on grid from 0 to qmax (bohr^-1) for nl form factors
!!   | qgrid_vl(mqgrid_vl)=values of q on grid from 0 to qmax (bohr^-1) for Vloc
!!   | usepaw= 0 for non paw calculation; =1 for paw calculation
!!   | usewvl=1 for wavelets calculation; 0 for non wavelets calculation
!!   | useylm=governs the way the nonlocal operator is to be applied:
!!   |            1=using Ylm, 0=using Legendre polynomials
!!   | vlspl_recipSpace=.true. if pseudo are expressed in reciprocal space.
!!   | znuclpsp=atomic number of atom as specified in input file to main routine
!!
!!  pawxcdev=choice of XC development (0=no dev. (use of angular mesh) ; 1 or 2=dev. on moments)
!!  xclevel= XC functional level
!!  zion=nominal valence of atom as specified in psp file
!!
!! OUTPUT
!!  epsatm=$ (4\pi)\int_0^\infty [r^2 (V(r)+\frac{Zv}{r}) dr]$(hartree)
!!  ffspl(psps%mqgrid_ff,2,psps%lnmax)=form factor f_l(q) and second derivative
!!   from spline fit for each angular momentum and each projector;
!!   if any, spin-orbit components begin at l=mpsang+1
!!  indlmn(6,psps%lmnmax)= array giving l,m,n,lm,ln,s for i=lmn
!!  pawrad <type(pawrad_type)>=paw radial mesh and related data
!!  pawtab <type(pawtab_type)>=paw tabulated starting data
!!  vlspl(mqgrid_vl,2)=q^2 Vloc(q) and second derivatives from spline fit
!!  xc_denpos= lowest allowed density (usually for the computation of the XC functionals)
!!  xcccrc=XC core correction cutoff radius (bohr) from psp file
!!
!! NOTES
!!  Spin-orbit not yet implemented (to be done)
!!
!! PARENTS
!!      pspatm_abinit
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine psp7in(accuracy_p,epsatm,ffspl,icoulomb,indlmn,ipsp,ixc,lloc,lmax,mmax,&
&                  pawrad,pawtab,pawxcdev,&
&                  psps,r2well,vlspl,xcccrc,&
&                  xclevel,xc_denpos,zion)

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use m_errors
 use m_radmesh,          only : nderiv_gen
 use m_splines
 use m_profiling

 use m_radmesh,  only : ifromr, compmesh,  copymesh

#if defined HAVE_DFT_LIBXC
  use libxc_functionals
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'psp7in'
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_32_util
 use interfaces_64_atompaw
 use interfaces_65_psp, except_this_one => psp7in
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: icoulomb,ipsp,ixc,lloc,lmax,mmax,pawxcdev
 integer,intent(in) :: xclevel
 real(dp),intent(in) :: accuracy_p,r2well,xc_denpos,zion
 real(dp),intent(out) :: epsatm,xcccrc
 type(pawrad_type),intent(out) :: pawrad
 type(pawtab_type),intent(out) :: pawtab
 type(pseudopotential_type),intent(inout) :: psps
!arrays
 integer,intent(out) :: indlmn(6,psps%lmnmax)
 real(dp),intent(out) :: ffspl(psps%mqgrid_ff,2,psps%lnmax)
 real(dp),intent(out) :: vlspl(psps%mqgrid_vl,2)

!Local variables ------------------------------
!scalars
 integer :: creatorid,ib,icoremesh,ii,il,ilm,ilmn,iln,imainmesh,imsh,iprojmesh
 integer :: ir,iread1,iread2,ishpfmesh,ivalemesh,ivlocmesh,jj,jlmn
 integer :: maxterm,mgauss_param,msz,nmesh
 integer :: pngau_,pspversion,prjopt,ptotgau_,usexcnhat,vlocopt
 real(dp) :: rc,rread1,rread2,rshp0,yp1,ypn
 complex(dp)::arg,fac
 logical :: has_dij0
 character :: blank=' ',numb=' '
 character(len=500) :: message,submessage
 character(len=80) :: pspline
 type(pawrad_type) :: core_mesh,shpf_mesh
 type(pawrad_type) :: tproj_mesh,vale_mesh,vloc_mesh
!arrays
 integer,allocatable :: ngauss_param(:),nprj(:),orbitals(:)
 real(dp),allocatable :: gauss_param(:,:),ncore(:)
 real(dp),allocatable :: shpf(:,:),tncore(:),tnvale(:),tnvale0(:),tproj(:,:),vhnzc(:),vlocr(:)
 real(dp),allocatable :: work1(:),work2(:),work3(:),work4(:)
 complex(dp),allocatable:: ff(:)
 type(pawrad_type),allocatable :: radmesh(:)
! debug
! integer::ierr
! real(dp),allocatable::dfdr(:,:),raux1(:),raux2(:)
! end debug

!************************************************************************

!File format of formatted PAW psp input (the 3 first lines
!have already been read in calling -pspatm- routine) :

!(1) title (character) line
!(2) psps%znuclpsp(ipsp), zion, pspdat
!(3) pspcod, pspxc, lmax, lloc, mmax, r2well
!(4) psp_version, creatorID
!(5) basis_size, lmn_size
!(6) orbitals (for l=1 to basis_size)
!(7) number_of_meshes
!For imsh=1 to number_of_meshes
!(8)  mesh_index, mesh_type ,mesh_size, rad_step[, log_step]
!(9) r_cut(SPH)
!(10) shape_type, r_shape[, shapefunction arguments]
!For iln=1 to basis_size
!(11) comment(character)
!(12) radial mesh index for phi
!(13) phi(r) (for ir=1 to phi_meshsz)
!For iln=1 to basis_size
!(14) comment(character)
!(15) radial mesh index for tphi
!(16) tphi(r) (for ir=1 to phi_mesh_size)
!For iln=1 to basis_size
!(17) comment(character)
!(18) radial mesh index for tproj
!(19) tproj(r) (for ir=1 to proj_mesh_size)
!(20) comment(character)
!(21) radial mesh index for core_density
!(22) core_density (for ir=1 to core_mesh_size)
!(23) comment(character)
!(24) radial mesh index for pseudo_core_density
!(25) tcore_density (for ir=1 to core_mesh_size)
!(26) comment(character)
!(27) Dij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!(28) comment(character)
!(29) Rhoij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!(30) comment(character)
!(31) radial mesh index for Vloc, format of Vloc (0=Vbare, 1=VH(tnzc), 2=VH(tnzc) without nhat in XC)
!(32) Vloc(r) (for ir=1 to vloc_mesh_size)
!===== Following lines only if shape_type=-1 =====
!For il=1 to 2*max(orbitals)+1
!(33) comment(character)
!(34) radial mesh index for shapefunc
!(35) shapefunc(r)*gnorm(l)*r**l (for ir=1 to shape_mesh_size)
!(36) comment(character)
!(37) radial mesh index for pseudo_valence_density
!(38) tvale(r) (for ir=1 to vale_mesh_size)
!
!Comments:
!* psp_version= ID of PAW_psp version
!4 characters string of the form 'pawn' (with n varying)
!* creatorID= ID of psp generator
!creatorid=1xyz : psp generated from Holzwarth AtomPAW generator version x.yz
!creatorid=2xyz : psp generated from Vanderbilt ultra-soft generator version x.yz
!creatorid=-1: psp for tests (for developpers only)
!* mesh_type= type of radial mesh
!mesh_type=1 (regular grid): rad(i)=(i-1)*AA
!mesh_type=2 (logari. grid): rad(i)=AA*(exp[BB*(i-1)]-1)
!mesh_type=3 (logari. grid): rad(i>1)=AA*exp[BB*(i-2)] and rad(1)=0
!mesh_type=4 (logari. grid): rad(i)=-AA*ln[1-BB*(i-1)] with BB=1/n
!* radial shapefunction type
!shape_type=-1 ; gl(r)=numeric (read from psp file)
!shape_type= 1 ; gl(r)=k(r).r^l; k(r)=exp[-(r/sigma)**lambda]
!shape_type= 2 ; gl(r)=k(r).r^l; k(r)=[sin(pi*r/rshp)/(pi*r/rshp)]**2 if r<=rshp
!shape_type= 3 ; gl(r)=Alpha(1,l)*jl(q(1,l)*r)+Alpha(2,l)*jl(q(2,l)*r) for each l


!==========================================================

 DBG_ENTER("COLL")

!==========================================================
!Read psp version in line 4 of the header
 pspversion=1
 read (tmp_unit,'(a80)') pspline;pspline=adjustl(pspline)
 if (pspline(1:3)=="paw".or.pspline(1:3)=="PAW") &
& read(unit=pspline(4:80),fmt=*) pspversion
 if (pspversion<1.or.pspversion>5) then
   write(message, '(a,i2,a,a,a)' )&
&   '  This version of PAW psp file (',pspversion,') is not compatible with',ch10,&
&   '  current version of Abinit.'
   MSG_ERROR(message)
 end if

!==========================================================
!Read lines 4 to 11 of the header

!Have to maintain compatibility with Abinit v4.2.x
 if (pspversion==1) then
   read (unit=pspline,fmt=*) pawtab%basis_size,pawtab%lmn_size
   ABI_ALLOCATE(orbitals,(pawtab%basis_size))
   read(tmp_unit,*) (orbitals(ib), ib=1,pawtab%basis_size)
   pawtab%l_size=2*maxval(orbitals)+1
   nmesh=3
   ABI_DATATYPE_ALLOCATE(radmesh,(nmesh))
   read(tmp_unit,'(a80)') pspline
   radmesh(1)%lstep=zero
   read(unit=pspline,fmt=*,err=10,end=10) radmesh(1)%mesh_type,&
&   radmesh(1)%rstep,radmesh(1)%lstep
   10 read(tmp_unit,*) pawtab%rpaw
   read(tmp_unit,*) radmesh(1)%mesh_size,radmesh(2)%mesh_size,&
&   radmesh(3)%mesh_size
   read(tmp_unit,'(a80)') pspline
   pawtab%shape_lambda=-1;pawtab%shape_sigma=1.d99
   read(unit=pspline,fmt=*,err=11,end=11) pawtab%shape_type,&
&   pawtab%shape_lambda,pawtab%shape_sigma
   11 read(tmp_unit,*) creatorid
   if (pawtab%shape_type==3) pawtab%shape_type=-1
   radmesh(2)%mesh_type=radmesh(1)%mesh_type
   radmesh(3)%mesh_type=radmesh(1)%mesh_type
   radmesh(2)%rstep=radmesh(1)%rstep
   radmesh(3)%rstep=radmesh(1)%rstep
   radmesh(2)%lstep=radmesh(1)%lstep
   radmesh(3)%lstep=radmesh(1)%lstep
 else

!  Here psp file for Abinit 4.3+
   read (unit=pspline(5:80),fmt=*) creatorid
   read (tmp_unit,*) pawtab%basis_size,pawtab%lmn_size
   ABI_ALLOCATE(orbitals,(pawtab%basis_size))
   read(tmp_unit,*) (orbitals(ib), ib=1,pawtab%basis_size)
   pawtab%l_size=2*maxval(orbitals)+1
   read(tmp_unit,*) nmesh
   ABI_DATATYPE_ALLOCATE(radmesh,(nmesh))
   do imsh=1,nmesh
     rread2=zero
     read(tmp_unit,'(a80)') pspline
     read(unit=pspline,fmt=*,err=20,end=20) ii,iread1,iread2,rread1,rread2
     20 continue
     if (ii<=nmesh) then
       radmesh(ii)%mesh_type=iread1
       radmesh(ii)%mesh_size=iread2
       radmesh(ii)%rstep=rread1
       radmesh(ii)%lstep=rread2
     else
       write(message, '(3a)' )&
&       '  Index of mesh out of range !',ch10,&
&       '  Action : check your pseudopotential file.'
       MSG_ERROR(message)
     end if
   end do
   read(tmp_unit,*) pawtab%rpaw
   read(tmp_unit,'(a80)') pspline
   read(unit=pspline,fmt=*) pawtab%shape_type
   pawtab%shape_lambda=-1;pawtab%shape_sigma=1.d99
 end if

!Here reading shapefunction parameters for Abinit 4.3...4.5
 if (pspversion==2) then
   if (pawtab%shape_type==1) read(unit=pspline,fmt=*) ii,pawtab%shape_lambda,pawtab%shape_sigma
   if (pawtab%shape_type==3) pawtab%shape_type=-1
   pawtab%rshp=zero

!  Here reading shapefunction parameters for Abinit 4.6+
 else if (pspversion>=3) then
   pawtab%rshp=zero
   if (pawtab%shape_type==-1) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp
   if (pawtab%shape_type== 1) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp, &
&   pawtab%shape_lambda,pawtab%shape_sigma
   if (pawtab%shape_type== 2) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp
   if (pawtab%shape_type== 3) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp
 end if
 21 continue
 rshp0=pawtab%rshp 
!If shapefunction type is gaussian, check exponent
 if (pawtab%shape_type==1) then
   if (pawtab%shape_lambda<2) then
     write(message, '(3a)' )&
&     '  For a gaussian shape function, exponent lambda must be >1 !',ch10,&
&     '  Action: check your psp file.'
     MSG_ERROR(message)
   end if
 end if
!If shapefunction type is Bessel, deduce here its parameters from rc
 if (pawtab%shape_type==3) then
   rc=pawtab%rshp;if (rc<1.d-8) rc=pawtab%rpaw
   do il=1,pawtab%l_size
     call shapebes(pawtab%shape_alpha(1:2,il),pawtab%shape_q(1:2,il),il-1,rc)
   end do
 end if

!==========================================================
!Mirror pseudopotential parameters to the output and log files

 write(message,'(a,i1)')' Pseudopotential format is: paw',pspversion
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(2(a,i3),a,64i4)') &
& ' basis_size (lnmax)=',pawtab%basis_size,' (lmn_size=',&
& pawtab%lmn_size,'), orbitals=',orbitals(1:pawtab%basis_size)
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(a,f11.8)')' Spheres core radius: rc_sph=',pawtab%rpaw
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(a,i1,a)')' ',nmesh,' radial meshes are used:'
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 do imsh=1,nmesh
   if (radmesh(imsh)%mesh_type==1) &
&   write(message,'(a,i1,a,i4,a,g12.5)') &
&   '  - mesh ',imsh,': r(i)=step*(i-1), size=',radmesh(imsh)%mesh_size,&
&   ' , step=',radmesh(imsh)%rstep
   if (radmesh(imsh)%mesh_type==2) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=AA*[exp(BB*(i-1))-1], size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' BB=',radmesh(imsh)%lstep
   if (radmesh(imsh)%mesh_type==3) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=AA*exp(BB*(i-2)), size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' BB=',radmesh(imsh)%lstep
   if (radmesh(imsh)%mesh_type==4) &
&   write(message,'(a,i1,a,i4,a,g12.5)') &
&   '  - mesh ',imsh,': r(i)=-AA*ln(1-(i-1)/n), n=size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end do
 if (pawtab%shape_type==-1) then
   write(message,'(a)')&
   ' Shapefunction is NUMERIC type: directly read from atomic data file'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==1) then
   write(message,'(2a,a,f6.3,a,i3)')&
&   ' Shapefunction is EXP type: shapef(r)=exp(-(r/sigma)**lambda)',ch10,&
&   '                            with sigma=',pawtab%shape_sigma,' and lambda=',pawtab%shape_lambda
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==2) then
   write(message,'(a)')&
   ' Shapefunction is SIN type: shapef(r)=[sin(pi*r/rshp)/(pi*r/rshp)]**2'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==3) then
   write(message,'(a)')&
&   ' Shapefunction is BESSEL type: shapef(r,l)=aa(1,l)*jl(q(1,l)*r)+aa(2,l)*jl(q(2,l)*r)'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%rshp<1.d-8) then
   write(message,'(a)') ' Radius for shape functions = sphere core radius'
 else
   write(message,'(a,f11.8)') ' Radius for shape functions = ',pawtab%rshp
 end if
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!==========================================================
!Perfom tests

!Spin-orbit not yet implemented
 if (any(psps%pspso(:)>1)) then
   write(message, '(a,a,a)' )&
&   '  Spin-orbit not yet implemented for PAW psps !',ch10,&
&   '  Action : check your pseudopotential or input file.'
   MSG_ERROR(message)
 end if

!Are lmax and orbitals compatibles ?
 if (lmax/=maxval(orbitals)) then
   write(message, '(a,a,a)' )&
&   '  lmax /= MAX(orbitals) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if

!Are psps%lmnmax and pawtab%lmn_size compatibles ?
 if (psps%lmnmax<pawtab%lmn_size) then
   message='  psps%lmnmax < lmn size (read from pseudo) !'
   MSG_BUG(message)
 end if

!Only mesh_type=1,2, 3 or 4 allowed
 do imsh=1,nmesh
   if (radmesh(imsh)%mesh_type>4) then
     write(message, '(a,a,a)' )&
&     '  Only mesh types 1,2, 3 or 4 allowed !',ch10,&
&     '  Action : check your pseudopotential or input file.'
     MSG_ERROR(message)
   end if
 end do

!==========================================================
!Initialize radial meshes

 do imsh=1,nmesh
   ABI_ALLOCATE(radmesh(imsh)%rad ,(radmesh(imsh)%mesh_size))
   ABI_ALLOCATE(radmesh(imsh)%radfact,(radmesh(imsh)%mesh_size))
   ABI_ALLOCATE(radmesh(imsh)%simfact,(radmesh(imsh)%mesh_size))
   call compmesh(radmesh(imsh),-1._dp)
 end do

!==========================================================
!Read tabulated atomic data

!---------------------------------
!Read wave-functions (phi)
 do ib=1,pawtab%basis_size
   read (tmp_unit,*)
   if (pspversion==1) iread1=1
   if (pspversion>1) read (tmp_unit,*) iread1
   if (ib==1) then
     pawrad%mesh_type=radmesh(iread1)%mesh_type
     pawrad%mesh_size=radmesh(iread1)%mesh_size
     pawrad%rstep=radmesh(iread1)%rstep
     pawrad%lstep=radmesh(iread1)%lstep
     call compmesh(pawrad,pawtab%rpaw)
     imainmesh=iread1
   else if (iread1/=imainmesh) then
     write(message, '(a,a,a)' )&
&     '  All Phi and tPhi must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   read (tmp_unit,*) (pawtab%phi(ir,ib),ir=1,pawrad%mesh_size)
 end do

!---------------------------------
!Read pseudo wave-functions (tphi)
 do ib=1,pawtab%basis_size
   read (tmp_unit,*)
   if (pspversion==1) iread1=1
   if (pspversion>1) read (tmp_unit,*) iread1
   if (iread1/=imainmesh) then
     write(message, '(a,a,a)' )&
&     '  All Phi and tPhi must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   read (tmp_unit,*) (pawtab%tphi(ir,ib),ir=1,pawrad%mesh_size)
 end do
 write(message,'(a,i1)') &
& ' Radial grid used for partial waves is grid ',imainmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Read projectors (tproj)
 jj=0
 do ib=1,pawtab%basis_size
   read (tmp_unit,*)
   if (pspversion==1) iread1=2
   if (pspversion>1) read (tmp_unit,*) iread1
   if (ib==1) then
     iprojmesh=iread1
     if(iread1==-1) iread1=2 !for wvl+paw
     call copymesh(radmesh(iread1),tproj_mesh)
     ABI_ALLOCATE(tproj,(tproj_mesh%mesh_size,pawtab%basis_size))
   else if (iread1/=iprojmesh) then
     write(message, '(a,a,a)' )&
&     '  All tprojectors must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
!  
   if(iprojmesh /= -1) then 
!    read projectors from a mesh
     read (tmp_unit,*) (tproj(ir,ib),ir=1,tproj_mesh%mesh_size)
   else
!    read Gaussian coefficients
     read(tmp_unit,*) pngau_, ptotgau_ !total number of gaussians
     if(ib==1) then
       pawtab%wvl%ptotgau=ptotgau_
       if(associated(pawtab%wvl%parg)) then 
         ABI_DEALLOCATE(pawtab%wvl%parg)   
       end if
       if(associated(pawtab%wvl%pfac)) then
         ABI_DEALLOCATE(pawtab%wvl%pfac)
       end if
       if(associated(pawtab%wvl%pngau)) then
         ABI_DEALLOCATE(pawtab%wvl%pngau)  
       end if
       ABI_ALLOCATE(pawtab%wvl%pngau,(pawtab%basis_size))
       ABI_ALLOCATE(pawtab%wvl%parg,(2,pawtab%wvl%ptotgau))
       ABI_ALLOCATE(pawtab%wvl%pfac,(2,pawtab%wvl%ptotgau))
     else
       if(pawtab%wvl%ptotgau .ne. ptotgau_) then
         write(message, '(a,a,a,a,a,a)' ) ch10,&
&         ' psp7in: ERROR -',ch10,&
&         '  total number of gaussians, should be the same for all projectors !',ch10,&
&         '  Action: check your pseudopotential file.'
         call wrtout(ab_out,message,'COLL')
         call wrtout(std_out,  message,'COLL')
         call leave_new('COLL')
       end if
     end if !ib==1
     read(tmp_unit,*)(pawtab%wvl%parg(:,ii),ii=jj+1,jj+pngau_)
     read(tmp_unit,*)(pawtab%wvl%pfac(:,ii),ii=jj+1,jj+pngau_)
     pawtab%wvl%pngau(ib)=pngau_
     jj=jj+pngau_
   end if !iprojmesh==-1
 end do
 write(message,'(a,i2)') &
& ' Radial grid used for projectors is grid ',iprojmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
!
!For wvl+paw:
!1) If projectors are given as sums of gaussians, then
!calculate projectors in the grid.
!2) If projectors are given in a grid, then we fit
!them to gaussians
 if(psps%usewvl==1) then
!  debug
!  do ib=1,pawtab%basis_size
!  ii=600+ib
!  do ir=1,tproj_mesh%mesh_size
!  write(ii,*)pawrad%rad(ir),tproj(ir,ib),pawtab%tphi(ir,ib),tproj(ir,ib)*pawtab%tphi(ir,ib)
!  end do
!  end do
!  end debug

   if(iprojmesh == -1) then
!    1) calculate projectors in the grid
     ABI_ALLOCATE(ff,(tproj_mesh%mesh_size))
     ii=0
     do ib=1,pawtab%basis_size
       ff(:)=czero
!      
       do jj=1,pawtab%wvl%pngau(ib)
         ii=ii+1
         arg=cmplx(pawtab%wvl%parg(1,ii),pawtab%wvl%parg(2,ii))
         fac=cmplx(pawtab%wvl%pfac(1,ii),pawtab%wvl%pfac(2,ii))
         ff(:)=ff(:)+fac*exp(arg*tproj_mesh%rad(:)**2)
       end do !igau
       tproj(:,ib)=real(ff(:))
!      jj=600+ib
!      do ir=1,tproj_mesh%mesh_size
!      write(jj,'(3f16.7)')tproj_mesh%rad(ir),tproj(ir,ib)
!      end do
     end do !ib
     ABI_DEALLOCATE(ff)
   else
!    2) fit projectors to gaussians
     write(message,'(a,a)')ch10,'Fitting tproj to Gaussians'
     call wrtout(std_out,message,'COLL')
!    
     maxterm=200 !default
     mgauss_param=maxterm*4 !option=4,see fit_gen
!    
     ABI_ALLOCATE(gauss_param,(mgauss_param,pawtab%basis_size))
     ABI_ALLOCATE(ngauss_param,(pawtab%basis_size))
!    
!    fit tproj to Gaussians
     call ptildefit(accuracy_p,pawtab%basis_size,maxterm,mgauss_param,ngauss_param,orbitals,&
&     gauss_param,tproj_mesh,pawtab%rpaw,tproj)
!    
!    tproj is now as a sum of sin+cos functions,
!    convert it to a sum of complex gaussians 
!    and fill %wvl object
     call sin2gauss(pawtab%basis_size,mgauss_param,&
&     ngauss_param,gauss_param,pawtab%wvl)
!    
     ABI_DEALLOCATE(gauss_param)
     ABI_DEALLOCATE(ngauss_param)
   end if !iprojmesh .ne. -1
 end if !usewvl==1
!---------------------------------
!Read core density (coredens)
 read (tmp_unit,*)
 if (pspversion==1) iread1=1
 if (pspversion>1) read (tmp_unit,*) iread1
 icoremesh=iread1
 call copymesh(radmesh(iread1),core_mesh)
 if ((radmesh(icoremesh)%mesh_type/=pawrad%mesh_type).or.&
& (radmesh(icoremesh)%rstep    /=pawrad%rstep)    .or.&
& (radmesh(icoremesh)%lstep    /=pawrad%lstep)) then
   write(message, '(a,a,a,a,a)' )&
&   '  Ncore must be given on a radial mesh with the same',ch10,&
&   '  type and step(s) than the main radial mesh (mesh for Phi) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
 ABI_ALLOCATE(ncore,(core_mesh%mesh_size))
 read (tmp_unit,*) (ncore(ir),ir=1,core_mesh%mesh_size)
!construct and save VH[z_NC] if requested
 if (pawtab%has_vhnzc == 1) then
   ABI_ALLOCATE(vhnzc,(core_mesh%mesh_size))
   call pawvhnzc(ncore,core_mesh,vhnzc,psps%znuclpsp(ipsp))
   pawtab%VHnZC(1:pawrad%mesh_size)=vhnzc(1:pawrad%mesh_size)
   pawtab%has_vhnzc=2
   ABI_DEALLOCATE(vhnzc)
 end if
!In wvl+paw save core_mesh_size
 if(psps%usewvl==1 .or. icoulomb .ne. 0) then
   pawtab%core_mesh_size=core_mesh%mesh_size
   if(associated(pawtab%coredens)) then
     ABI_DEALLOCATE(pawtab%coredens)
   end if
   ABI_ALLOCATE(pawtab%coredens ,(pawtab%core_mesh_size))
   if(associated(pawtab%tcoredens)) then
     ABI_DEALLOCATE(pawtab%tcoredens)
   end if
   ABI_ALLOCATE(pawtab%tcoredens,(pawtab%core_mesh_size,6))
 else
   pawtab%core_mesh_size=pawrad%mesh_size
 end if
 pawtab%rcore=core_mesh%rad(pawtab%core_mesh_size)
 pawtab%coredens(1:pawtab%core_mesh_size)=ncore(1:pawtab%core_mesh_size)

!---------------------------------
!Read pseudo core density (tcoredens)
 read (tmp_unit,*)
 if (pspversion==1) iread1=1
 if (pspversion>1) read (tmp_unit,*) iread1
 if (iread1/=icoremesh) then
   write(message, '(a,a,a,a,a,a,a,a)' )&
&   '  Pseudized core density (tNcore) must be given',ch10,&
&   '  on the same radial mesh as core density (Ncore) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
 ABI_ALLOCATE(tncore,(core_mesh%mesh_size))
 read (tmp_unit,*) (tncore(ir),ir=1,core_mesh%mesh_size)
 if (maxval(abs(tncore(:)))<tol6) then
   pawtab%usetcore=0
   pawtab%tcoredens(1:pawtab%core_mesh_size,:)=zero
 else
   pawtab%usetcore=1
   pawtab%tcoredens(1:pawtab%core_mesh_size,1)=tncore(1:pawtab%core_mesh_size)
 end if
 write(message,'(a,i1)') &
& ' Radial grid used for (t)core density is grid ',icoremesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!
!For PAW+WVL calculate up to the 5th derivative
!of tcoredens
 if(psps%usewvl==1 .or. icoulomb .ne. 0) then
   call psp7cc_wvl(core_mesh,pawtab%tcoredens)
 end if

!---------------------------------
!Read frozen part of Dij terms (dij0)
 read (tmp_unit,*)
 read (tmp_unit,*) (pawtab%dij0(ib),ib=1,pawtab%lmn2_size)

!---------------------------------
!Read initial guess of rhoij (rhoij0)
 read (tmp_unit,*)
 read (tmp_unit,*) (pawtab%rhoij0(ib),ib=1,pawtab%lmn2_size)

!---------------------------------
!Read local pseudopotential=Vh(tn_zc) or Vbare
 read (tmp_unit,*)
 if (pspversion==1) ivlocmesh=3
 vlocopt=1
 if (pspversion==2) then
   read (tmp_unit,*) ivlocmesh
 else if (pspversion>2) then
!  read (tmp_unit,fmt=*,err=30,end=30) ivlocmesh,vlocopt
   message=blank
   read (tmp_unit,fmt='(a)') message
   read (message,fmt=*) ivlocmesh
   write(numb,'(i1)')ivlocmesh
   ii=index(message,numb)
   if(len_trim(trim(message(ii+1:)))/=0)then
     submessage=trim(message(ii+1:))
     if(len_trim(submessage)/=0)then
       do ii=1,len_trim(submessage)
         numb=submessage(ii:ii)
         if(numb==blank)cycle
         jj=index('0123456789',numb)
         if(jj<1 .or. jj>10)exit
         vlocopt=jj-1
       end do
     end if
   end if
 end if
 usexcnhat=0;if (vlocopt==1) usexcnhat=1
 call copymesh(radmesh(ivlocmesh),vloc_mesh)
 ABI_ALLOCATE(vlocr,(vloc_mesh%mesh_size))
 read (tmp_unit,*) (vlocr(ir),ir=1,vloc_mesh%mesh_size)
 write(message,'(a,i1)') &
& ' Radial grid used for Vloc is grid ',ivlocmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Eventually read "numeric" shapefunctions (if shape_type=-1)
 if (pawtab%shape_type==-1) then
   do il=1,pawtab%l_size
     read (tmp_unit,*)
     if (pspversion==1) iread1=1
     if (pspversion>1) read (tmp_unit,*) iread1
     if (il==1) then
       call copymesh(radmesh(iread1),shpf_mesh)
       ishpfmesh=iread1
       ABI_ALLOCATE(shpf,(shpf_mesh%mesh_size,pawtab%l_size))
     else if (iread1/=ishpfmesh) then
       write(message, '(a,a,a)' )&
&       '  All shape functions must be given on the same radial mesh !',ch10,&
&       '  Action: check your pseudopotential file.'
       MSG_ERROR(message)
     end if
     read (tmp_unit,*) (shpf(ir,il),ir=1,shpf_mesh%mesh_size)
   end do
   write(message,'(a,i1)') &
&   ' Radial grid used for shape functions is grid ',iread1
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')

!  Has to spline shape functions if mesh is not the "main" mesh
   if (ishpfmesh/=imainmesh) then
     msz=shpf_mesh%mesh_size
     ABI_ALLOCATE(work1,(msz))
     ABI_ALLOCATE(work2,(msz))
     ABI_ALLOCATE(work3,(msz))
     ABI_ALLOCATE(work4,(pawrad%mesh_size))
     work3(:)=shpf_mesh%rad(:)
     work4(:)=pawrad%rad(:)
     do il=1,pawtab%l_size
       call bound_deriv(shpf(1:msz,il),shpf_mesh,msz,yp1,ypn)
       call spline(work3,shpf(:,il),msz,yp1,ypn,work1)
       call splint(msz,work3,shpf(:,il),work1,pawrad%mesh_size,work4,pawtab%shapefunc(:,il))
     end do
     ABI_DEALLOCATE(work1)
     ABI_DEALLOCATE(work2)
     ABI_DEALLOCATE(work3)
     ABI_DEALLOCATE(work4)
   else
     pawtab%shapefunc(:,:)=shpf(:,:)
   end if
   ABI_DEALLOCATE(shpf)
 end if

!---------------------------------
!Read pseudo valence density (if psp version >=4)
 if (pspversion>=4) then
   read (tmp_unit,*)
   read (tmp_unit,*) iread1
   ivalemesh=iread1
   call copymesh(radmesh(iread1),vale_mesh)
   ABI_ALLOCATE(tnvale,(vale_mesh%mesh_size))
   read (tmp_unit,*) (tnvale(ir),ir=1,vale_mesh%mesh_size)
   pawtab%usetvale=1
   write(message,'(a,i1)') &
&   ' Radial grid used for pseudo valence density is grid ',ivalemesh
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
!  save tnvale, if we will rewrite the atomic data file
   if(psps%usewvl==1 .and. iprojmesh /= -1) then 
     ABI_ALLOCATE(tnvale0,(vale_mesh%mesh_size))
     tnvale0=tnvale
   end if
 else
   pawtab%usetvale=0
 end if

!==========================================================
!Initialize various parameters in structured data

!Some dims
 pawtab%l_size=2*maxval(orbitals)+1
 pawtab%mesh_size=pawrad%mesh_size
 pawtab%lmn2_size=pawtab%lmn_size*(pawtab%lmn_size+1)/2

!indlmn calculation (indices for (l,m,n) basis)
 ABI_ALLOCATE(nprj,(0:maxval(orbitals)))
 ilmn=0;iln=0;nprj=0
 do ib=1,pawtab%basis_size
   il=orbitals(ib)
   nprj(il)=nprj(il)+1
   iln=iln+1
   do ilm=1,2*il+1
     indlmn(1,ilmn+ilm)=il
     indlmn(2,ilmn+ilm)=ilm-(il+1)
     indlmn(3,ilmn+ilm)=nprj(il)
     indlmn(4,ilmn+ilm)=il*il+ilm
     indlmn(5,ilmn+ilm)=iln
     indlmn(6,ilmn+ilm)=1
   end do
   ilmn=ilmn+2*il+1
 end do
 ABI_DEALLOCATE(nprj)

!Are ilmn (found here) and pawtab%lmn_size compatibles ?
 if (ilmn/=pawtab%lmn_size) then
   write(message, '(a,a,a,a,a)' )&
&   '  Calculated lmn size differs from',ch10,&
&   '  lmn_size read from pseudo !',ch10,&
&   ' Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if

 has_dij0=.true.
 call psp7calc(core_mesh,epsatm,ffspl,has_dij0,imainmesh,indlmn,ixc,psps%lmnmax,psps%lnmax,&
& mmax,psps%mqgrid_ff,psps%mqgrid_vl,ncore,nmesh,pawrad,pawtab,pawxcdev,pspversion,&
& psps%qgrid_ff,psps%qgrid_vl,&
& radmesh,tncore,tnvale,tproj,tproj_mesh,usexcnhat,vale_mesh,vloc_mesh,vlocopt,&
& vlocr,vlspl,xcccrc,xclevel,xc_denpos,zion,psps%znuclpsp(ipsp))

 if(psps%usewvl==1 .or. icoulomb > 0) then
   call psp7wvl1(pawtab,tnvale,psps%usewvl,vale_mesh,vloc_mesh,vlocr)
 end if

!==================================================
!WVL+PAW
!print again the atomic-data file:
 if(iprojmesh .ne. -1 .and. psps%usewvl==1) call wrpawps_local()


!==========================================================
!Free temporary allocated space
 ABI_DEALLOCATE(tproj)
 do imsh=1,nmesh
   ABI_DEALLOCATE(radmesh(imsh)%rad)
   ABI_DEALLOCATE(radmesh(imsh)%radfact)
   ABI_DEALLOCATE(radmesh(imsh)%simfact)
 end do
 ABI_DEALLOCATE(orbitals)
 ABI_DATATYPE_DEALLOCATE(radmesh)
 ABI_DEALLOCATE(vlocr)
 ABI_DEALLOCATE(ncore)
 ABI_DEALLOCATE(tncore)
 if(pawtab%shape_type==-1) then
   ABI_DEALLOCATE(shpf_mesh%rad)
   ABI_DEALLOCATE(shpf_mesh%radfact)
   ABI_DEALLOCATE(shpf_mesh%simfact)
 end if
 ABI_DEALLOCATE(tproj_mesh%rad)
 ABI_DEALLOCATE(tproj_mesh%radfact)
 ABI_DEALLOCATE(tproj_mesh%simfact)
 ABI_DEALLOCATE(core_mesh%rad)
 ABI_DEALLOCATE(core_mesh%radfact)
 ABI_DEALLOCATE(core_mesh%simfact)
 ABI_DEALLOCATE(vloc_mesh%rad)
 ABI_DEALLOCATE(vloc_mesh%radfact)
 ABI_DEALLOCATE(vloc_mesh%simfact)
 if (pspversion>=4)  then
   ABI_DEALLOCATE(tnvale)
   ABI_DEALLOCATE(vale_mesh%rad)
   ABI_DEALLOCATE(vale_mesh%radfact)
   if (.FALSE.) call wrtout(std_out,"Another silly trick for XLF","COLL")
   ABI_DEALLOCATE(vale_mesh%simfact)
 end if


 DBG_EXIT('COLL')

 contains
!!***

!!****f* psp7in/sin2gauss
!! NAME
!!  sin2gauss
!!
!! FUNCTION
!!  Converts a f(x)=sum_i^N_i a_i sin(b_i x)+ c_i cos( d_i x) to
!!    f(x)=sum_j e_j exp(f_j x), where e and f are complex numbers.
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! basis_size =  size of the lmn basis
!! mparam = number of terms in the summatory (N_i, see the expression above)
!! nparam = Array containing the parameters (a_i, b_i,c_i,d_i)
!! wvl = wavelets data type
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!! On output wvl%pfac and wvl%parg are filled with complex parameters (e_i, f_i)
!!
!! NOTES
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!
!! SOURCE

 subroutine sin2gauss(basis_size,mparam,nparam,&
&param,wvl)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'sin2gauss'
!End of the abilint section

  implicit none
  !
!Arguments ------------------------------------
  integer,intent(in)::mparam,basis_size
  integer,intent(in)::nparam(basis_size)
  real(dp),intent(in)::param(mparam,basis_size)
  type(wvlpaw_type),intent(inout)::wvl

!Local variables ------------------------------
  integer::i,ii,ib,ngauss,nterm
  real(dp)::sep
  real(dp)::a1(mparam),a2(mparam),a3(mparam),a4(mparam),a5(mparam)
  real(dp)::b1r(mparam),b2r(mparam),b1i(mparam),b2i(mparam)
  !
  !extra variables, use to debug
  !
  !integer::igau,nr,unitp
  !real(dp)::step,rmax
  !real(dp),allocatable::r(:), y(:)
  !complex::fac,arg
  !complex(dp),allocatable::f(:)
!************************************************************************

!  
!  
!  Convert from \sum(sin+cos) expressions to sums of complex gaussians
!  (only works for option=4, see fit_gen)
!  
!  get number of coefficients:
   ii=0
   do ib=1,basis_size
     nterm=nparam(ib)/4  !option=4, there are 4 parameters for each term
     ii=ii+nterm*2 !two gaussians for each term
   end do
!  
!  Allocate objects
!  
   ngauss=ii
   wvl%ptotgau=ngauss !total number of complex gaussians
   ABI_ALLOCATE(wvl%pfac,(2,ngauss))
   ABI_ALLOCATE(wvl%parg,(2,ngauss))
   ABI_ALLOCATE(wvl%pngau,(basis_size))
   wvl%pngau(1:basis_size)=nparam(1:basis_size)/2 !option=4
!  
   ii=0
   do ib=1,basis_size
!    
!    Get parameters in sin+cos expansion:
!    Option4: \sum a1 exp(-a2 x^2) ( a3 sin(k x^2) + a4 cos(k x^2))
!    
     nterm=nparam(ib)/4  !option=4
!    
     a1(1:nterm)=param(1:nterm,ib)
     a2(1:nterm)=param(nterm+1:nterm*2,ib)
     a3(1:nterm)=param(nterm*2+1:nterm*3,ib)
     a4(1:nterm)=param(nterm*3+1:nterm*4,ib)
     sep=1.1d0
     do i=1,nterm
       a5(i)=sep**(i)
     end do
!    
!    Now translate them to a sum of complex gaussians: 
!    pngau(ib)=nterm*2  
!    Two gaussians by term:
!    
!    First gaussian
     b1r(1:nterm)= a1(1:nterm)*a4(1:nterm)/2.d0 !coefficient, real
     b1i(1:nterm)=-a1(1:nterm)*a3(1:nterm)/2.d0 !coefficient, imag
     b2r(1:nterm)=-a2(1:nterm)   !exponential, real
     b2i(1:nterm)= a5(1:nterm)   !exponential, imag
!    
     wvl%pfac(1,ii+1:ii+nterm)=b1r(1:nterm)
     wvl%pfac(2,ii+1:ii+nterm)=b1i(1:nterm)
     wvl%parg(1,ii+1:ii+nterm)=b2r(1:nterm) 
     wvl%parg(2,ii+1:ii+nterm)=b2i(1:nterm) 
!    Second gaussian
     wvl%pfac(1,ii+nterm+1:ii+nterm*2)= b1r(1:nterm)
     wvl%pfac(2,ii+nterm+1:ii+nterm*2)=-b1i(1:nterm)
     wvl%parg(1,ii+nterm+1:ii+nterm*2)= b2r(1:nterm) 
     wvl%parg(2,ii+nterm+1:ii+nterm*2)=-b2i(1:nterm) 
!    
     ii=ii+nterm*2
   end do
!  
!  begin debug
!  
!  write(*,*)'sin2gauss, comment me'
!  nr=3000
!  rmax=10.d0
!  ABI_ALLOCATE(r,(nr))
!  ABI_ALLOCATE(f,(nr))
!  ABI_ALLOCATE(y,(nr))
!  step=rmax/real(nr-1,dp)
!  do ir=1,nr
!  r(ir)=real(ir-1,dp)*step
!  end do
!  !
!  ii=0
!  do ib=1,basis_size
!  unitp=500+ib
!  f(:)=czero
!  !
!  do igau=1,wvl%pngau(ib)
!  ii=ii+1
!  arg=cmplx(wvl%parg(1,ii),wvl%parg(2,ii))
!  fac=cmplx(wvl%pfac(1,ii),wvl%pfac(2,ii))
!  f(:)=f(:)+fac*exp(arg*r(:)**2)
!  end do
!  do ir=1,nr
!  write(unitp,'(3f16.7)')r(ir),real(f(ir))!,y(ir)
!  end do
!  end do
!  ABI_DEALLOCATE(r)
!  ABI_DEALLOCATE(f)
!  ABI_DEALLOCATE(y)
!  
!  end debug
!  

 end subroutine sin2gauss
!!***

!!****f* psp7in/wrpawps_local
!! NAME
!!  wrpawps_local
!!
!! FUNCTION
!!  Writes paw pseudopotential file
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!  This is based on ATOMPAW routines by M. Torrent.
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine wrpawps_local()


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'wrpawps_local'
!End of the abilint section

 character(200)   :: fname,title
 integer         :: funit
!
!!------------------------------------------------------------------
!!---- Local variables
!!------------------------------------------------------------------
!
! integer, parameter :: nmesh_max=5
! integer :: coremeshsz,ib,icoremesh,ierr,ii,il,ilmn,iprjmesh,ir
! integer :: ivalemesh,ivlocmesh,iwavmesh,jlmn,maxexp,mparam,mtyp,nmesh
! integer :: nx,option
! integer :: prjmeshsz,valemeshsz,vlocmeshsz,wavmeshsz
! real(dp) :: rstep,lstep,raux
 character*4 :: pspfmt
 character*8 :: strdate
! character(80):: outfile
! integer :: meshsz(nmesh_max),meshtp(nmesh_max)
! integer,allocatable::nparam_array(:)
! real(dp) :: qq,ecoul
! real(dp) :: radstp(nmesh_max),logstp(nmesh_max)
! real(dp), allocatable ::dfdr(:,:),dfdr2(:,:),ff(:),ffit(:),ntmp(:)
! real(dp), allocatable ::rholoc(:),rholoc2(:),rr_log(:),shpf(:),vhtnzc2(:)
! real(dp), allocatable ::param(:,:)
! type(GridInfo) :: Grid_core,Grid_vale

!Local variables-------------------------------
!character(len=500) :: message                   ! to be uncommented, if needed
 
! *************************************************************************
   
!  DEBUG
!  write (std_out,*) ' wrpawps_local : enter'
!  ENDDEBUG


!  ------------------------------------------------------------------
!  ---- Executable code
!  ------------------------------------------------------------------
!  
!  
!  Meshes definitions (if necessary define a logarithmic radial grid)
!  --- Use of an auxilliary log grid
!  if (loggrd%uselog) then
!  loggrd%rad_step=(pawrad%rad(pshead%wav_meshsz)*(one-tol12))*exp(-loggrd%log_step*dble(loggrd%meshsz-2))
!  mtyp=3;rstep=loggrd%rad_step;lstep=loggrd%log_step
!  wavmeshsz=loggrd%meshsz
!  prjmeshsz=pshead%prj_meshsz
!  coremeshsz=int((one+tol12)*log(pshead%rad_step*dble(pshead%core_meshsz-1)/loggrd%rad_step)/loggrd%log_step)+2
!  if (pshead%vale_meshsz>0) then
!  valemeshsz=int((one+tol12)*log(pshead%rad_step*dble(pshead%vale_meshsz-1)/loggrd%rad_step)/loggrd%log_step)+2
!  else
!  valemeshsz=0
!  end if
!  if (pshead%vlocopt==0) then
!  vlocmeshsz=int((one+tol12)*log(pshead%rad_step*dble(pshead%sph_meshsz-1)/loggrd%rad_step)/loggrd%log_step)+2
!  else
!  vlocmeshsz=int((one+tol12)*log(pshead%rad_step*dble(pshead%vloc_meshsz-1)/loggrd%rad_step)/loggrd%log_step)+2
!  endif
!  allocate(rr_log(max(wavmeshsz,coremeshsz,valemeshsz,vlocmeshsz)));rr_log(1)=zero
!  do ir=2,max(wavmeshsz,coremeshsz,valemeshsz,vlocmeshsz)
!  rr_log(ir)=loggrd%rad_step*exp(loggrd%log_step*dble(ir-2))
!  enddo
!  write(std_out,'(3(/,2x,a),2(/,2x,a,g10.4),4(/,2x,a,i4))') &
!  &    'Atompaw2Abinit info:',&
!  &    '  All quantities (except nl projectors) are transfered',&
!  &    '  into a logarithmic grid (r(i)=A*exp[B(i-2)])...',&
!  &    '  Log. grid parameters: rad_step=',loggrd%rad_step,&
!  &    '                        log_step=',loggrd%log_step,&
!  &    '                        Size       =',wavmeshsz,&
!  &    '                        Size (core)=',coremeshsz,&
!  &    '                        Size (vale)=',valemeshsz,&
!  &    '                        Size (Vloc)=',vlocmeshsz
!  else
!  !--- No use of an auxilliary log grid
!  mtyp=pshead%mesh_type;rstep=pshead%rad_step;lstep=pshead%log_step
!  wavmeshsz=min(pshead%sph_meshsz+5,pshead%wav_meshsz)
!  prjmeshsz=pshead%prj_meshsz
!  coremeshsz=pshead%core_meshsz
!  valemeshsz=pshead%vale_meshsz
!  if (pshead%vlocopt==0)then
!  vlocmeshsz=pshead%sph_meshsz
!  else
!  vlocmeshsz=pshead%vloc_meshsz
!  endif
!  endif
!  !--- Build mesh definitions
!  nmesh=1;iwavmesh=1
!  meshtp(1)=mtyp;meshsz(1)=wavmeshsz;radstp(1)=rstep;logstp(1)=lstep
!  if (loggrd%uselog.or.wavmeshsz/=prjmeshsz) then
!  nmesh=nmesh+1;iprjmesh=nmesh;meshsz(nmesh)=pshead%prj_meshsz
!  meshtp(nmesh)=pshead%mesh_type;radstp(nmesh)=pshead%rad_step;logstp(nmesh)=pshead%log_step
!  else
!  iprjmesh=iwavmesh
!  endif
!  if (wavmeshsz/=coremeshsz) then
!  if (prjmeshsz/=coremeshsz) then
!  nmesh=nmesh+1;icoremesh=nmesh;meshsz(nmesh)=coremeshsz
!  meshtp(nmesh)=mtyp;radstp(nmesh)=rstep;logstp(nmesh)=lstep
!  else
!  icoremesh=iprjmesh
!  endif
!  else
!  icoremesh=iwavmesh
!  endif
!  if (wavmeshsz/=vlocmeshsz) then
!  if(prjmeshsz/=vlocmeshsz) then
!  if(coremeshsz/=vlocmeshsz) then
!  nmesh=nmesh+1;ivlocmesh=nmesh;meshsz(nmesh)=vlocmeshsz
!  meshtp(nmesh)=mtyp;radstp(nmesh)=rstep;logstp(nmesh)=lstep
!  else
!  ivlocmesh=icoremesh
!  endif
!  else
!  ivlocmesh=iprjmesh
!  endif
!  else
!  ivlocmesh=iwavmesh
!  endif
!  if (wavmeshsz/=valemeshsz) then
!  if(prjmeshsz/=valemeshsz) then
!  if(coremeshsz/=valemeshsz) then
!  if(vlocmeshsz/=valemeshsz) then
!  nmesh=nmesh+1;ivalemesh=nmesh;meshsz(nmesh)=valemeshsz
!  meshtp(nmesh)=mtyp;radstp(nmesh)=rstep;logstp(nmesh)=lstep
!  else
!  ivalemesh=ivlocmesh
!  endif
!  else
!  ivalemesh=icoremesh
!  endif
!  else
!  ivalemesh=iprjmesh
!  endif
!  else
!  ivalemesh=iwavmesh
!  endif
!  
!  !Some translations AtomPAW -> ABINIT
!  call NullifyGrid(Grid_core);call NullifyGrid(Grid_vale)
!  if (pshead%core_meshsz>0) call grid2pawrad(Grid_core,pawrad,pshead%core_meshsz,-1)
!  if (pshead%vale_meshsz>0) call grid2pawrad(Grid_vale,pawrad,pshead%vale_meshsz,-1)
!  
!  Open the file for writing
   write(fname,'(2a)')trim(psps%filpsp(ipsp)),".wvl"
   open(unit=funit,file=fname,form='formatted',status='unknown')

!  Write the header
   call date_and_time(strdate)
   title='PAW+WVL atomic data - Generated by ABINIT'
   write(funit,'(a)') trim(title)
   write(funit,'(1x,f7.3,1x,f7.3,1x,a,14x,a)') &
&   psps%znuclpsp(ipsp),psps%ziontypat(ipsp),& !,psps%zionpsp(ipsp),&
&   trim(strdate),&
&   " : zatom,zion,pspdat"

!  third line 
   write (funit,'(1x,i2,1x,i7,1x,i2,1x,i2,1x,i5,x,f4.2,14x,a)') &
&   psps%pspcod(ipsp),psps%pspxc(ipsp),lmax,lloc,mmax,r2well,&
&   " : pspcod,pspxc,lmax,lloc,mmax,r2well"
!  
!  fourth line
   creatorid=1331
   write(pspfmt,'("paw",i1)')pspversion
   write (funit,'(1x,a4,1x,i4,29x,a)') &
&   pspfmt,creatorid,&
&   " : pspfmt,creatorID"
   write (funit,'(2(1x,i2),33x,a)') &
&   pawtab%basis_size,&
&   pawtab%lmn_size,&
&   " : basis_size,lmn_size"
   do ib=1,pawtab%basis_size
     write (funit,'(1x,i1)',ADVANCE='NO') orbitals(ib)
   end do
   if (pawtab%basis_size<20) then
     do ib=pawtab%basis_size+1,20
       write (funit,'(a)',ADVANCE='NO') '  '
     end do
   end if
   write (funit,'(a)') ": orbitals"
   write (funit,'(1x,i1,37x,a)') &
&   nmesh," : number_of_meshes"
   do ii=1,nmesh
     if (radmesh(ii)%mesh_type==1) then
       write (funit,'(1x,i1,1x,i1,1x,i4,1x,es22.16,15x,a,i1,a)') &
&       ii,radmesh(ii)%mesh_type,radmesh(ii)%mesh_size,radmesh(ii)%rstep,&
&       " : mesh ",ii,", type,size,rad_step[,log_step]"
     else
       write (funit,'(1x,i1,1x,i1,1x,i4,1x,es22.16,1x,es22.16,a,i1,a)') &
&       ii,radmesh(ii)%mesh_type,radmesh(ii)%mesh_size,radmesh(ii)%rstep,radmesh(ii)%lstep,&
&       " : mesh ",ii,", type,size,rad_step[,log_step]"
     end if
   end do
   write (funit,'(1x,f13.10,25x,a)') &
&   pawtab%rpaw,&
&   " : r_cut(PAW)"
!  Here writing shapefunction parameters for Abinit 4.3...4.5
   if(pspversion==2) then
     if (pawtab%shape_type==1) then
       write (funit,'(1x,i1,1x," 0.",i2,1x,es20.12,10x,a)') &
&       pawtab%shape_type,pawtab%shape_lambda,pawtab%shape_sigma,&
&       " : shape_type,rshape,lambda,sigma"
     end if
!    Here writing shapefunction parameters for Abinit 4.6+
   else if (pspversion>=3) then
     if(pawtab%shape_type==1) then
       write (funit,'(1x,i1,1x,f13.10,x,i5,x,f13.10,x,a)') &
&       pawtab%shape_type,rshp0,pawtab%shape_lambda,pawtab%shape_sigma,&
&       " : shape_type,rshape,lambda,sigma"
     else
       write (funit,'(1x,i1,1x,f13.10,20x,a)') &
&       pawtab%shape_type,rshp0,&
&       " : shape_type,rshape"

     end if
   end if

!  Write basis functions
   do ib=1,pawtab%basis_size
     write(funit,'(a,i1,a)') "===== PHI ",ib,&
&     " =====   [phi(r)=PHI(r)/r*Ylm(th,ph)]"
     write(funit,'(i2,a)') imainmesh,"  : radial mesh index"
     write(funit,'(3(1x,es23.16))') (pawtab%phi(ir,ib),ir=1,pawrad%mesh_size)
   end do

!  Write pseudo bais functions
   do ib=1,pawtab%basis_size
     write(funit,'(a,i1,a)') "===== TPHI ",ib,&
&     " =====   [tphi(r)=TPHI(r)/r*Ylm(th,ph)]"
     write(funit,'(i2,a)') imainmesh,"  : radial mesh index"
     write(funit,'(3(1x,es23.16))') (pawtab%tphi(ir,ib),ir=1,pawrad%mesh_size)
   end do

!  Write projectors
   prjopt=2 !print projectors in a Gaussian mesh
   if(prjopt==1) then !in a mesh
!    do ib=1,pawtab%basis_size
!    write(funit,'(a,i1,a)') "===== TPROJECTOR ",ib,&
!    &       " =====   [tp(r)=TPROJECTOR(r)/r*Ylm(th,ph)]"
!    write(funit,'(i2,a)') iprjmesh,"  : radial mesh index"
!    write(funit,'(3(1x,es23.16))') (tproj(ir,ib),ir=1,pshead%prj_meshsz)
!    enddo
   else !Gaussians format
     jj=0
     do ib=1,pawtab%basis_size
       write(funit,'(a,i1,a)') "===== TPROJECTOR ",ib,&
&       " =====   [tp(r)=TPROJECTOR(r)/r*Ylm(th,ph)]"
       write(funit,'(a,a)') "-1","  : sum of complex gaussians"
       write(funit,'(i5,x,i5,x,a)')pawtab%wvl%pngau(ib),pawtab%wvl%ptotgau, ":ngauss, total ngauss"
       write(funit,'(3(1x,es23.16))')(pawtab%wvl%parg(:,ii),&
&       ii=jj+1,jj+pawtab%wvl%pngau(ib))
       write(funit,'(3(1x,es23.16))')(pawtab%wvl%pfac(:,ii),&
&       ii=jj+1,jj+pawtab%wvl%pngau(ib))
       jj=jj+pawtab%wvl%pngau(ib)
     end do
   end if

!  Write core density
   write(funit,'(a)') "===== CORE_DENSITY ====="
   write(funit,'(i2,a)') icoremesh,"  : radial mesh index"
   write(funit,'(3(1x,es23.16))') (ncore(ir),ir=1,core_mesh%mesh_size)

!  Write pseudo core density
   write(funit,'(a)') "===== PSEUDO_CORE_DENSITY ====="
   write(funit,'(i2,a)') icoremesh,"  : radial mesh index"
   write(funit,'(3(1x,es23.16))') (tncore(ir),ir=1,core_mesh%mesh_size)

!  Write Dij0 and Rhoij0
   write(funit,'(a)') "===== Dij0 ====="
   ii=0
   do jlmn=1,pawtab%lmn_size
     write(funit,'(100(1x,es23.16))') (pawtab%dij0(ii+ilmn),ilmn=1,jlmn)
     ii=ii+jlmn
   end do
   write(funit,'(a)') "===== Rhoij0 ====="
   ii=0
   do jlmn=1,pawtab%lmn_size
     write(funit,'(100(1x,es23.16))') (pawtab%rhoij0(ii+ilmn),ilmn=1,jlmn)
     ii=ii+jlmn
   end do
!  
   if (vlocopt==0) write(funit,'(a)') "===== Vbare (Vloc(r)) ====="
   if (vlocopt==1) write(funit,'(a)') "===== VHntZC (Vloc(r)) with compensation charge in XC ====="
   if (vlocopt==2) write(funit,'(a)') "===== VHntZC (Vloc(r)) without compensation charge in XC ====="
!  
   write(funit,'(i2,1x,i2,a)') ivlocmesh,vlocopt, &
&   "  : radial mesh index, Vloc format (0=Vbare, 1=VH(tnzc) with hat in XC, 2=VH(tnzc) w/o hat in XC)"
   write(funit,'(3(1x,es23.16))')(vlocr(ir),ir=1,vloc_mesh%mesh_size) 

!  Write (eventually) shape functions
   if (pawtab%shape_type==-1) then
     do il=1,pawtab%l_size
       write(funit,'(a,i1,a)') "===== SHAPEF (l=",il-1,") ====="
       write(funit,'(a)') " 1  : radial mesh index"
       write(funit,'(3(1x,es23.16))') (shpf(ir,il),ir=1,shpf_mesh%mesh_size)
     end do
   end if

!  Write pseudo valence density
   if (pawtab%usetvale==1) then
     write(funit,'(a)') "===== PSEUDO_VALENCE_DENSITY ====="
     write(funit,'(i2,a)') ivalemesh,"  : radial mesh index"
     write(funit,'(3(1x,es23.16))') (tnvale0(ir),ir=1,vale_mesh%mesh_size)
   end if

!  Close the file and end
   close(funit)

!  DEBUG
!  write (std_out,*) ' wrpawps_local : exit'
!  stop
!  ENDDEBUG

 end subroutine wrpawps_local

end subroutine psp7in
!!***

!!****f* ABINIT/ptildefit
!! NAME
!! ptildefit
!!
!! FUNCTION
!! Fit tproj to Gaussians
!!
!! COPYRIGHT
!! Copyright (C) 2011-2013 ABINIT group (TRangel)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  accuracy_p=accuracy wanted for Gaussians fit.
!!    Defined as: accuracy_p= sum_x abs(f(x)-y(x))/nx. 
!!    nx is the number of points, f(x) and y(x) are the fitted and original functions.
!!  basis_size= size of the PAW basis
!!  orbitals= indicates the l quantum number for all orbitals.
!!  rpaw= PAW radius
!!  pawrad <type(pawrad_type)>=paw radial mesh and related data
!!  tproj= projectors
!!  maxterm= maximum number of terms used to fit the projectors.
!!  mparam= maximum number of parameters (Gaussian coefficients and factors) used.
!!
!! OUTPUT
!!  nparam_array= number of parameters found.
!!  param = parameters found  (Gaussian coefficients and factors).
!!
!! NOTES
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine ptildefit(accuracy_p,basis_size,maxterm,mparam,nparam_array,orbitals,param,pawrad,&
& rpaw,tproj)

 use m_profiling
 use defs_basis
 use defs_datatypes
 use m_errors
 use m_radmesh,  only : compmesh,copymesh

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'ptildefit'
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_64_atompaw
 use interfaces_65_psp, except_this_one => ptildefit
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 real(dp),intent(in)::accuracy_p
 integer,intent(in)::basis_size
 integer,intent(in)::orbitals(basis_size)
 real(dp),intent(in)::rpaw
 type(pawrad_type),intent(in) :: pawrad
 real(dp),intent(in)    :: tproj(pawrad%mesh_size,basis_size)
 integer,intent(in)::maxterm,mparam
 !
 integer,intent(out)::nparam_array(basis_size)
 real(dp),intent(out)::param(mparam,basis_size)
 type(pawrad_type)::mesh_tmp

!Local variables ------------------------------
 integer :: ibasis,il
 integer :: msz
 integer :: nr
 integer :: option
 real(dp),allocatable::tproj_tmp(:)
 character(len=500) :: message
 character(80)::outfile
 !debug: uncomment
 !integer::ii,ir,nterm ,unitp
 !real(dp),allocatable::y(:)
 !end debug
!************************************************************************

 option=4  !see fit_gen
!
 nparam_array(:)=0
 msz=pawrad%mesh_size
!
!augment the mesh size
!this is to make the Gaussians go to zero after paw_radius
!This is done by creating a new pawrad objet: mesh_tmp
 if(pawrad%mesh_type==1) nr=msz*2 !lineargrid
 if(pawrad%mesh_type/=1) nr=int(msz*1.2) !log grid
!
!write(*,*)'Fit done from 0 to',r_fit
 call compute_mesh(pawrad%lstep,mesh_tmp,nr,pawrad%mesh_type,pawrad%rstep)
!
 ABI_ALLOCATE(tproj_tmp,(nr))
!fill with zeros

 do ibasis=1,basis_size
!  
   write(message,'(a," - Fitting wfn ",i4," to Gaussians")')ch10,ibasis
   call wrtout(std_out,  message,'COLL')
!  
   tproj_tmp(:)=0.d0
!  take out r^il factor:
!  il=psps%indlmn(1,ilmn,itypat)
   il=orbitals(ibasis)
   tproj_tmp(1:msz)=tproj(1:msz,ibasis)/((pawrad%rad(1:msz)+tol8)**(il))
!  take out 1/r factor from eq.(3) of M. Torrent CMS 42, 337 (2008)
!  since: <phi|proj>=1 from atompaw, and phi=phi*r, alors proj=proj/r
   tproj_tmp(2:msz)=tproj_tmp(2:msz)/(pawrad%rad(2:msz)) 
!  
!  debug
!  write(*,*)'psp7in: erase me: l2325 debug: write fort.600'
!  i=600+ibasis
!  do ii=1,msz   !nr
!  write(i,*)mesh_tmp%rad(ii),tproj_tmp(ii)
!  write(i,*)pawrad%rad(ii),tproj_tmp(ii),tproj(ii,ibasis),pawtab%tphi
!  end do
!  
!  Obtain the name for the output file
   if(ibasis<10) then
     write(outfile,'("wfn",i1,".fit")')ibasis
   elseif(ibasis<100) then
     write(outfile,'("wfn",i2,".fit")')ibasis
     write(message,'(a,a,a,a)')ch10,&
&     "ptildefit : bug, ib (basis index) is too big!",ch10,&
&     "Action: check your pseudopotentials"
     call wrtout(ab_out,message,'COLL')
     call wrtout(std_out,  message,'COLL')
     call leave_new('COLL')
   end if
!  
   call fit_gen(accuracy_p,maxterm,mparam,nparam_array(ibasis),nr,&
&   param(:,ibasis),mesh_tmp,option,outfile,rpaw,tproj_tmp)

!  check
!  ABI_ALLOCATE(y,(mesh_tmp%mesh_size))
!  nterm=nparam_array(ibasis)/4
!  call calcgaussc4(nparam_array(ibasis),nterm,mesh_tmp%mesh_size,1,param(:,ibasis),&
!  &  mesh_tmp%rad,y)
!  ! unitp=600+ibasis
!  ! do ir=1,mesh_tmp%mesh_size
!  !  write(unitp,'(2(f16.7,x,f16.7))')mesh_tmp%rad(ir),y(ir)
!  ! end do
!  ABI_DEALLOCATE(y) 
!  
 end do
!
!Deallocate
!
 ABI_DEALLOCATE(mesh_tmp%rad)
 ABI_DEALLOCATE(mesh_tmp%radfact)
 ABI_DEALLOCATE(mesh_tmp%simfact)
 ABI_DEALLOCATE(tproj_tmp)

end subroutine ptildefit
!!***

!!****f* ABINIT/compute_mesh
!! NAME
!!  compute_mesh
!!
!! FUNCTION
!!  Wrap to call compmesh
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! mesh= contains mesh information
!! mesh_size = number of points in the mesh
!! mesh_type = mesh type (see more on pawrad definition)
!! rstep = r_step as defined in pawrad
!! lstep = l_step as defind in pawrad
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!! At output the object mesh is computed
!!
!! NOTES
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!
!! SOURCE

subroutine compute_mesh(lstep,mesh,mesh_size,mesh_type,rstep)

 use m_profiling
 use defs_basis
 use defs_datatypes
 use m_errors
 use m_radmesh,  only : compmesh,copymesh

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'compute_mesh'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 !see pawrad_type for info
 integer,intent(in) :: mesh_size
 integer,intent(in) :: mesh_type 
 real(dp),intent(in):: lstep
 real(dp),intent(in):: rstep
 type(pawrad_type),intent(inout)::mesh

!************************************************************************

 ABI_ALLOCATE(mesh%rad,(mesh_size))
 ABI_ALLOCATE(mesh%radfact,(mesh_size))
 ABI_ALLOCATE(mesh%simfact,(mesh_size))
!
 mesh%mesh_size=mesh_size
 mesh%mesh_type=mesh_type
 mesh%rstep=rstep
 mesh%lstep=lstep
 call compmesh(mesh,-1._dp)

end subroutine compute_mesh
!!***

!!****f* ABINIT/smooth_function
!! NAME
!!  smooth_function
!!
!! FUNCTION
!!  Smooths a general function f(r).
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! func= funcion f(r)
!! grid= r values
!! nr = number of points
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!! This function is very simple, it only does a linear interpolation.
!! This should not be used, this is only for testing.
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
subroutine smooth_function(func,grid,nr)

use m_profiling
use defs_basis
use defs_datatypes
use m_errors

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'smooth_function'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in)::nr
 real(dp),intent(inout)::func(nr)
 real(dp),intent(in)::grid(nr)
 !

!Local variables ------------------------------
 integer::ir,ir1,ir2,jr
 real(dp)::fact,tol
!************************************************************************

!
 tol=0.1d0
 ir1=0;ir2=0
 do ir=2,nr
   if(abs(func(ir)-func(ir-1))>tol) then   
     if (ir1==0) ir1=ir-1
     ir2=ir+1
   else if(ir2>0) then
!    if first case, then set first elements equal to first good value.
     if(ir1==1) then
       func(1:ir2-1)=func(ir2)
     else
!      do a linear interpolation
       fact=(func(ir2)-func(ir1))/(grid(ir2)-grid(ir1))
       do jr=ir1+1,ir2-1
         func(jr)=func(ir1)+fact*(grid(jr)-grid(ir1))
       end do
     end if
     ir1=0;ir2=0
   end if
 end do
end subroutine smooth_function
!!***
