!{\src2tex{textfont=tt}}
!!****f* ABINIT/expibi
!! NAME
!! expibi
!!
!! FUNCTION
!! Routine that computes exp(-i (b2-b1).R) at each site.
!!
!! COPYRIGHT
!! Copyright (C) 2005-2013 ABINIT group
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  dkvecs(3,3) :: $\Delta k$ increments in three directions for current k-point grid
!!  field_type :: 3 for electric field, 9 for magnetic field
!!  gprimd(3,3) :: dimensioned primitive translations of reciprocal lattice
!!  mpi_atmtab(:)=--optional-- indexes of the atoms treated by current proc
!!  mpi_comm_atom=--optional-- MPI communicator over atoms
!!  my_natom :: controls atom loop in parallelism
!!  natom :: number of atoms in unit cell
!!  rprimd(3,3) :: dimensioned primitive translations of real space lattice
!!  xred(natom,3) :: reduced coordinates of atoms in unit cell
!!
!! OUTPUT
!!  calc_expibi(2,natom,field_type) :: phase factors at each atom for different vector shifts
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      initberry,initorbmag
!!
!! CHILDREN
!!      free_my_atmtab,get_my_atmtab
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

 subroutine expibi(calc_expibi,dkvecs,field_type,gprimd,my_natom,natom,rprimd,xred, &
&                  mpi_atmtab,mpi_comm_atom) ! optional arguments (parallelism)

 use m_profiling
 use m_errors
 use defs_basis
 use m_paral_atom

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'expibi'
!End of the abilint section

 implicit none

!Arguments---------------------------
!scalars
 integer,intent(in) :: field_type,my_natom,natom
 integer,optional,intent(in) :: mpi_comm_atom
 real(dp),intent(out) :: calc_expibi(2,my_natom,field_type)
!arrays
 integer,optional,target,intent(in) :: mpi_atmtab(:)
 real(dp),intent(in) :: dkvecs(3,3),gprimd(3,3),rprimd(3,3),xred(3,natom)

!Local variables---------------------------
!scalars
 integer :: bdir,bdir_ctr,bsig,bsigstart,bsigend,calc_indx
 integer :: iatom,iatom_tot,kdir,mu
 logical :: my_atmtab_allocated,paral_atom
 character(len=500) :: message
 real(dp) :: bdotr
!arrays
 integer,pointer :: my_atmtab(:)
 real(dp) :: bb(3),bcart(3),xcart(3)

! *************************************************************************

!Compatibility tests
 if ( (field_type /= 3) .and. (field_type /= 9) ) then
   message = ' expibi called with invalid field_type '
   MSG_ERROR(message)
 end if

 calc_expibi(:,:,:) = zero

!calc_expibi(2,my_natom,field_type)
!used for PAW field calculations (distributed over atomic sites)
!stores the on-site phase factors arising from
!$\langle\phi_{i,k}|\phi_{j,k+\sigma_k k_k}\rangle$
!where $\sigma = \pm 1$. These overlaps arise in various Berry
!phase calculations of electric and magnetic polarization. The on-site
!phase factor is $\exp[-i\sigma_k k_k)\cdot I]$ where
!$I$ is the nuclear position. Only the following
!are computed and saved, in the given order:
!
!if field_type = 3 (electric field):
!1)    -k_1
!2)    -k_2
!3)    -k_3
!
!if field_type = 9 (magnetic field):
!1) -k_3 - k_1
!2)    0 - k_1
!3) +k_3 - k_1
!4) -k_1 - k_2
!5)    0 - k_2
!6) +k_1 - k_2
!7) -k_2 - k_3
!8)    0 - k_3
!9) +k_2 - k_3

!Set up parallelism over atoms
 paral_atom=(present(mpi_comm_atom).and.my_natom/=natom)
 nullify(my_atmtab);if (present(mpi_atmtab)) my_atmtab => mpi_atmtab
 call get_my_atmtab(mpi_comm_atom,my_atmtab,my_atmtab_allocated,paral_atom,natom,my_natom_ref=my_natom)

 bsigstart = 0; bsigend = 0
 if (field_type == 9) then
   bsigstart = -1; bsigend = +1
 end if

 do iatom = 1, my_natom
   iatom_tot=iatom;if (paral_atom) iatom_tot=my_atmtab(iatom)

   calc_indx = 0
   do bdir_ctr = 1, 3
     bdir = mod(bdir_ctr+1,3)+1
     do bsig = bsigstart, bsigend

       kdir = bdir_ctr
       calc_indx = calc_indx + 1
       bb(:) = bsig*dkvecs(:,bdir)-dkvecs(:,kdir)
!      get cartesian positions of atom in cell
       do mu=1,3
         bcart(mu)=dot_product(bb(:),gprimd(mu,:))
         xcart(mu)=dot_product(rprimd(mu,:),xred(:,iatom_tot))
       end do
       bdotr = dot_product(xcart,bcart)
!      here is exp(i (-k).R) for the given site
       calc_expibi(1,iatom,calc_indx) = cos(two_pi*bdotr)
       calc_expibi(2,iatom,calc_indx) = sin(two_pi*bdotr)
     end do ! end loop over bsig
   end do ! end loop over bdir_ctr

 end do ! end loop on my_natom

!Destroy atom table used for parallelism
 call free_my_atmtab(my_atmtab,my_atmtab_allocated)

 end subroutine expibi
!!***
